#**************************************************************************
# DSemu - The Next Generation                                             *
# Unix gcc Makefile [Makefile.nix]                                        *
# Copyright Imran Nazar, 2005; released under the BSD Public Licence.     *
#**************************************************************************

#---Definitions------------------------------------------------------------
CC = g++ -O2 -Wall -pedantic -ansi -fPIC -c -ggdb3
LD = g++

FLTK_CFLAGS = `fltk-config --cflags`
FLTK_LDFLAGS = `fltk-config --ldstaticflags`
FLTK_VER = `fltk-config --version`

TARGETS = dsemu \
uifltk.so \
null.so \
nullgpu.so \
nullapu.so \
arm7tdmi.so \
gbammu.so \
gbagpu.so \
gbaapu.so \
gbatimer.so \
arm9es.so \
dsmmumain.so \
dsmmusub.so \
dsgpu.so

# You can define the following things in DEFS:
# -DARM7TDMI_DEBUG: Make the ARM7 dump registers after each instruction
# -DARM9ES_DEBUG: Make the ARM9 dump registers after each instruction
DEFS = -DARM7TDMI_X86

all: sysdep fltk-check fltk-post $(TARGETS)

#---Executables------------------------------------------------------------

dsemu: dsemu.o err.o plugin.o
	$(LD) -o $@ $^ -ldl

%.so: %.o
	$(LD) -shared -o $@ $^ $(LINKLIBS)

uifltk.so: uifltk.o uifltk-ui.o uifltk-log.o uifltk-bkpt.o uifltk-sub.o \
           uifltk-menu.o uifltk-about.o err.o
arm7tdmi.so: arm7tdmi.o arm7tdmi-olut.o arm7tdmi-opcodes.o arm-dp.7.o arm-ls.7.o \
	gbaswi.o thumb-opcodes.7.o thumb7-olut.o thumb7-opcodes.o armdasm.o err.o font5x7.o \
	arm-dp-x86.7.o arm-ls-x86.7.o thumb7-opcodes-x86.o thumb-opcodes-x86.7.o \
	arm7tdmi-opcodes-x86.o arm7tdmi-olut-x86.o thumb7-olut-x86.o gbaswi-x86.o
arm9es.so: arm9es.o arm9es-olut.o arm9es-opcodes.o arm-dp.9.o arm-ls.9.o dsswi.o \
	thumb-opcodes.9.o thumb9-olut.o thumb9-opcodes.o armdasm.o arm-cpnull.o arm-cpsys.o err.o font5x7.o
gbammu.so: gbammu.o gbammu-dma.o font5x7.o err.o
dsmmumain.so: dsmmumain.o dsmmumain-dma.o ndshead.o font5x7.o err.o
dsmmusub.so: dsmmusub.o dsmmusub-dma.o ndshead.o font5x7.o err.o
gbagpu.so: gbagpu.o font5x7.o err.o
dsgpu.so: dsgpu.o font5x7.o err.o
gbaapu.so: gbaapu.o err.o pa_lib.o pa_trace.o pa_unix.o pa_unix_oss.o
gbatimer.so: gbatimer.o err.o
nullgpu.so: nullgpu.o err.o
nullapu.so: nullapu.o err.o
null.so: null.o err.o

uifltk.so: LINKLIBS=$(FLTK_LDFLAGS)
gbaapu.so: LINKLIBS=-lm -lpthread

#---Sources----------------------------------------------------------------

%.o: %.cpp
	$(CC) $(DEFS) -o $@ $<

dsemu.o: plugin.h plggui.h
err.o: err.h
plugin.o: plugin.h config.h
armdasm.o: armdasm.h
gbammu.o: gbammu.h plgmmu.h plgmmu32.h config.h
gbammu-dma.o: gbammu.h
gbatimer.o: gbatimer.h
gbagpu.o: gbagpu.h config.h
dsgpu.o: dsgpu.h
nullgpu.o: nullgpu.h
gbaapu.o: gbaapu.h
nullapu.o: nullapu.h
font5x7.o: font5x7.h fontdata.h
null.o: null.h
arm7tdmi.o arm9es.o: plgcpu.h
%-x86.o: x86emit.h

arm-dp.7.o: arm-dp.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm7tdmi.h\" $<

arm-ls.7.o: arm-ls.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm7tdmi.h\" $<

arm-dp.9.o: arm-dp.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm9es.h\" $<

arm-ls.9.o: arm-ls.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm9es.h\" $<

thumb-opcodes.7.o: thumb-opcodes.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm7tdmi.h\" $<

thumb-opcodes.9.o: thumb-opcodes.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm9es.h\" $<

arm-dp-x86.7.o: arm-dp-x86.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm7tdmi.h\" $<

arm-ls-x86.7.o: arm-ls-x86.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm7tdmi.h\" $<

#arm-dp-x86.9.o: arm-dp-x86.cpp
#	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm9es.h\" $<

#arm-ls-x86.9.o: arm-ls-x86.cpp
#	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm9es.h\" $<

thumb-opcodes-x86.7.o: thumb-opcodes-x86.cpp
	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm7tdmi.h\" $<

#thumb-opcodes-x86.9.o: thumb-opcodes-x86.cpp
#	$(CC) $(DEFS) -o $@ -D__ARMHDR=\"arm9es.h\" $<

uiflt%.o: DEFS=$(FLTK_CFLAGS)
uiflt%.o: uifltk.h

pa%.o: portaudio.h
pa_uni%.o: pa_unix.h

#---Utilities--------------------------------------------------------------

.PHONY: clean
clean:
	rm -f sysdep.h *.o *.so dsemu

.PHONY: dist
dist:
	rm -f sysdep.h
	mkdir -p dsemu-ng-20060103/src
	cp *.cpp *.c *.h *.ini Make* *.img README Licence-BSD dsemu-ng-20060103/src
	tar cjvf dsemu-ng-20060103.tar.bz2 dsemu-ng-20060103/
	rm -rf dsemu-ng-20060103

.PHONY: sysdep
sysdep:
	rm -f sysdep.h
	echo '#define DSEMU_VERSION_STR "DSemu-ng-20060103"' >> sysdep.h
	gcc -o endtest endtest.c
	if [ -z `./endtest` ]; then echo "#define BOK_BIG_ENDIAN 0" >> sysdep.h; else echo "#define BOK_BIG_ENDIAN 1" >> sysdep.h; fi
	rm -f endtest
	if [ `fltk-config --version 2> /dev/null` > "0" ]; then touch fltk-check; else rm -f fltk-check; fi

fltk-check:
	$(error FLTK seems to be missing. Cannot continue)

.PHONY: fltk-post
fltk-post:
	rm -f fltk-check 0

#*** EOF: Makefile.nix ****************************************************
