/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM cores: Common: Data processing opcodes [arm-dp.cpp]        *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#ifdef ARM7TDMI_X86

// __ARMHDR is set by the Makefile to be either arm7tdmi.h or arm9es.h
#include __ARMHDR

#include "datadefs.h"
#include "armmasks.h"
#include "armhelp.h"

//---Addressing modes------------------------------------------------------
// BUGBUG: Doubling of Rm15hack in DP addressing is GBA-specific
// ACK: Stephen Stair (DP addressing, _RN would be overwritten if Rn==Rd)

#define addrDPrn 

#define addrDPimm \

#define addrDPreg \

#define addrDPlli \

#define addrDPllr \

#define addrDPlri \

#define addrDPlrr \

#define addrDPari \

#define addrDParr \

#define addrDPrri \

#define addrDPrrr \

//---Opcode actions--------------------------------------------------------
								      
#define opAND
#define opEOR
#define opSUB
#define opRSB
#define opADD
#define opADC
#define opSBC
#define opRSC
#define opTST
#define opTEQ
#define opCMP
#define opCMN
#define opORR
#define opMOV
#define opBIC
#define opMVN

//---Flag setting----------------------------------------------------------
// ACK: Mike Murphy [emu7800.sf.net] (V flag setting inspired by M6502.cs)
								      
#define cpsrDP \
								      
#define flagDPnz \

#define flagDPnzTST \

#define flagDPcBIT \

#define flagDPvADD 
#define flagDPvCMN
#define flagDPvSUB
#define flagDPvRSB
#define flagDPvCMP
								      
#define flagDPcADD
#define flagDPcCMN
#define flagDPcSUB
#define flagDPcRSB
#define flagDPcCMP
								      
// Once the defines are in place, it's a simple matter of tacking
// addressing, opcode and flagset together. A few hundred times.
								      
OPC xopANDlli() { addrDPrn; addrDPlli; opAND; return 2; }
OPC xopANDllr() { addrDPrn; addrDPllr; opAND; return 2; }
OPC xopANDlri() { addrDPrn; addrDPlri; opAND; return 2; }
OPC xopANDlrr() { addrDPrn; addrDPlrr; opAND; return 2; }
OPC xopANDari() { addrDPrn; addrDPari; opAND; return 2; }
OPC xopANDarr() { addrDPrn; addrDParr; opAND; return 2; }
OPC xopANDrri() { addrDPrn; addrDPrri; opAND; return 2; }
OPC xopANDrrr() { addrDPrn; addrDPrrr; opAND; return 2; }
OPC xopANDimm() { addrDPrn; addrDPimm; opAND; return 1; }

OPC xopEORlli() { addrDPrn; addrDPlli; opEOR; return 2; }
OPC xopEORllr() { addrDPrn; addrDPllr; opEOR; return 2; }
OPC xopEORlri() { addrDPrn; addrDPlri; opEOR; return 2; }
OPC xopEORlrr() { addrDPrn; addrDPlrr; opEOR; return 2; }
OPC xopEORari() { addrDPrn; addrDPari; opEOR; return 2; }
OPC xopEORarr() { addrDPrn; addrDParr; opEOR; return 2; }
OPC xopEORrri() { addrDPrn; addrDPrri; opEOR; return 2; }
OPC xopEORrrr() { addrDPrn; addrDPrrr; opEOR; return 2; }
OPC xopEORimm() { addrDPrn; addrDPimm; opEOR; return 1; }

OPC xopSUBlli() { addrDPrn; addrDPlli; opSUB; return 2; }
OPC xopSUBllr() { addrDPrn; addrDPllr; opSUB; return 2; }
OPC xopSUBlri() { addrDPrn; addrDPlri; opSUB; return 2; }
OPC xopSUBlrr() { addrDPrn; addrDPlrr; opSUB; return 2; }
OPC xopSUBari() { addrDPrn; addrDPari; opSUB; return 2; }
OPC xopSUBarr() { addrDPrn; addrDParr; opSUB; return 2; }
OPC xopSUBrri() { addrDPrn; addrDPrri; opSUB; return 2; }
OPC xopSUBrrr() { addrDPrn; addrDPrrr; opSUB; return 2; }
OPC xopSUBimm() { addrDPrn; addrDPimm; opSUB; return 1; }

OPC xopRSBlli() { addrDPrn; addrDPlli; opRSB; return 2; }
OPC xopRSBllr() { addrDPrn; addrDPllr; opRSB; return 2; }
OPC xopRSBlri() { addrDPrn; addrDPlri; opRSB; return 2; }
OPC xopRSBlrr() { addrDPrn; addrDPlrr; opRSB; return 2; }
OPC xopRSBari() { addrDPrn; addrDPari; opRSB; return 2; }
OPC xopRSBarr() { addrDPrn; addrDParr; opRSB; return 2; }
OPC xopRSBrri() { addrDPrn; addrDPrri; opRSB; return 2; }
OPC xopRSBrrr() { addrDPrn; addrDPrrr; opRSB; return 2; }
OPC xopRSBimm() { addrDPrn; addrDPimm; opRSB; return 1; }

OPC xopADDlli() { addrDPrn; addrDPlli; opADD; return 2; }
OPC xopADDllr() { addrDPrn; addrDPllr; opADD; return 2; }
OPC xopADDlri() { addrDPrn; addrDPlri; opADD; return 2; }
OPC xopADDlrr() { addrDPrn; addrDPlrr; opADD; return 2; }
OPC xopADDari() { addrDPrn; addrDPari; opADD; return 2; }
OPC xopADDarr() { addrDPrn; addrDParr; opADD; return 2; }
OPC xopADDrri() { addrDPrn; addrDPrri; opADD; return 2; }
OPC xopADDrrr() { addrDPrn; addrDPrrr; opADD; return 2; }
OPC xopADDimm() { addrDPrn; addrDPimm; opADD; return 1; }

OPC xopADClli() { addrDPrn; addrDPlli; opADC; return 2; }
OPC xopADCllr() { addrDPrn; addrDPllr; opADC; return 2; }
OPC xopADClri() { addrDPrn; addrDPlri; opADC; return 2; }
OPC xopADClrr() { addrDPrn; addrDPlrr; opADC; return 2; }
OPC xopADCari() { addrDPrn; addrDPari; opADC; return 2; }
OPC xopADCarr() { addrDPrn; addrDParr; opADC; return 2; }
OPC xopADCrri() { addrDPrn; addrDPrri; opADC; return 2; }
OPC xopADCrrr() { addrDPrn; addrDPrrr; opADC; return 2; }
OPC xopADCimm() { addrDPrn; addrDPimm; opADC; return 1; }

OPC xopSBClli() { addrDPrn; addrDPlli; opSBC; return 2; }
OPC xopSBCllr() { addrDPrn; addrDPllr; opSBC; return 2; }
OPC xopSBClri() { addrDPrn; addrDPlri; opSBC; return 2; }
OPC xopSBClrr() { addrDPrn; addrDPlrr; opSBC; return 2; }
OPC xopSBCari() { addrDPrn; addrDPari; opSBC; return 2; }
OPC xopSBCarr() { addrDPrn; addrDParr; opSBC; return 2; }
OPC xopSBCrri() { addrDPrn; addrDPrri; opSBC; return 2; }
OPC xopSBCrrr() { addrDPrn; addrDPrrr; opSBC; return 2; }
OPC xopSBCimm() { addrDPrn; addrDPimm; opSBC; return 1; }

OPC xopRSClli() { addrDPrn; addrDPlli; opRSC; return 2; }
OPC xopRSCllr() { addrDPrn; addrDPllr; opRSC; return 2; }
OPC xopRSClri() { addrDPrn; addrDPlri; opRSC; return 2; }
OPC xopRSClrr() { addrDPrn; addrDPlrr; opRSC; return 2; }
OPC xopRSCari() { addrDPrn; addrDPari; opRSC; return 2; }
OPC xopRSCarr() { addrDPrn; addrDParr; opRSC; return 2; }
OPC xopRSCrri() { addrDPrn; addrDPrri; opRSC; return 2; }
OPC xopRSCrrr() { addrDPrn; addrDPrrr; opRSC; return 2; }
OPC xopRSCimm() { addrDPrn; addrDPimm; opRSC; return 1; }

OPC xopORRlli() { addrDPrn; addrDPlli; opORR; return 2; }
OPC xopORRllr() { addrDPrn; addrDPllr; opORR; return 2; }
OPC xopORRlri() { addrDPrn; addrDPlri; opORR; return 2; }
OPC xopORRlrr() { addrDPrn; addrDPlrr; opORR; return 2; }
OPC xopORRari() { addrDPrn; addrDPari; opORR; return 2; }
OPC xopORRarr() { addrDPrn; addrDParr; opORR; return 2; }
OPC xopORRrri() { addrDPrn; addrDPrri; opORR; return 2; }
OPC xopORRrrr() { addrDPrn; addrDPrrr; opORR; return 2; }
OPC xopORRimm() { addrDPrn; addrDPimm; opORR; return 1; }

OPC xopMOVlli() { addrDPrn; addrDPlli; opMOV; return 2; }
OPC xopMOVllr() { addrDPrn; addrDPllr; opMOV; return 2; }
OPC xopMOVlri() { addrDPrn; addrDPlri; opMOV; return 2; }
OPC xopMOVlrr() { addrDPrn; addrDPlrr; opMOV; return 2; }
OPC xopMOVari() { addrDPrn; addrDPari; opMOV; return 2; }
OPC xopMOVarr() { addrDPrn; addrDParr; opMOV; return 2; }
OPC xopMOVrri() { addrDPrn; addrDPrri; opMOV; return 2; }
OPC xopMOVrrr() { addrDPrn; addrDPrrr; opMOV; return 2; }
OPC xopMOVimm() { addrDPrn; addrDPimm; opMOV; return 1; }

OPC xopBIClli() { addrDPrn; addrDPlli; opBIC; return 2; }
OPC xopBICllr() { addrDPrn; addrDPllr; opBIC; return 2; }
OPC xopBIClri() { addrDPrn; addrDPlri; opBIC; return 2; }
OPC xopBIClrr() { addrDPrn; addrDPlrr; opBIC; return 2; }
OPC xopBICari() { addrDPrn; addrDPari; opBIC; return 2; }
OPC xopBICarr() { addrDPrn; addrDParr; opBIC; return 2; }
OPC xopBICrri() { addrDPrn; addrDPrri; opBIC; return 2; }
OPC xopBICrrr() { addrDPrn; addrDPrrr; opBIC; return 2; }
OPC xopBICimm() { addrDPrn; addrDPimm; opBIC; return 1; }

OPC xopMVNlli() { addrDPrn; addrDPlli; opMVN; return 2; }
OPC xopMVNllr() { addrDPrn; addrDPllr; opMVN; return 2; }
OPC xopMVNlri() { addrDPrn; addrDPlri; opMVN; return 2; }
OPC xopMVNlrr() { addrDPrn; addrDPlrr; opMVN; return 2; }
OPC xopMVNari() { addrDPrn; addrDPari; opMVN; return 2; }
OPC xopMVNarr() { addrDPrn; addrDParr; opMVN; return 2; }
OPC xopMVNrri() { addrDPrn; addrDPrri; opMVN; return 2; }
OPC xopMVNrrr() { addrDPrn; addrDPrrr; opMVN; return 2; }
OPC xopMVNimm() { addrDPrn; addrDPimm; opMVN; return 1; }

OPC xopANDSlli() { addrDPrn; addrDPlli; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSllr() { addrDPrn; addrDPllr; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSlri() { addrDPrn; addrDPlri; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSlrr() { addrDPrn; addrDPlrr; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSari() { addrDPrn; addrDPari; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSarr() { addrDPrn; addrDParr; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSrri() { addrDPrn; addrDPrri; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSrrr() { addrDPrn; addrDPrrr; opAND; flagDPnz; cpsrDP; return 2; }
OPC xopANDSimm() { addrDPrn; addrDPimm; opAND; flagDPnz; cpsrDP; return 1; }

OPC xopEORSlli() { addrDPrn; addrDPlli; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSllr() { addrDPrn; addrDPllr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSlri() { addrDPrn; addrDPlri; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSlrr() { addrDPrn; addrDPlrr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSari() { addrDPrn; addrDPari; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSarr() { addrDPrn; addrDParr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSrri() { addrDPrn; addrDPrri; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSrrr() { addrDPrn; addrDPrrr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC xopEORSimm() { addrDPrn; addrDPimm; opEOR; flagDPnz; cpsrDP; return 1; }

OPC xopSUBSlli() { addrDPrn; addrDPlli; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSllr() { addrDPrn; addrDPllr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSlri() { addrDPrn; addrDPlri; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSlrr() { addrDPrn; addrDPlrr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSari() { addrDPrn; addrDPari; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSarr() { addrDPrn; addrDParr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSrri() { addrDPrn; addrDPrri; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSrrr() { addrDPrn; addrDPrrr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSUBSimm() { addrDPrn; addrDPimm; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 1; }

OPC xopRSBSlli() { addrDPrn; addrDPlli; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSllr() { addrDPrn; addrDPllr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSlri() { addrDPrn; addrDPlri; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSlrr() { addrDPrn; addrDPlrr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSari() { addrDPrn; addrDPari; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSarr() { addrDPrn; addrDParr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSrri() { addrDPrn; addrDPrri; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSrrr() { addrDPrn; addrDPrrr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSBSimm() { addrDPrn; addrDPimm; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 1; }

OPC xopADDSlli() { addrDPrn; addrDPlli; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSllr() { addrDPrn; addrDPllr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSlri() { addrDPrn; addrDPlri; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSlrr() { addrDPrn; addrDPlrr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSari() { addrDPrn; addrDPari; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSarr() { addrDPrn; addrDParr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSrri() { addrDPrn; addrDPrri; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSrrr() { addrDPrn; addrDPrrr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADDSimm() { addrDPrn; addrDPimm; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 1; }

OPC xopADCSlli() { addrDPrn; addrDPlli; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSllr() { addrDPrn; addrDPllr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSlri() { addrDPrn; addrDPlri; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSlrr() { addrDPrn; addrDPlrr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSari() { addrDPrn; addrDPari; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSarr() { addrDPrn; addrDParr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSrri() { addrDPrn; addrDPrri; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSrrr() { addrDPrn; addrDPrrr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC xopADCSimm() { addrDPrn; addrDPimm; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 1; }

OPC xopSBCSlli() { addrDPrn; addrDPlli; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSllr() { addrDPrn; addrDPllr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSlri() { addrDPrn; addrDPlri; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSlrr() { addrDPrn; addrDPlrr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSari() { addrDPrn; addrDPari; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSarr() { addrDPrn; addrDParr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSrri() { addrDPrn; addrDPrri; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSrrr() { addrDPrn; addrDPrrr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC xopSBCSimm() { addrDPrn; addrDPimm; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 1; }

OPC xopRSCSlli() { addrDPrn; addrDPlli; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSllr() { addrDPrn; addrDPllr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSlri() { addrDPrn; addrDPlri; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSlrr() { addrDPrn; addrDPlrr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSari() { addrDPrn; addrDPari; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSarr() { addrDPrn; addrDParr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSrri() { addrDPrn; addrDPrri; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSrrr() { addrDPrn; addrDPrrr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC xopRSCSimm() { addrDPrn; addrDPimm; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 1; }

OPC xopTSTSlli() { addrDPrn; addrDPlli; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSllr() { addrDPrn; addrDPllr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSlri() { addrDPrn; addrDPlri; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSlrr() { addrDPrn; addrDPlrr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSari() { addrDPrn; addrDPari; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSarr() { addrDPrn; addrDParr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSrri() { addrDPrn; addrDPrri; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSrrr() { addrDPrn; addrDPrrr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC xopTSTSimm() { addrDPrn; addrDPimm; opTST; flagDPnzTST; cpsrDP; return 1; }

OPC xopTEQSlli() { addrDPrn; addrDPlli; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSllr() { addrDPrn; addrDPllr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSlri() { addrDPrn; addrDPlri; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSlrr() { addrDPrn; addrDPlrr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSari() { addrDPrn; addrDPari; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSarr() { addrDPrn; addrDParr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSrri() { addrDPrn; addrDPrri; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSrrr() { addrDPrn; addrDPrrr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC xopTEQSimm() { addrDPrn; addrDPimm; opTEQ; flagDPnzTST; cpsrDP; return 1; }

OPC xopCMPSlli() { addrDPrn; addrDPlli; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSllr() { addrDPrn; addrDPllr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSlri() { addrDPrn; addrDPlri; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSlrr() { addrDPrn; addrDPlrr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSari() { addrDPrn; addrDPari; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSarr() { addrDPrn; addrDParr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSrri() { addrDPrn; addrDPrri; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSrrr() { addrDPrn; addrDPrrr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC xopCMPSimm() { addrDPrn; addrDPimm; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 1; }

OPC xopCMNSlli() { addrDPrn; addrDPlli; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSllr() { addrDPrn; addrDPllr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSlri() { addrDPrn; addrDPlri; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSlrr() { addrDPrn; addrDPlrr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSari() { addrDPrn; addrDPari; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSarr() { addrDPrn; addrDParr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSrri() { addrDPrn; addrDPrri; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSrrr() { addrDPrn; addrDPrrr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC xopCMNSimm() { addrDPrn; addrDPimm; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 1; }

OPC xopORRSlli() { addrDPrn; addrDPlli; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSllr() { addrDPrn; addrDPllr; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSlri() { addrDPrn; addrDPlri; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSlrr() { addrDPrn; addrDPlrr; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSari() { addrDPrn; addrDPari; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSarr() { addrDPrn; addrDParr; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSrri() { addrDPrn; addrDPrri; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSrrr() { addrDPrn; addrDPrrr; opORR; flagDPnz; cpsrDP; return 2; }
OPC xopORRSimm() { addrDPrn; addrDPimm; opORR; flagDPnz; cpsrDP; return 1; }

OPC xopMOVSlli() { addrDPrn; addrDPlli; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSllr() { addrDPrn; addrDPllr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSlri() { addrDPrn; addrDPlri; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSlrr() { addrDPrn; addrDPlrr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSari() { addrDPrn; addrDPari; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSarr() { addrDPrn; addrDParr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSrri() { addrDPrn; addrDPrri; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSrrr() { addrDPrn; addrDPrrr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC xopMOVSimm() { addrDPrn; addrDPimm; opMOV; flagDPnz; cpsrDP; return 1; }

OPC xopBICSlli() { addrDPrn; addrDPlli; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSllr() { addrDPrn; addrDPllr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSlri() { addrDPrn; addrDPlri; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSlrr() { addrDPrn; addrDPlrr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSari() { addrDPrn; addrDPari; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSarr() { addrDPrn; addrDParr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSrri() { addrDPrn; addrDPrri; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSrrr() { addrDPrn; addrDPrrr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC xopBICSimm() { addrDPrn; addrDPimm; opBIC; flagDPnz; cpsrDP; return 1; }

OPC xopMVNSlli() { addrDPrn; addrDPlli; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSllr() { addrDPrn; addrDPllr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSlri() { addrDPrn; addrDPlri; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSlrr() { addrDPrn; addrDPlrr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSari() { addrDPrn; addrDPari; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSarr() { addrDPrn; addrDParr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSrri() { addrDPrn; addrDPrri; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSrrr() { addrDPrn; addrDPrrr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC xopMVNSimm() { addrDPrn; addrDPimm; opMVN; flagDPnz; cpsrDP; return 1; }

// Also in this file are various opcodes dealing with data, which aren't
// strictly within the manual's DP remit, but are common among the cores.

//---Multiply--------------------------------------------------------------
// ACK: Mark Andrews, "Atari Roots" [1984] (Long mul algorithm extended)
// ACK: Commodore Hacking, issue 8 (Signed long mul negation policy)
// NOTE: The Rd and Rs fields are swapped in the instruction definition;
//       the destination register for multiplies is Rs!

OPC xopMUL()
{
    return 1;
}

OPC xopMLA()
{
    return 1;
}

OPC xopMULS()
{
    return 1;
}

OPC xopMLAS()
{
    return 1;
}

OPC xopUMULL()
{
    return 1;
}

OPC xopUMLAL()
{
    return 1;
}

OPC xopSMULL()
{
    return 1; 
}

OPC xopSMLAL()
{
    return 1;
}

OPC xopUMULLS()
{
    return 1;
}

OPC xopUMLALS()
{
    return 1;
}

OPC xopSMULLS()
{
    return 1;
}

OPC xopSMLALS()
{
    return 1; 
}

//---Move between register and cpsr----------------------------------------

OPC xopMRSrc()
{
    return 1;
}

OPC xopMRSrs()
{
    return 1;
}

OPC xopMSRrc()
{
    return 1;
}

OPC xopMSRrs()
{
    return 1;
}

OPC xopMSRic()
{
    return 1;
}

OPC xopMSRis()
{
    return 1;
}

//---Swap between registers and memory-------------------------------------

OPC xopSWP()
{
    return 1;
}

OPC xopSWPB()
{
    return 1;
}

#endif

/*** EOF: arm-dp.cpp *****************************************************/

