/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM cores: Common: Data processing opcodes [arm-dp.cpp]        *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

// __ARMHDR is set by the Makefile to be either arm7tdmi.h or arm9es.h
#include __ARMHDR

#include "datadefs.h"
#include "armmasks.h"
#include "armhelp.h"

//---Addressing modes------------------------------------------------------
// BUGBUG: Doubling of Rm15hack in DP addressing is GBA-specific
// ACK: Stephen Stair (DP addressing, _RN would be overwritten if Rn==Rd)

#define addrDPrn reg.tmp2=_RN+Rn15hack

#define addrDPimm \
  reg.tmp1 = _ROR(reg.curop&MSK_IMM8, (reg.curop&MSK_RS)>>7); \
  reg.tmpc = (reg.curop&MSK_RS)?(reg.tmp1>>31):reg.flags[FLAG_C]

#define addrDPreg \
  reg.tmp1 = _RM+Rm15hack; \
  reg.tmpc = reg.flags[FLAG_C]

#define addrDPlli \
  reg.tmp1 = (_RM+Rm15hack) << helpDPshfti; \
  reg.tmpc = (helpDPshfti)?(((_RM+Rm15hack)&(1<<(32-helpDPshfti)))?1:0):reg.flags[FLAG_C]

#define addrDPllr \
  reg.tmp1 = (_RM+Rm15hack+Rm15hack) << helpDPshftr; \
  reg.tmpc = helpDPshftr \
             ?((helpDPshftr>32)?0:(((_RM+Rm15hack+Rm15hack)&(1<<(32-helpDPshftr)))?1:0)) \
             :reg.flags[FLAG_C]

#define addrDPlri \
  if(!helpDPshfti) reg.tmp1=0; else \
  reg.tmp1 = (_RM+Rm15hack) >> helpDPshfti; \
  reg.tmpc = helpDPshfti?(((_RM+Rm15hack)&(1<<(helpDPshfti-1)))?1:0):(_RM>>31)

#define addrDPlrr \
  reg.tmp1 = (helpDPshftr>=32)?0:((_RM+Rm15hack+Rm15hack) >> helpDPshftr); \
  reg.tmpc = helpDPshftr \
             ?((helpDPshftr>32)?0:(((_RM+Rm15hack+Rm15hack)&(1<<(helpDPshftr-1)))?1:0)) \
             :reg.flags[FLAG_C]

#define addrDPari \
  if(!helpDPshfti) reg.tmp1=((_RM+Rm15hack)&0x80000000)?0xFFFFFFFF:0; \
  else reg.tmp1 = (signed)(_RM+Rm15hack) >> helpDPshfti; \
  reg.tmpc = helpDPshfti?(((_RM+Rm15hack)&(1<<(helpDPshfti-1)))?1:0):((_RM+Rm15hack)>>31)

#define addrDParr \
  if(helpDPshftr>=32) reg.tmp1=((_RM+Rm15hack+Rm15hack)&0x80000000)?~0:0; \
  else reg.tmp1 = (signed)(_RM+Rm15hack+Rm15hack) >> helpDPshftr; \
  reg.tmpc = helpDPshftr \
             ?((helpDPshftr>=32)?((_RM+Rm15hack+Rm15hack)>>31):(((_RM+Rm15hack+Rm15hack)&(1<<(helpDPshftr-1)))?1:0)) \
             :reg.flags[FLAG_C]

#define addrDPrri \
  reg.tmp1 = (helpDPshfti)?_ROR((_RM+Rm15hack), helpDPshfti):(((_RM+Rm15hack)>>1)|(reg.flags[FLAG_C]<<31)); \
  reg.tmpc = helpDPshfti?(((_RM+Rm15hack)&(1<<(helpDPshfti-1)))?1:0):(_RM&1)

#define addrDPrrr \
  reg.tmp1 = _ROR(_RM+Rm15hack+Rm15hack, (helpDPshftr&31)); \
  reg.tmpc = helpDPshftr \
             ?((helpDPshftr&31)?(((_RM+Rm15hack+Rm15hack)&(1<<(((helpDPshftr&31)-1))))?1:0):((_RM+Rm15hack+Rm15hack)>>31)) \
             :reg.flags[FLAG_C]

//---Opcode actions--------------------------------------------------------
								      
#define opAND _RD = reg.tmp2 & reg.tmp1
#define opEOR _RD = reg.tmp2 ^ reg.tmp1
#define opSUB _RD = reg.tmp2 - reg.tmp1
#define opRSB _RD = reg.tmp1 - reg.tmp2
#define opADD _RD = reg.tmp2 + reg.tmp1
#define opADC reg.tmp1 += reg.flags[FLAG_C]; _RD = reg.tmp2 + reg.tmp1
#define opSBC reg.tmp1 += (1-reg.flags[FLAG_C]); _RD = reg.tmp2 - reg.tmp1
#define opRSC reg.tmp1 -= (1-reg.flags[FLAG_C]); _RD = reg.tmp1 - reg.tmp2
#define opTST reg.tmp3 = reg.tmp2 & reg.tmp1
#define opTEQ reg.tmp3 = reg.tmp2 ^ reg.tmp1
#define opCMP reg.tmp3 = reg.tmp2 - reg.tmp1
#define opCMN reg.tmp3 = reg.tmp2 + reg.tmp1
#define opORR _RD = reg.tmp2 | reg.tmp1
#define opMOV _RD = reg.tmp1
#define opBIC _RD = reg.tmp2 & (~reg.tmp1)
#define opMVN _RD = ~reg.tmp1

//---Flag setting----------------------------------------------------------
// ACK: Mike Murphy [emu7800.sf.net] (V flag setting inspired by M6502.cs)
								      
#define cpsrDP \
  if(((reg.curop&MSK_RD)>>SHFT_RD)==15) { \
    /*if((reg.curmode != MODE_USR) && (reg.curmode != MODE_SYS)) opUNP();*/ \
    reg.cpsr=reg.spsr[reg.curmode]; \
    cpsrSplit(); \
  }
								      
#define flagDPnz \
  reg.flags[FLAG_N] = _RD>>31; \
  reg.flags[FLAG_Z] = (_RD)?0:1; \
  reg.flags[FLAG_C] = reg.tmpc

#define flagDPnzTST \
  reg.flags[FLAG_N] = reg.tmp3>>31; \
  reg.flags[FLAG_Z] = (reg.tmp3)?0:1

#define flagDPcBIT \
  reg.flags[FLAG_C] = reg.tmpc

#define flagDPvADD reg.flags[FLAG_V]=( (reg.tmp2^_RD) & (~(reg.tmp1^reg.tmp2)) & 0x80000000)?1:0
#define flagDPvCMN reg.flags[FLAG_V]=( (reg.tmp2^reg.tmp3) & (~(reg.tmp1^reg.tmp2)) & 0x80000000)?1:0
#define flagDPvSUB reg.flags[FLAG_V]=( (~(reg.tmp1^_RD)) & (reg.tmp1^reg.tmp2) & 0x80000000)?1:0
#define flagDPvRSB reg.flags[FLAG_V]=( (~(reg.tmp2^_RD)) & (reg.tmp1^reg.tmp2) & 0x80000000)?1:0
#define flagDPvCMP reg.flags[FLAG_V]=( (~(reg.tmp1^reg.tmp3)) & (reg.tmp1^reg.tmp2) & 0x80000000)?1:0
//#define flagDPvSUB reg.flags[FLAG_V]=(((reg.tmp2&(~reg.tmp1)&(~_RD))|((~reg.tmp2)&reg.tmp1&_RD))&0x80000000)?1:0
//#define flagDPvRSB reg.flags[FLAG_V]=(((reg.tmp1&(~reg.tmp2)&(~_RD))|((~reg.tmp1)&reg.tmp2&_RD))&0x80000000)?1:0
//#define flagDPvCMP reg.flags[FLAG_V]=(((reg.tmp1&(~reg.tmp2)&(~reg.tmp3))|((~reg.tmp1)&reg.tmp2&reg.tmp3))&0x80000000)?1:0
								      
#define flagDPcADD reg.flags[FLAG_C]=(((_RD<reg.tmp2)||(_RD<reg.tmp1))?1:0)
#define flagDPcCMN reg.flags[FLAG_C]=(((reg.tmp3<reg.tmp2)||(reg.tmp3<reg.tmp1))?1:0)
//#define flagDPcSUB reg.flags[FLAG_C]=((reg.tmp1>reg.tmp2)?0:1)
//#define flagDPcRSB reg.flags[FLAG_C]=((reg.tmp1<reg.tmp2)?0:1)
//#define flagDPcCMP flagDPcSUB
#define flagDPcSUB reg.flags[FLAG_C]=(((reg.tmp2&(~reg.tmp1))|(reg.tmp2&(~_RD))|((~reg.tmp1)&(~_RD)))&0x80000000)?1:0
#define flagDPcRSB reg.flags[FLAG_C]=(((reg.tmp1&(~reg.tmp2))|(reg.tmp1&(~_RD))|((~reg.tmp2)&(~_RD)))&0x80000000)?1:0
#define flagDPcCMP reg.flags[FLAG_C]=(((reg.tmp1&(~reg.tmp2))|(reg.tmp1&(~reg.tmp3))|((~reg.tmp2)&(~reg.tmp3)))&0x80000000)?1:0
								      
// Once the defines are in place, it's a simple matter of tacking
// addressing, opcode and flagset together. A few hundred times.
								      
OPC opANDlli() { addrDPrn; addrDPlli; opAND; return 2; }
OPC opANDllr() { addrDPrn; addrDPllr; opAND; return 2; }
OPC opANDlri() { addrDPrn; addrDPlri; opAND; return 2; }
OPC opANDlrr() { addrDPrn; addrDPlrr; opAND; return 2; }
OPC opANDari() { addrDPrn; addrDPari; opAND; return 2; }
OPC opANDarr() { addrDPrn; addrDParr; opAND; return 2; }
OPC opANDrri() { addrDPrn; addrDPrri; opAND; return 2; }
OPC opANDrrr() { addrDPrn; addrDPrrr; opAND; return 2; }
OPC opANDimm() { addrDPrn; addrDPimm; opAND; return 1; }

OPC opEORlli() { addrDPrn; addrDPlli; opEOR; return 2; }
OPC opEORllr() { addrDPrn; addrDPllr; opEOR; return 2; }
OPC opEORlri() { addrDPrn; addrDPlri; opEOR; return 2; }
OPC opEORlrr() { addrDPrn; addrDPlrr; opEOR; return 2; }
OPC opEORari() { addrDPrn; addrDPari; opEOR; return 2; }
OPC opEORarr() { addrDPrn; addrDParr; opEOR; return 2; }
OPC opEORrri() { addrDPrn; addrDPrri; opEOR; return 2; }
OPC opEORrrr() { addrDPrn; addrDPrrr; opEOR; return 2; }
OPC opEORimm() { addrDPrn; addrDPimm; opEOR; return 1; }

OPC opSUBlli() { addrDPrn; addrDPlli; opSUB; return 2; }
OPC opSUBllr() { addrDPrn; addrDPllr; opSUB; return 2; }
OPC opSUBlri() { addrDPrn; addrDPlri; opSUB; return 2; }
OPC opSUBlrr() { addrDPrn; addrDPlrr; opSUB; return 2; }
OPC opSUBari() { addrDPrn; addrDPari; opSUB; return 2; }
OPC opSUBarr() { addrDPrn; addrDParr; opSUB; return 2; }
OPC opSUBrri() { addrDPrn; addrDPrri; opSUB; return 2; }
OPC opSUBrrr() { addrDPrn; addrDPrrr; opSUB; return 2; }
OPC opSUBimm() { addrDPrn; addrDPimm; opSUB; return 1; }

OPC opRSBlli() { addrDPrn; addrDPlli; opRSB; return 2; }
OPC opRSBllr() { addrDPrn; addrDPllr; opRSB; return 2; }
OPC opRSBlri() { addrDPrn; addrDPlri; opRSB; return 2; }
OPC opRSBlrr() { addrDPrn; addrDPlrr; opRSB; return 2; }
OPC opRSBari() { addrDPrn; addrDPari; opRSB; return 2; }
OPC opRSBarr() { addrDPrn; addrDParr; opRSB; return 2; }
OPC opRSBrri() { addrDPrn; addrDPrri; opRSB; return 2; }
OPC opRSBrrr() { addrDPrn; addrDPrrr; opRSB; return 2; }
OPC opRSBimm() { addrDPrn; addrDPimm; opRSB; return 1; }

OPC opADDlli() { addrDPrn; addrDPlli; opADD; return 2; }
OPC opADDllr() { addrDPrn; addrDPllr; opADD; return 2; }
OPC opADDlri() { addrDPrn; addrDPlri; opADD; return 2; }
OPC opADDlrr() { addrDPrn; addrDPlrr; opADD; return 2; }
OPC opADDari() { addrDPrn; addrDPari; opADD; return 2; }
OPC opADDarr() { addrDPrn; addrDParr; opADD; return 2; }
OPC opADDrri() { addrDPrn; addrDPrri; opADD; return 2; }
OPC opADDrrr() { addrDPrn; addrDPrrr; opADD; return 2; }
OPC opADDimm() { addrDPrn; addrDPimm; opADD; return 1; }

OPC opADClli() { addrDPrn; addrDPlli; opADC; return 2; }
OPC opADCllr() { addrDPrn; addrDPllr; opADC; return 2; }
OPC opADClri() { addrDPrn; addrDPlri; opADC; return 2; }
OPC opADClrr() { addrDPrn; addrDPlrr; opADC; return 2; }
OPC opADCari() { addrDPrn; addrDPari; opADC; return 2; }
OPC opADCarr() { addrDPrn; addrDParr; opADC; return 2; }
OPC opADCrri() { addrDPrn; addrDPrri; opADC; return 2; }
OPC opADCrrr() { addrDPrn; addrDPrrr; opADC; return 2; }
OPC opADCimm() { addrDPrn; addrDPimm; opADC; return 1; }

OPC opSBClli() { addrDPrn; addrDPlli; opSBC; return 2; }
OPC opSBCllr() { addrDPrn; addrDPllr; opSBC; return 2; }
OPC opSBClri() { addrDPrn; addrDPlri; opSBC; return 2; }
OPC opSBClrr() { addrDPrn; addrDPlrr; opSBC; return 2; }
OPC opSBCari() { addrDPrn; addrDPari; opSBC; return 2; }
OPC opSBCarr() { addrDPrn; addrDParr; opSBC; return 2; }
OPC opSBCrri() { addrDPrn; addrDPrri; opSBC; return 2; }
OPC opSBCrrr() { addrDPrn; addrDPrrr; opSBC; return 2; }
OPC opSBCimm() { addrDPrn; addrDPimm; opSBC; return 1; }

OPC opRSClli() { addrDPrn; addrDPlli; opRSC; return 2; }
OPC opRSCllr() { addrDPrn; addrDPllr; opRSC; return 2; }
OPC opRSClri() { addrDPrn; addrDPlri; opRSC; return 2; }
OPC opRSClrr() { addrDPrn; addrDPlrr; opRSC; return 2; }
OPC opRSCari() { addrDPrn; addrDPari; opRSC; return 2; }
OPC opRSCarr() { addrDPrn; addrDParr; opRSC; return 2; }
OPC opRSCrri() { addrDPrn; addrDPrri; opRSC; return 2; }
OPC opRSCrrr() { addrDPrn; addrDPrrr; opRSC; return 2; }
OPC opRSCimm() { addrDPrn; addrDPimm; opRSC; return 1; }

OPC opORRlli() { addrDPrn; addrDPlli; opORR; return 2; }
OPC opORRllr() { addrDPrn; addrDPllr; opORR; return 2; }
OPC opORRlri() { addrDPrn; addrDPlri; opORR; return 2; }
OPC opORRlrr() { addrDPrn; addrDPlrr; opORR; return 2; }
OPC opORRari() { addrDPrn; addrDPari; opORR; return 2; }
OPC opORRarr() { addrDPrn; addrDParr; opORR; return 2; }
OPC opORRrri() { addrDPrn; addrDPrri; opORR; return 2; }
OPC opORRrrr() { addrDPrn; addrDPrrr; opORR; return 2; }
OPC opORRimm() { addrDPrn; addrDPimm; opORR; return 1; }

OPC opMOVlli() { addrDPrn; addrDPlli; opMOV; return 2; }
OPC opMOVllr() { addrDPrn; addrDPllr; opMOV; return 2; }
OPC opMOVlri() { addrDPrn; addrDPlri; opMOV; return 2; }
OPC opMOVlrr() { addrDPrn; addrDPlrr; opMOV; return 2; }
OPC opMOVari() { addrDPrn; addrDPari; opMOV; return 2; }
OPC opMOVarr() { addrDPrn; addrDParr; opMOV; return 2; }
OPC opMOVrri() { addrDPrn; addrDPrri; opMOV; return 2; }
OPC opMOVrrr() { addrDPrn; addrDPrrr; opMOV; return 2; }
OPC opMOVimm() { addrDPrn; addrDPimm; opMOV; return 1; }

OPC opBIClli() { addrDPrn; addrDPlli; opBIC; return 2; }
OPC opBICllr() { addrDPrn; addrDPllr; opBIC; return 2; }
OPC opBIClri() { addrDPrn; addrDPlri; opBIC; return 2; }
OPC opBIClrr() { addrDPrn; addrDPlrr; opBIC; return 2; }
OPC opBICari() { addrDPrn; addrDPari; opBIC; return 2; }
OPC opBICarr() { addrDPrn; addrDParr; opBIC; return 2; }
OPC opBICrri() { addrDPrn; addrDPrri; opBIC; return 2; }
OPC opBICrrr() { addrDPrn; addrDPrrr; opBIC; return 2; }
OPC opBICimm() { addrDPrn; addrDPimm; opBIC; return 1; }

OPC opMVNlli() { addrDPrn; addrDPlli; opMVN; return 2; }
OPC opMVNllr() { addrDPrn; addrDPllr; opMVN; return 2; }
OPC opMVNlri() { addrDPrn; addrDPlri; opMVN; return 2; }
OPC opMVNlrr() { addrDPrn; addrDPlrr; opMVN; return 2; }
OPC opMVNari() { addrDPrn; addrDPari; opMVN; return 2; }
OPC opMVNarr() { addrDPrn; addrDParr; opMVN; return 2; }
OPC opMVNrri() { addrDPrn; addrDPrri; opMVN; return 2; }
OPC opMVNrrr() { addrDPrn; addrDPrrr; opMVN; return 2; }
OPC opMVNimm() { addrDPrn; addrDPimm; opMVN; return 1; }

OPC opANDSlli() { addrDPrn; addrDPlli; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSllr() { addrDPrn; addrDPllr; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSlri() { addrDPrn; addrDPlri; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSlrr() { addrDPrn; addrDPlrr; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSari() { addrDPrn; addrDPari; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSarr() { addrDPrn; addrDParr; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSrri() { addrDPrn; addrDPrri; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSrrr() { addrDPrn; addrDPrrr; opAND; flagDPnz; cpsrDP; return 2; }
OPC opANDSimm() { addrDPrn; addrDPimm; opAND; flagDPnz; cpsrDP; return 1; }

OPC opEORSlli() { addrDPrn; addrDPlli; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSllr() { addrDPrn; addrDPllr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSlri() { addrDPrn; addrDPlri; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSlrr() { addrDPrn; addrDPlrr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSari() { addrDPrn; addrDPari; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSarr() { addrDPrn; addrDParr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSrri() { addrDPrn; addrDPrri; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSrrr() { addrDPrn; addrDPrrr; opEOR; flagDPnz; cpsrDP; return 2; }
OPC opEORSimm() { addrDPrn; addrDPimm; opEOR; flagDPnz; cpsrDP; return 1; }

OPC opSUBSlli() { addrDPrn; addrDPlli; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSllr() { addrDPrn; addrDPllr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSlri() { addrDPrn; addrDPlri; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSlrr() { addrDPrn; addrDPlrr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSari() { addrDPrn; addrDPari; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSarr() { addrDPrn; addrDParr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSrri() { addrDPrn; addrDPrri; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSrrr() { addrDPrn; addrDPrrr; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSUBSimm() { addrDPrn; addrDPimm; opSUB; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 1; }

OPC opRSBSlli() { addrDPrn; addrDPlli; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSllr() { addrDPrn; addrDPllr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSlri() { addrDPrn; addrDPlri; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSlrr() { addrDPrn; addrDPlrr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSari() { addrDPrn; addrDPari; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSarr() { addrDPrn; addrDParr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSrri() { addrDPrn; addrDPrri; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSrrr() { addrDPrn; addrDPrrr; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSBSimm() { addrDPrn; addrDPimm; opRSB; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 1; }

OPC opADDSlli() { addrDPrn; addrDPlli; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSllr() { addrDPrn; addrDPllr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSlri() { addrDPrn; addrDPlri; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSlrr() { addrDPrn; addrDPlrr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSari() { addrDPrn; addrDPari; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSarr() { addrDPrn; addrDParr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSrri() { addrDPrn; addrDPrri; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSrrr() { addrDPrn; addrDPrrr; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADDSimm() { addrDPrn; addrDPimm; opADD; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 1; }

OPC opADCSlli() { addrDPrn; addrDPlli; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSllr() { addrDPrn; addrDPllr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSlri() { addrDPrn; addrDPlri; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSlrr() { addrDPrn; addrDPlrr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSari() { addrDPrn; addrDPari; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSarr() { addrDPrn; addrDParr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSrri() { addrDPrn; addrDPrri; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSrrr() { addrDPrn; addrDPrrr; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 2; }
OPC opADCSimm() { addrDPrn; addrDPimm; opADC; flagDPnz; flagDPcADD; flagDPvADD; cpsrDP; return 1; }

OPC opSBCSlli() { addrDPrn; addrDPlli; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSllr() { addrDPrn; addrDPllr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSlri() { addrDPrn; addrDPlri; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSlrr() { addrDPrn; addrDPlrr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSari() { addrDPrn; addrDPari; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSarr() { addrDPrn; addrDParr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSrri() { addrDPrn; addrDPrri; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSrrr() { addrDPrn; addrDPrrr; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 2; }
OPC opSBCSimm() { addrDPrn; addrDPimm; opSBC; flagDPnz; flagDPcSUB; flagDPvSUB; cpsrDP; return 1; }

OPC opRSCSlli() { addrDPrn; addrDPlli; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSllr() { addrDPrn; addrDPllr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSlri() { addrDPrn; addrDPlri; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSlrr() { addrDPrn; addrDPlrr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSari() { addrDPrn; addrDPari; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSarr() { addrDPrn; addrDParr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSrri() { addrDPrn; addrDPrri; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSrrr() { addrDPrn; addrDPrrr; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 2; }
OPC opRSCSimm() { addrDPrn; addrDPimm; opRSC; flagDPnz; flagDPcRSB; flagDPvRSB; cpsrDP; return 1; }

OPC opTSTSlli() { addrDPrn; addrDPlli; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSllr() { addrDPrn; addrDPllr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSlri() { addrDPrn; addrDPlri; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSlrr() { addrDPrn; addrDPlrr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSari() { addrDPrn; addrDPari; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSarr() { addrDPrn; addrDParr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSrri() { addrDPrn; addrDPrri; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSrrr() { addrDPrn; addrDPrrr; opTST; flagDPnzTST; cpsrDP; return 2; }
OPC opTSTSimm() { addrDPrn; addrDPimm; opTST; flagDPnzTST; cpsrDP; return 1; }

OPC opTEQSlli() { addrDPrn; addrDPlli; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSllr() { addrDPrn; addrDPllr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSlri() { addrDPrn; addrDPlri; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSlrr() { addrDPrn; addrDPlrr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSari() { addrDPrn; addrDPari; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSarr() { addrDPrn; addrDParr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSrri() { addrDPrn; addrDPrri; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSrrr() { addrDPrn; addrDPrrr; opTEQ; flagDPnzTST; cpsrDP; return 2; }
OPC opTEQSimm() { addrDPrn; addrDPimm; opTEQ; flagDPnzTST; cpsrDP; return 1; }

OPC opCMPSlli() { addrDPrn; addrDPlli; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSllr() { addrDPrn; addrDPllr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSlri() { addrDPrn; addrDPlri; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSlrr() { addrDPrn; addrDPlrr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSari() { addrDPrn; addrDPari; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSarr() { addrDPrn; addrDParr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSrri() { addrDPrn; addrDPrri; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSrrr() { addrDPrn; addrDPrrr; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 2; }
OPC opCMPSimm() { addrDPrn; addrDPimm; opCMP; flagDPnzTST; flagDPcCMP; flagDPvCMP; cpsrDP; return 1; }

OPC opCMNSlli() { addrDPrn; addrDPlli; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSllr() { addrDPrn; addrDPllr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSlri() { addrDPrn; addrDPlri; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSlrr() { addrDPrn; addrDPlrr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSari() { addrDPrn; addrDPari; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSarr() { addrDPrn; addrDParr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSrri() { addrDPrn; addrDPrri; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSrrr() { addrDPrn; addrDPrrr; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 2; }
OPC opCMNSimm() { addrDPrn; addrDPimm; opCMN; flagDPnzTST; flagDPcCMN; flagDPvCMN; cpsrDP; return 1; }

OPC opORRSlli() { addrDPrn; addrDPlli; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSllr() { addrDPrn; addrDPllr; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSlri() { addrDPrn; addrDPlri; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSlrr() { addrDPrn; addrDPlrr; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSari() { addrDPrn; addrDPari; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSarr() { addrDPrn; addrDParr; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSrri() { addrDPrn; addrDPrri; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSrrr() { addrDPrn; addrDPrrr; opORR; flagDPnz; cpsrDP; return 2; }
OPC opORRSimm() { addrDPrn; addrDPimm; opORR; flagDPnz; cpsrDP; return 1; }

OPC opMOVSlli() { addrDPrn; addrDPlli; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSllr() { addrDPrn; addrDPllr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSlri() { addrDPrn; addrDPlri; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSlrr() { addrDPrn; addrDPlrr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSari() { addrDPrn; addrDPari; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSarr() { addrDPrn; addrDParr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSrri() { addrDPrn; addrDPrri; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSrrr() { addrDPrn; addrDPrrr; opMOV; flagDPnz; cpsrDP; return 2; }
OPC opMOVSimm() { addrDPrn; addrDPimm; opMOV; flagDPnz; cpsrDP; return 1; }

OPC opBICSlli() { addrDPrn; addrDPlli; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSllr() { addrDPrn; addrDPllr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSlri() { addrDPrn; addrDPlri; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSlrr() { addrDPrn; addrDPlrr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSari() { addrDPrn; addrDPari; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSarr() { addrDPrn; addrDParr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSrri() { addrDPrn; addrDPrri; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSrrr() { addrDPrn; addrDPrrr; opBIC; flagDPnz; cpsrDP; return 2; }
OPC opBICSimm() { addrDPrn; addrDPimm; opBIC; flagDPnz; cpsrDP; return 1; }

OPC opMVNSlli() { addrDPrn; addrDPlli; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSllr() { addrDPrn; addrDPllr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSlri() { addrDPrn; addrDPlri; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSlrr() { addrDPrn; addrDPlrr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSari() { addrDPrn; addrDPari; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSarr() { addrDPrn; addrDParr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSrri() { addrDPrn; addrDPrri; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSrrr() { addrDPrn; addrDPrrr; opMVN; flagDPnz; cpsrDP; return 2; }
OPC opMVNSimm() { addrDPrn; addrDPimm; opMVN; flagDPnz; cpsrDP; return 1; }

// Also in this file are various opcodes dealing with data, which aren't
// strictly within the manual's DP remit, but are common among the cores.

//---Multiply--------------------------------------------------------------
// ACK: Mark Andrews, "Atari Roots" [1984] (Long mul algorithm extended)
// ACK: Commodore Hacking, issue 8 (Signed long mul negation policy)
// NOTE: The Rd and Rs fields are swapped in the instruction definition;
//       the destination register for multiplies is Rs!

OPC opMUL()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RN==15)) opUNP();*/
/*  if(_OP_RM==_OP_RN) opUNP();*/
    _RN=_RM*_RS;
    return 1;
}

OPC opMLA()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
/*  if(_OP_RM==_OP_RN) opUNP();*/
    _RN=_RM*_RS + _RD;
    return 1;
}

OPC opMULS()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RN==15)) opUNP();*/
/*  if(_OP_RM==_OP_RN) opUNP();*/
    _RN=_RM*_RS; reg.tmp3=_RN; flagDPnzTST;
    return 1;
}

OPC opMLAS()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
/*  if(_OP_RM==_OP_RN) opUNP();*/
    _RN=_RM*_RS + _RD; reg.tmp3=_RN; flagDPnzTST;
    return 1;
}

OPC opUMULL()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS;

    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    _RD = pl; _RN=ph;
    return 1;
}

OPC opUMLAL()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS, plorig=_RD;

    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    _RD += pl; _RN += ph; if(plorig>_RD) _RN++;
    return 1;
}

OPC opSMULL()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS, neg=0;

    if(r&0x80000000) { neg++; r=(~r)+1; }
    if(d&0x80000000) { neg++; d=(~d)+1; }
    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    if(neg==1) { pl=(~pl)+1; ph=(~ph)+1; }
    _RD = pl; _RN=ph;
    return 1; 
}

OPC opSMLAL()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS, neg=0, plorig=_RD;

    if(r&0x80000000) { neg++; r=(~r)+1; }
    if(d&0x80000000) { neg++; d=(~d)+1; }
    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    if(plorig>pl) ph++;
    if(neg==1) { pl=(~pl)+1; ph=(~ph)+1; }
    _RD += pl; _RN+=ph;
    return 1;
}

OPC opUMULLS()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS;

    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    _RD = pl; _RN=ph;
    reg.flags[FLAG_Z]=(!_RD && !_RN)?1:0;
    reg.flags[FLAG_N]=(_RN&0x80000000)?1:0;
    return 1;
}

OPC opUMLALS()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS, plorig=_RD;

    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    _RD += pl; _RN += ph; if(plorig>_RD) _RN++;
    reg.flags[FLAG_Z]=(!_RD && !_RN)?1:0;
    reg.flags[FLAG_N]=(_RN&0x80000000)?1:0;
    return 1;
}

OPC opSMULLS()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS, neg=0;

    if(r&0x80000000) { neg++; r=(~r)+1; }
    if(d&0x80000000) { neg++; d=(~d)+1; }
    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    if(neg==1) { pl=(~pl)+1; ph=(~ph)+1; }
    _RD = pl; _RN=ph;
    reg.flags[FLAG_Z]=(!_RD && !_RN)?1:0;
    reg.flags[FLAG_N]=(_RN&0x80000000)?1:0;
    return 1;
}

OPC opSMLALS()
{
/*  if((_OP_RM==15)|
       (_OP_RS==15)|
       (_OP_RD==15)|
       (_OP_RN==15)) opUNP();*/
    u32 pl=0, ph=0, dshft=0, r=_RM, d=_RS, neg=0, plorig=_RD;

    if(r&0x80000000) { neg++; r=(~r)+1; }
    if(d&0x80000000) { neg++; d=(~d)+1; }
    for(int i=0; i<32; ++i)
    {
	if(r&1) { pl+=d; ph+=dshft; }
	r>>=1; dshft<<=1; dshft+=(d&0x80000000)?1:0; d<<=1;
    }
    
    if(plorig>pl) ph++;
    if(neg==1) { pl=(~pl)+1; ph=(~ph)+1; }
    _RD += pl; _RN+=ph;
    reg.flags[FLAG_Z]=(!_RD && !_RN)?1:0;
    reg.flags[FLAG_N]=(_RN&0x80000000)?1:0;
    return 1; 
}

//---Move between register and cpsr----------------------------------------

OPC opMRSrc()
{
    cpsrUpdate();
/*  if(_OP_RD==15) opUNP();*/
    _RD = reg.cpsr;
    return 1;
}

OPC opMRSrs()
{
    cpsrUpdate();
/*  if(_OP_RD==15) opUNP();*/
    if(reg.curmode==MODE_USR || reg.curmode==MODE_SYS) opUNP();
    _RD = reg.spsr[reg.curmode];
    return 1;
}

OPC opMSRrc()
{
/*  if(_OP_RD==15) opUNP();*/
    reg.tmp1 = _RM; reg.tmp2 = reg.cpsr;
    if(reg.curop & 0x00080000) { reg.tmp2 &= 0x00FFFFFF; reg.tmp2 |= (reg.tmp1&0xFF000000); }
    if(reg.curmode != MODE_USR)
    {
        if(reg.curop & 0x00040000) { reg.tmp2 &= 0xFF00FFFF; reg.tmp2 |= (reg.tmp1&0x00FF0000); }
        if(reg.curop & 0x00020000) { reg.tmp2 &= 0xFFFF00FF; reg.tmp2 |= (reg.tmp1&0x0000FF00); }
        if(reg.curop & 0x00010000) { reg.tmp2 &= 0xFFFFFF00; reg.tmp2 |= (reg.tmp1&0x000000FF); }
    }
    reg.cpsr = reg.tmp2;
    cpsrSplit();
    return 1;
}

OPC opMSRrs()
{
/*  if(_OP_RD==15) opUNP();*/
    if(reg.curmode==MODE_USR || reg.curmode==MODE_SYS) opUNP();
    reg.tmp1 = _RM; reg.tmp2 = reg.spsr[reg.curmode];
    if(reg.curop & 0x00080000) { reg.tmp2 &= 0x00FFFFFF; reg.tmp2 |= (reg.tmp1&0xFF000000); }
    if(reg.curmode != MODE_USR)
    {
        if(reg.curop & 0x00040000) { reg.tmp2 &= 0xFF00FFFF; reg.tmp2 |= (reg.tmp1&0x00FF0000); }
        if(reg.curop & 0x00020000) { reg.tmp2 &= 0xFFFF00FF; reg.tmp2 |= (reg.tmp1&0x0000FF00); }
        if(reg.curop & 0x00010000) { reg.tmp2 &= 0xFFFFFF00; reg.tmp2 |= (reg.tmp1&0x000000FF); }
    }
    reg.spsr[reg.curmode] = reg.tmp2;
    return 1;
}

OPC opMSRic()
{
/*  if(_OP_RD==15) opUNP();*/
    reg.tmp1 = _ROR(reg.curop&MSK_IMM8, (reg.curop&MSK_RS)>>7); reg.tmp2=reg.cpsr;
    if(reg.curop & 0x00080000) { reg.tmp2 &= 0x00FFFFFF; reg.tmp2 |= (reg.tmp1&0xFF000000); }
    if(reg.curmode != MODE_USR)
    {
        if(reg.curop & 0x00040000) { reg.tmp2 &= 0xFF00FFFF; reg.tmp2 |= (reg.tmp1&0x00FF0000); }
        if(reg.curop & 0x00020000) { reg.tmp2 &= 0xFFFF00FF; reg.tmp2 |= (reg.tmp1&0x0000FF00); }
        if(reg.curop & 0x00010000) { reg.tmp2 &= 0xFFFFFF00; reg.tmp2 |= (reg.tmp1&0x000000FF); }
    }
    reg.cpsr = reg.tmp2;
    cpsrSplit();
    return 1;
}

OPC opMSRis()
{
/*  if(_OP_RD==15) opUNP();*/
    if(reg.curmode==MODE_USR || reg.curmode==MODE_SYS) opUNP();
    reg.tmp1 = _ROR(reg.curop&MSK_IMM8, (reg.curop&MSK_RS)>>7); reg.tmp2=reg.spsr[reg.curmode];
    if(reg.curop & 0x00080000) { reg.tmp2 &= 0x00FFFFFF; reg.tmp2 |= (reg.tmp1&0xFF000000); }
    if(reg.curmode != MODE_USR)
    {
        if(reg.curop & 0x00040000) { reg.tmp2 &= 0xFF00FFFF; reg.tmp2 |= (reg.tmp1&0x00FF0000); }
        if(reg.curop & 0x00020000) { reg.tmp2 &= 0xFFFF00FF; reg.tmp2 |= (reg.tmp1&0x0000FF00); }
        if(reg.curop & 0x00010000) { reg.tmp2 &= 0xFFFFFF00; reg.tmp2 |= (reg.tmp1&0x000000FF); }
    }
    reg.spsr[reg.curmode] = reg.tmp2;
    return 1;
}

//---Swap between registers and memory-------------------------------------

OPC opSWP()
{
/*  if((((reg.curop&MSK_RM)>>SHFT_RM)==15)|
       (((reg.curop&MSK_RN)>>SHFT_RN)==15)|
       (((reg.curop&MSK_RD)>>SHFT_RD)==15)) opUNP();*/
/*  if(((reg.curop&MSK_RN)>>SHFT_RN)==((reg.curop&MSK_RD)>>SHFT_RD)) opUNP();*/
/*  if(((reg.curop&MSK_RN)>>SHFT_RN)==((reg.curop&MSK_RM)>>SHFT_RM)) opUNP();*/
    reg.tmp1=_RN; reg.tmp2=_RD; _RD=MMU->rdW(reg.tmp1); LSror;
    MMU->wrW(reg.tmp1, reg.tmp2);
    return 1;
}

OPC opSWPB()
{
/*  if((((reg.curop&MSK_RM)>>SHFT_RM)==15)|
       (((reg.curop&MSK_RN)>>SHFT_RN)==15)|
       (((reg.curop&MSK_RD)>>SHFT_RD)==15)) opUNP();*/
/*  if(((reg.curop&MSK_RN)>>SHFT_RN)==((reg.curop&MSK_RD)>>SHFT_RD)) opUNP();*/
/*  if(((reg.curop&MSK_RN)>>SHFT_RN)==((reg.curop&MSK_RM)>>SHFT_RM)) opUNP();*/
    reg.tmp2=_RD; _RD=(u32)MMU->rdB(_RN); 
    MMU->wrB(_RN, reg.tmp2&255);
    return 1;
}

/*** EOF: arm-dp.cpp *****************************************************/

