/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM cores: Common: Load/Store [arm-ls.cpp]                     *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

// __ARMHDR is set by the Makefile to be either arm7tdmi.h or arm9es.h
#include __ARMHDR

#include "armmasks.h"
#include "datadefs.h"
#include "plgmmu32.h"
#include "armhelp.h"

//---Addressing modes------------------------------------------------------
// Only a few defines need exist here, since the opcodes themselves
// combine them in many configurations. The combination method must be:
// Offset addressing: [ofm|ofp]([Si|Hi|Hr|Sr*]);
// Pre-indexed: [ofm|ofp]([Si|Hi|Hr|Sr*]); wb;
// Post-indexed: [ptm|ptp]([Si|Hi|Hr|Sr*]);

// NOTE: Halfword-style opcodes address their immediates in a different
//       way to the standard word/byte LS.

#define LSofm if(_OP_RM==15) opUNP(); reg.tmp1 = _RN + Rn15hack -
#define LSofp if(_OP_RM==15) opUNP(); reg.tmp1 = _RN + Rn15hack +

#define LSwb _RN = reg.tmp1

#define LSptm if(_OP_RM==15 || _OP_RN==15) opUNP(); reg.tmp1 = _RN + Rn15hack; _RN -=
#define LSptp if(_OP_RM==15 || _OP_RN==15) opUNP(); reg.tmp1 = _RN + Rn15hack; _RN +=

#define LSi (reg.curop&MSK_IMM12)

#define LHi ((reg.curop&MSK_RM)|((reg.curop&MSK_RS)>>4))
#define LHr _RM + Rm15hack

#define LSrll ((_RM+Rm15hack) << helpDPshfti)
#define LSrlr ((helpDPshfti) ? ((_RM+Rm15hack) >> (helpDPshfti)) : 0)
#define LSrar ((helpDPshfti) ? ((signed)(_RM+Rm15hack) >> (helpDPshfti)) : (((_RM+Rm15hack)>>31)?0xFFFFFFFF:0))
#define LSrrr ((helpDPshfti)?(_ROR(_RM+Rm15hack,helpDPshfti)):(((_RM+Rm15hack)>>1)|(reg.flags[FLAG_C]<<31)))

//---Simple and halfword-style addressing----------------------------------
								 
OPC opLDRptim(){ LSptm(LSi); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptip(){ LSptp(LSi); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofim(){ LSofm(LSi); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofip(){ LSofp(LSi); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprim(){ LSofm(LSi); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprip(){ LSofp(LSi); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }

OPC opSTRptim(){ LSptm(LSi); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptip(){ LSptp(LSi); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofim(){ LSofm(LSi); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofip(){ LSofp(LSi); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprim(){ LSofm(LSi); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprip(){ LSofp(LSi); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }

OPC opLDRBptim(){ LSptm(LSi); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptip(){ LSptp(LSi); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofim(){ LSofm(LSi); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofip(){ LSofp(LSi); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprim(){ LSofm(LSi); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprip(){ LSofp(LSi); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }

OPC opSTRBptim(){ LSptm(LSi); MMU->wrB(reg.tmp1,(_RD&255)); return 2; }
OPC opSTRBptip(){ LSptp(LSi); MMU->wrB(reg.tmp1,(_RD&255)); return 2; }
OPC opSTRBofim(){ LSofm(LSi); MMU->wrB(reg.tmp1,(_RD&255)); return 2; }
OPC opSTRBofip(){ LSofp(LSi); MMU->wrB(reg.tmp1,(_RD&255)); return 2; }
OPC opSTRBprim(){ LSofm(LSi); LSwb; MMU->wrB(reg.tmp1,(_RD&255)); return 2; }
OPC opSTRBprip(){ LSofp(LSi); LSwb; MMU->wrB(reg.tmp1,(_RD&255)); return 2; }

OPC opLDRHptim(){ LSptm(LHi); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHptip(){ LSptp(LHi); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHptrm(){ LSptm(LHr); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHptrp(){ LSptp(LHr); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHofim(){ LSofm(LHi); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHofip(){ LSofp(LHi); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHofrm(){ LSofm(LHr); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHofrp(){ LSofp(LHr); /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHprim(){ LSofm(LHi); LSwb; /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHprip(){ LSofp(LHi); LSwb; /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHprrm(){ LSofm(LHr); LSwb; /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }
OPC opLDRHprrp(){ LSofp(LHr); LSwb; /*if(reg.tmp1&1) opUNP();*/ _RD=MMU->rdH(reg.tmp1); return 3; }

OPC opSTRHptim(){ LSptm(LHi); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHptip(){ LSptp(LHi); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHptrm(){ LSptm(LHr); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHptrp(){ LSptp(LHr); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHofim(){ LSofm(LHi); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHofip(){ LSofp(LHi); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHofrm(){ LSofm(LHr); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHofrp(){ LSofp(LHr); /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2;}
OPC opSTRHprim(){ LSofm(LHi); LSwb; /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2; }
OPC opSTRHprip(){ LSofp(LHi); LSwb; /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2; }
OPC opSTRHprrm(){ LSofm(LHr); LSwb; /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2; }
OPC opSTRHprrp(){ LSofp(LHr); LSwb; /*if(reg.tmp1&1) opUNP();*/ MMU->wrH(reg.tmp1,_RD&65535); return 2; }

OPC opLDRTptim(){ MMU->priv(MMU_PRIV_LO); LSptm(LSi); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptip(){ MMU->priv(MMU_PRIV_LO); LSptp(LSi); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTprim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTprip(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }

OPC opSTRTptim(){ MMU->priv(MMU_PRIV_LO); LSptm(LSi); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptip(){ MMU->priv(MMU_PRIV_LO); LSptp(LSi); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2;}
OPC opSTRTprip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2;}

OPC opLDRBTptim(){ MMU->priv(MMU_PRIV_LO); LSptm(LSi); _RD=(u32)MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptip(){ MMU->priv(MMU_PRIV_LO); LSptp(LSi); _RD=(u32)MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); _RD=(u32)MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); _RD=(u32)MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTprim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); LSwb; _RD=(u32)MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTprip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); LSwb; _RD=(u32)MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }

OPC opSTRBTptim(){ MMU->priv(MMU_PRIV_LO); LSptm(LSi); MMU->wrB(reg.tmp1,(_RD&255)); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptip(){ MMU->priv(MMU_PRIV_LO); LSptp(LSi); MMU->wrB(reg.tmp1,(_RD&255)); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); MMU->wrB(reg.tmp1,(_RD&255)); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); MMU->wrB(reg.tmp1,(_RD&255)); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprim(){ MMU->priv(MMU_PRIV_LO); LSofm(LSi); LSwb; MMU->wrB(reg.tmp1,(_RD&255)); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprip(){ MMU->priv(MMU_PRIV_LO); LSofp(LSi); LSwb; MMU->wrB(reg.tmp1,(_RD&255)); MMU->priv(MMU_PRIV_HI); return 2; }

OPC opLDRSBptim(){ LSptm(LHi); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBptip(){ LSptp(LHi); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBptrm(){ LSptm(LHr); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBptrp(){ LSptp(LHr); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBofim(){ LSofm(LHi); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBofip(){ LSofp(LHi); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBofrm(){ LSofm(LHr); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBofrp(){ LSofp(LHr); reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBprim(){ LSofm(LHi); LSwb; reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBprip(){ LSofp(LHi); LSwb; reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBprrm(){ LSofm(LHr); LSwb; reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }
OPC opLDRSBprrp(){ LSofp(LHr); LSwb; reg.tmp2=MMU->rdB(reg.tmp1); _RD=(reg.tmp2&128)?(0xFFFFFF00|reg.tmp2):reg.tmp2; return 3; }

OPC opLDRSHptim(){ LSptm(LHi); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHptip(){ LSptp(LHi); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHptrm(){ LSptm(LHr); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHptrp(){ LSptp(LHr); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHofim(){ LSofm(LHi); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHofip(){ LSofp(LHi); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHofrm(){ LSofm(LHr); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHofrp(){ LSofp(LHr); reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHprim(){ LSofm(LHi); LSwb; reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHprip(){ LSofp(LHi); LSwb; reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHprrm(){ LSofm(LHr); LSwb; reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }
OPC opLDRSHprrp(){ LSofp(LHr); LSwb; reg.tmp2=MMU->rdH(reg.tmp1); _RD=(reg.tmp2&32768)?(0xFFFF0000|reg.tmp2):reg.tmp2; return 2; }

//---Shifted addressing (Word/byte opcodes only, of course)----------------

OPC opLDRptrmll(){ LSptm(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrmlr(){ LSptm(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrmar(){ LSptm(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrmrr(){ LSptm(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrpll(){ LSptp(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrplr(){ LSptp(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrpar(){ LSptp(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRptrprr(){ LSptp(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrmll(){ LSofm(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrmlr(){ LSofm(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrmar(){ LSofm(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrmrr(){ LSofm(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrpll(){ LSofp(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrplr(){ LSofp(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrpar(){ LSofp(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRofrprr(){ LSofp(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrmll(){ LSofm(LSrll); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrmlr(){ LSofm(LSrlr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrmar(){ LSofm(LSrar); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrmrr(){ LSofm(LSrrr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrpll(){ LSofp(LSrll); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrplr(){ LSofp(LSrlr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrpar(){ LSofp(LSrar); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }
OPC opLDRprrprr(){ LSofp(LSrrr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror; return 3; }

OPC opSTRptrmll(){ LSptm(LSrll); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrmlr(){ LSptm(LSrlr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrmar(){ LSptm(LSrar); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrmrr(){ LSptm(LSrrr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrpll(){ LSptp(LSrll); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrplr(){ LSptp(LSrlr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrpar(){ LSptp(LSrar); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRptrprr(){ LSptp(LSrrr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrmll(){ LSofm(LSrll); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrmlr(){ LSofm(LSrlr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrmar(){ LSofm(LSrar); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrmrr(){ LSofm(LSrrr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrpll(){ LSofp(LSrll); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrplr(){ LSofp(LSrlr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrpar(){ LSofp(LSrar); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRofrprr(){ LSofp(LSrrr); MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrmll(){ LSofm(LSrll); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrmlr(){ LSofm(LSrlr); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrmar(){ LSofm(LSrar); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrmrr(){ LSofm(LSrrr); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrpll(){ LSofp(LSrll); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrplr(){ LSofp(LSrlr); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrpar(){ LSofp(LSrar); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }
OPC opSTRprrprr(){ LSofp(LSrrr); LSwb; MMU->wrW(reg.tmp1,_RD); return 2; }

OPC opLDRBptrmll(){ LSptm(LSrll); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrmlr(){ LSptm(LSrlr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrmar(){ LSptm(LSrar); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrmrr(){ LSptm(LSrrr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrpll(){ LSptp(LSrll); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrplr(){ LSptp(LSrlr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrpar(){ LSptp(LSrar); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBptrprr(){ LSptp(LSrrr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrmll(){ LSofm(LSrll); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrmlr(){ LSofm(LSrlr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrmar(){ LSofm(LSrar); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrmrr(){ LSofm(LSrrr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrpll(){ LSofp(LSrll); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrplr(){ LSofp(LSrlr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrpar(){ LSofp(LSrar); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBofrprr(){ LSofp(LSrrr); _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrmll(){ LSofm(LSrll); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrmlr(){ LSofm(LSrlr); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrmar(){ LSofm(LSrar); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrmrr(){ LSofm(LSrrr); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrpll(){ LSofp(LSrll); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrplr(){ LSofp(LSrlr); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrpar(){ LSofp(LSrar); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }
OPC opLDRBprrprr(){ LSofp(LSrrr); LSwb; _RD=MMU->rdB(reg.tmp1); return 3; }

OPC opSTRBptrmll(){ LSptm(LSrll); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrmlr(){ LSptm(LSrlr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrmar(){ LSptm(LSrar); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrmrr(){ LSptm(LSrrr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrpll(){ LSptp(LSrll); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrplr(){ LSptp(LSrlr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrpar(){ LSptp(LSrar); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBptrprr(){ LSptp(LSrrr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrmll(){ LSofm(LSrll); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrmlr(){ LSofm(LSrlr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrmar(){ LSofm(LSrar); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrmrr(){ LSofm(LSrrr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrpll(){ LSofp(LSrll); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrplr(){ LSofp(LSrlr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrpar(){ LSofp(LSrar); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBofrprr(){ LSofp(LSrrr); MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrmll(){ LSofm(LSrll); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrmlr(){ LSofm(LSrlr); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrmar(){ LSofm(LSrar); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrmrr(){ LSofm(LSrrr); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrpll(){ LSofp(LSrll); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrplr(){ LSofp(LSrlr); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrpar(){ LSofp(LSrar); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }
OPC opSTRBprrprr(){ LSofp(LSrrr); LSwb; MMU->wrB(reg.tmp1,_RD&255); return 2; }

OPC opLDRTptrmll(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrmlr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrmar(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrmrr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrpll(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrplr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrpar(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTptrprr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTofrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); _RD=MMU->rdW(reg.tmp1); LSror; MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRTprrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}
OPC opLDRTprrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); LSwb; _RD=MMU->rdW(reg.tmp1); LSror;MMU->priv(MMU_PRIV_HI); return 3;}

OPC opSTRTptrmll(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrll); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrmlr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrlr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrmar(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrar); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrmrr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrrr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrpll(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrll); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrplr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrlr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrpar(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrar); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTptrprr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrrr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTofrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRTprrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); LSwb; MMU->wrW(reg.tmp1,_RD); MMU->priv(MMU_PRIV_HI); return 2; }

OPC opLDRBTptrmll(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrll); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrmlr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrlr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrmar(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrar); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrmrr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrrr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrpll(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrll); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrplr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrlr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrpar(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrar); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTptrprr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrrr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTofrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI); return 3; }
OPC opLDRBTprrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}
OPC opLDRBTprrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); LSwb; _RD=MMU->rdB(reg.tmp1); MMU->priv(MMU_PRIV_HI);return 3;}

OPC opSTRBTptrmll(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrll); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrmlr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrlr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrmar(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrar); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrmrr(){ MMU->priv(MMU_PRIV_LO); LSptm(LSrrr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrpll(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrll); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrplr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrlr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrpar(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrar); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTptrprr(){ MMU->priv(MMU_PRIV_LO); LSptp(LSrrr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTofrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrmll(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrll); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrmlr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrlr); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrmar(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrar); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrmrr(){ MMU->priv(MMU_PRIV_LO); LSofm(LSrrr); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrpll(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrll); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrplr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrlr); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrpar(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrar); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }
OPC opSTRBTprrprr(){ MMU->priv(MMU_PRIV_LO); LSofp(LSrrr); LSwb; MMU->wrB(reg.tmp1,_RD&255); MMU->priv(MMU_PRIV_HI); return 2; }

// Load/store multiple is also in this file, since it's common between
// the cores.

//---Initial pointer setup-------------------------------------------------
// Note: Rn is written back differently dependent on direction

#define addrLMIA reg.tmp1 = (_RN&~3)
#define addrLMIB reg.tmp1 = (_RN&~3)+4
#define addrLMDA reg.tmp1 = (_RN&~3)-(ssum16(reg.curop&65535)*4)+4
#define addrLMDB reg.tmp1 = (_RN&~3)-(ssum16(reg.curop&65535)*4)

#define addrLMwi _RN += ssum16(reg.curop&65535)*4
#define addrLMwd _RN -= ssum16(reg.curop&65535)*4

//---Opcode defines (Simple loops, really)---------------------------------
// BUGBUG: User-mode LDM assumes linear storage of r0-r15 (safe assumption)

#define opLDM \
    for(int a=0;a<=14;a++) \
    { \
        if(reg.curop&(1<<a)) \
        { \
            reg.r[a]=MMU->rdW(reg.tmp1); \
            reg.tmp1+=4; \
        } \
    } \
    if(reg.curop&0x00008000) \
    { \
        reg.r[15]=MMU->rdW(reg.tmp1)&0xFFFFFFFC; \
        reg.tmp1+=4; \
    } \
    return 3

#define opSTM \
    for(int a=0;a<=15;a++) \
    { \
        if(reg.curop&(1<<a)) \
        { \
            MMU->wrW(reg.tmp1,reg.r[a]); \
            reg.tmp1+=4; \
        } \
    } \
    return 2

#define opLDMu \
    u32 curmode = reg.curmode; \
    u32 *userregs = &reg.r0; \
    modeSwitch(curmode, MODE_USR); \
    modeSwitch(MODE_USR, curmode); \
    for(int a=0;a<=14;a++) \
    { \
	if(reg.curop&(1<<a)) \
	{ \
	    userregs[a] = MMU->rdW(reg.tmp1); \
	    reg.tmp1+=4; \
	} \
    } \
    return 3

#define opSTMu \
    u32 curmode = reg.curmode; \
    u32 *userregs = &reg.r0; \
    modeSwitch(curmode, MODE_USR); \
    modeSwitch(MODE_USR, curmode); \
    for(int a=0;a<=15;a++) \
    { \
	if(reg.curop&(1<<a)) \
	{ \
	    MMU->wrW(reg.tmp1, userregs[a]); \
	    reg.tmp1+=4; \
	} \
    } \
    return 2

// Just as with the one-value load/store, once the opcodes are defined,
// it's a matter of plugging defines together.

OPC opLDMIA() { addrLMIA; opLDM; }
OPC opLDMIB() { addrLMIB; opLDM; }
OPC opLDMDA() { addrLMDA; opLDM; }
OPC opLDMDB() { addrLMDB; opLDM; }

OPC opLDMIAw() { addrLMIA; addrLMwi; opLDM; }
OPC opLDMIBw() { addrLMIB; addrLMwi; opLDM; }
OPC opLDMDAw() { addrLMDA; addrLMwd; opLDM; }
OPC opLDMDBw() { addrLMDB; addrLMwd; opLDM; }

OPC opSTMIA() { addrLMIA; opSTM; }
OPC opSTMIB() { addrLMIB; opSTM; }
OPC opSTMDA() { addrLMDA; opSTM; }
OPC opSTMDB() { addrLMDB; opSTM; }

OPC opSTMIAw() { addrLMIA; addrLMwi; opSTM; }
OPC opSTMIBw() { addrLMIB; addrLMwi; opSTM; }
OPC opSTMDAw() { addrLMDA; addrLMwd; opSTM; }
OPC opSTMDBw() { addrLMDB; addrLMwd; opSTM; }

OPC opLDMIAu() { addrLMIA; opLDMu; }
OPC opLDMIBu() { addrLMIB; opLDMu; }
OPC opLDMDAu() { addrLMDA; opLDMu; }
OPC opLDMDBu() { addrLMDB; opLDMu; }

OPC opLDMIAuw() { addrLMIA; addrLMwi; opLDMu; }
OPC opLDMIBuw() { addrLMIB; addrLMwi; opLDMu; }
OPC opLDMDAuw() { addrLMDA; addrLMwd; opLDMu; }
OPC opLDMDBuw() { addrLMDB; addrLMwd; opLDMu; }

OPC opSTMIAu() { addrLMIA; opSTMu; }
OPC opSTMIBu() { addrLMIB; opSTMu; }
OPC opSTMDAu() { addrLMDA; opSTMu; }
OPC opSTMDBu() { addrLMDB; opSTMu; }

OPC opSTMIAuw() { addrLMIA; addrLMwi; opSTMu; }
OPC opSTMIBuw() { addrLMIB; addrLMwi; opSTMu; }
OPC opSTMDAuw() { addrLMDA; addrLMwd; opSTMu; }
OPC opSTMDBuw() { addrLMDB; addrLMwd; opSTMu; }

/*** EOF: arm-ls.cpp *****************************************************/

