/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM7TDMI core: Opcode lookup table [arm7tdmi-olut.cpp]         *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#ifdef ARM7TDMI_X86

#include "arm7tdmi.h"

//---The 4096-entry opcode table-------------------------------------------
// Addressed on bits [27-20][7-4] of the opcode
// No coprocessor support exists in ARM7TDMI, hence the opcodes are UNI.
// Opcodes undefined in ARMv4 but replaced in ARMv5 are marked.

const ARM7TDMI::opfptr ARM7TDMI::XOLUT[4096]={
    /*0x00*/
    xopANDlli      ,
    xopANDllr      ,
    xopANDlri      ,
    xopANDlrr      ,
    xopANDari      ,
    xopANDarr      ,
    xopANDrri      ,
    xopANDrrr      ,
    xopANDlli      ,
    xopMUL         ,
    xopANDlri      ,
    xopSTRHptrm    ,
    xopANDari      ,
    xopUND,     // Defined in ARM5
    xopANDrri      ,
    xopUND,     // Defined in ARM5

    /*0x01*/
    xopANDSlli     ,
    xopANDSllr     ,
    xopANDSlri     ,
    xopANDSlrr     ,
    xopANDSari     ,
    xopANDSarr     ,
    xopANDSrri     ,
    xopANDSrrr     ,
    xopANDSlli     ,
    xopMULS        ,
    xopANDSlri     ,
    xopLDRHptrm    ,
    xopANDSari     ,
    xopLDRSBptrm   ,
    xopANDSrri     ,
    xopLDRSHptrm   ,

    /*0x02*/
    xopEORlli      ,
    xopEORllr      ,
    xopEORlri      ,
    xopEORlrr      ,
    xopEORari      ,
    xopEORarr      ,
    xopEORrri      ,
    xopEORrrr      ,
    xopEORlli      ,
    xopMLA         ,
    xopEORlri      ,
    xopSTRHptrm    ,
    xopEORari      ,
    xopUND,     // Defined in ARM5
    xopEORrri      ,
    xopUND,     // Defined in ARM5

    /*0x03*/
    xopEORSlli     ,
    xopEORSllr     ,
    xopEORSlri     ,
    xopEORSlrr     ,
    xopEORSari     ,
    xopEORSarr     ,
    xopEORSrri     ,
    xopEORSrrr     ,
    xopEORSlli     ,
    xopMLAS        ,
    xopEORSlri     ,
    xopLDRHptrm    ,
    xopEORSari     ,
    xopLDRSBptrm   ,
    xopEORSrri     ,
    xopLDRSHptrm   ,

    /*0x04*/
    xopSUBlli      ,
    xopSUBllr      ,
    xopSUBlri      ,
    xopSUBlrr      ,
    xopSUBari      ,
    xopSUBarr      ,
    xopSUBrri      ,
    xopSUBrrr      ,
    xopSUBlli      ,
    xopUNP         ,  // Unallocated
    xopSUBlri      ,
    xopSTRHptim    ,
    xopSUBari      ,
    xopUND,     // Defined in ARM5
    xopSUBrri      ,
    xopUND,     // Defined in ARM5

    /*0x05*/
    xopSUBSlli     ,
    xopSUBSllr     ,
    xopSUBSlri     ,
    xopSUBSlrr     ,
    xopSUBSari     ,
    xopSUBSarr     ,
    xopSUBSrri     ,
    xopSUBSrrr     ,
    xopSUBSlli     ,
    xopUNL         ,  // Unallocated [reindexed]
    xopSUBSlri     ,
    xopLDRHptim    ,
    xopSUBSari     ,
    xopLDRSBptim   ,
    xopSUBSrri     ,
    xopLDRSHptim   ,

    /*0x06*/
    xopRSBlli      ,
    xopRSBllr      ,
    xopRSBlri      ,
    xopRSBlrr      ,
    xopRSBari      ,
    xopRSBarr      ,
    xopRSBrri      ,
    xopRSBrrr      ,
    xopRSBlli      ,
    xopUNP         ,  // Unallocated
    xopRSBlri      ,
    xopSTRHptim    ,
    xopRSBari      ,
    xopUND,     // Defined in ARM5
    xopRSBrri      ,
    xopUND,     // Defined in ARM5

    /*0x07*/
    xopRSBSlli     ,
    xopRSBSllr     ,
    xopRSBSlri     ,
    xopRSBSlrr     ,
    xopRSBSari     ,
    xopRSBSarr     ,
    xopRSBSrri     ,
    xopRSBSrrr     ,
    xopRSBSlli     ,
    xopUNP         ,  // Unallocated
    xopRSBSlri     ,
    xopLDRHptim    ,
    xopRSBSari     ,
    xopLDRSBptim   ,
    xopRSBSrri     ,
    xopLDRSHptim   ,

    /*0x08*/
    xopADDlli      ,
    xopADDllr      ,
    xopADDlri      ,
    xopADDlrr      ,
    xopADDari      ,
    xopADDarr      ,
    xopADDrri      ,
    xopADDrrr      ,
    xopADDlli      ,
    xopUMULL       ,
    xopADDlri      ,
    xopSTRHptrp    ,
    xopADDari      ,
    xopUND,     // Defined in ARM5
    xopADDrri      ,
    xopUND,     // Defined in ARM5

    /*0x09*/
    xopADDSlli     ,
    xopADDSllr     ,
    xopADDSlri     ,
    xopADDSlrr     ,
    xopADDSari     ,
    xopADDSarr     ,
    xopADDSrri     ,
    xopADDSrrr     ,
    xopADDSlli     ,
    xopUMULLS      ,
    xopADDSlri     ,
    xopLDRHptrp    ,
    xopADDSari     ,
    xopLDRSBptrp   ,
    xopADDSrri     ,
    xopLDRSHptrp   ,

    /*0x0A*/
    xopADClli      ,
    xopADCllr      ,
    xopADClri      ,
    xopADClrr      ,
    xopADCari      ,
    xopADCarr      ,
    xopADCrri      ,
    xopADCrrr      ,
    xopADClli      ,
    xopUMLAL       ,
    xopADClri      ,
    xopSTRHptrp    ,
    xopADCari      ,
    xopUND,     // Defined in ARM5
    xopADCrri      ,
    xopUND,     // Defined in ARM5

    /*0x0B*/
    xopADCSlli     ,
    xopADCSllr     ,
    xopADCSlri     ,
    xopADCSlrr     ,
    xopADCSari     ,
    xopADCSarr     ,
    xopADCSrri     ,
    xopADCSrrr     ,
    xopADCSlli     ,
    xopUMLALS      ,
    xopADCSlri     ,
    xopLDRHptrp    ,
    xopADCSari     ,
    xopLDRSBptrp   ,
    xopADCSrri     ,
    xopLDRSHptrp   ,

    /*0x0C*/
    xopSBClli      ,
    xopSBCllr      ,
    xopSBClri      ,
    xopSBClrr      ,
    xopSBCari      ,
    xopSBCarr      ,
    xopSBCrri      ,
    xopSBCrrr      ,
    xopSBClli      ,
    xopSMULL       ,
    xopSBClri      ,
    xopSTRHptip    ,
    xopSBCari      ,
    xopUND,     // Defined in ARM5
    xopSBCrri      ,
    xopUND,     // Defined in ARM5

    /*0x0D*/
    xopSBCSlli     ,
    xopSBCSllr     ,
    xopSBCSlri     ,
    xopSBCSlrr     ,
    xopSBCSari     ,
    xopSBCSarr     ,
    xopSBCSrri     ,
    xopSBCSrrr     ,
    xopSBCSlli     ,
    xopSMULLS      ,
    xopSBCSlri     ,
    xopLDRHptip    ,
    xopSBCSari     ,
    xopLDRSBptip   ,
    xopSBCSrri     ,
    xopLDRSHptip   ,

    /*0x0E*/
    xopRSClli      ,
    xopRSCllr      ,
    xopRSClri      ,
    xopRSClrr      ,
    xopRSCari      ,
    xopRSCarr      ,
    xopRSCrri      ,
    xopRSCrrr      ,
    xopRSClli      ,
    xopSMLAL       ,
    xopRSClri      ,
    xopSTRHptip    ,
    xopRSCari      ,
    xopUND,     // Defined in ARM5
    xopRSCrri      ,
    xopUND,     // Defined in ARM5

    /*0x0F*/
    xopRSCSlli     ,
    xopRSCSllr     ,
    xopRSCSlri     ,
    xopRSCSlrr     ,
    xopRSCSari     ,
    xopRSCSarr     ,
    xopRSCSrri     ,
    xopRSCSrrr     ,
    xopRSCSlli     ,
    xopSMLALS      ,
    xopRSCSlri     ,
    xopLDRHptip    ,
    xopRSCSari     ,
    xopLDRSBptip   ,
    xopRSCSrri     ,
    xopLDRSHptip   ,

    /*0x10*/
    xopMRSrc       ,
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUND         ,  // Defined in ARM5
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUND         ,  // Defined in ARM5
    xopSWP         ,
    xopUND         ,  // Defined in ARM5
    xopSTRHofrm    ,
    xopUND         ,  // Defined in ARM5
    xopUND         ,  // Defined in ARM5
    xopUND         ,  // Defined in ARM5
    xopUND         ,  // Defined in ARM5

    /*0x11*/
    xopTSTSlli     ,
    xopTSTSllr     ,
    xopTSTSlri     ,
    xopTSTSlrr     ,
    xopTSTSari     ,
    xopTSTSarr     ,
    xopTSTSrri     ,
    xopTSTSrrr     ,
    xopTSTSlli     ,
    xopUNP         ,  // Unallocated
    xopTSTSlri     ,
    xopLDRHofrm    ,
    xopTSTSari     ,
    xopLDRSBofrm   ,
    xopTSTSrri     ,
    xopLDRSHofrm   ,

    /*0x12*/
    xopMSRrc       ,
    xopBX          ,
    xopUNP         ,  // Unallocated
    xopUND         ,  // Defined in ARM5T
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Defined in ARM5E!
    xopUNP         ,  // Unallocated
    xopUND        ,   // Defined in ARM5
    xopUNP         ,  // Defined in ARM5E!
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Defined in ARM5E!
    xopSTRHprrm    ,
    xopUNP         ,  // Defined in ARM5E!
    xopUND,     // Defined in ARM5
    xopUNP         ,  // Defined in ARM5E!
    xopUND,     // Defined in ARM5

    /*0x13*/
    xopTEQSlli     ,
    xopTEQSllr     ,
    xopTEQSlri     ,
    xopTEQSlrr     ,
    xopTEQSari     ,
    xopTEQSarr     ,
    xopTEQSrri     ,
    xopTEQSrrr     ,
    xopTEQSlli     ,
    xopUNP         ,  // Unallocated
    xopTEQSlri     ,
    xopLDRHprrm    ,
    xopTEQSari     ,
    xopLDRSBprrm   ,
    xopTEQSrri     ,
    xopLDRSHprrm   ,

    /*0x14*/
    xopMRSrs       ,
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Defined in ARM5E!
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Defined in ARM5E!
    xopSWPB        ,
    xopUNP         ,  // Defined in ARM5E!
    xopSTRHofim    ,
    xopUNP         ,  // Defined in ARM5E!
    xopUND,     // Defined in ARM5
    xopUNP         ,  // Defined in ARM5E!
    xopUND,     // Defined in ARM5

    /*0x15*/
    xopCMPSlli     ,
    xopCMPSllr     ,
    xopCMPSlri     ,
    xopCMPSlrr     ,
    xopCMPSari     ,
    xopCMPSarr     ,
    xopCMPSrri     ,
    xopCMPSrrr     ,
    xopCMPSlli     ,
    xopUNP         ,  // Unallocated
    xopCMPSlri     ,
    xopLDRHofim    ,
    xopCMPSari     ,
    xopLDRSBofim   ,
    xopCMPSrri     ,
    xopLDRSHofim   ,

    /*0x16*/
    xopMSRrs       ,
    xopUND         ,  // Defined in ARM5
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Defined in ARM5E!
    xopUNP         ,  // Unallocated
    xopUNP         ,  // Unallocated
    xopUND,     // Defined in ARM5
    xopUNP         ,  // Unallocated
    xopUND,     // Defined in ARM5
    xopSTRHprim    ,
    xopUND,     // Defined in ARM5
    xopUND,     // Defined in ARM5
    xopUND,     // Defined in ARM5
    xopUND,     // Defined in ARM5

    /*0x17*/
    xopCMNSlli     ,
    xopCMNSllr     ,
    xopCMNSlri     ,
    xopCMNSlrr     ,
    xopCMNSari     ,
    xopCMNSarr     ,
    xopCMNSrri     ,
    xopCMNSrrr     ,
    xopCMNSlli     ,
    xopUNP         ,  // Unallocated
    xopCMNSlri     ,
    xopLDRHprim    ,
    xopCMNSari     ,
    xopLDRSBprim   ,
    xopCMNSrri     ,
    xopLDRSHprim   ,

    /*0x18*/
    xopORRlli      ,
    xopORRllr      ,
    xopORRlri      ,
    xopORRlrr      ,
    xopORRari      ,
    xopORRarr      ,
    xopORRrri      ,
    xopORRrrr      ,
    xopORRlli      ,
    xopUNP         ,  // Unallocated
    xopORRlri      ,
    xopSTRHofrp    ,
    xopORRari      ,
    xopUND,     // Defined in ARM5
    xopORRrri      ,
    xopUND,     // Defined in ARM5

    /*0x19*/
    xopORRSlli     ,
    xopORRSllr     ,
    xopORRSlri     ,
    xopORRSlrr     ,
    xopORRSari     ,
    xopORRSarr     ,
    xopORRSrri     ,
    xopORRSrrr     ,
    xopORRSlli     ,
    xopUNP         ,  // Unallocated
    xopORRSlri     ,
    xopLDRHofrp    ,
    xopORRSari     ,
    xopLDRSBofrp   ,
    xopORRSrri     ,
    xopLDRSHofrp   ,

    /*0x1A*/
    xopMOVlli      ,
    xopMOVllr      ,
    xopMOVlri      ,
    xopMOVlrr      ,
    xopMOVari      ,
    xopMOVarr      ,
    xopMOVrri      ,
    xopMOVrrr      ,
    xopMOVlli      ,
    xopUNP         ,  // Unallocated
    xopMOVlri      ,
    xopSTRHprrp    ,
    xopMOVari      ,
    xopUND,     // Defined in ARM5
    xopMOVrri      ,
    xopUND,     // Defined in ARM5

    /*0x1B*/
    xopMOVSlli     ,
    xopMOVSllr     ,
    xopMOVSlri     ,
    xopMOVSlrr     ,
    xopMOVSari     ,
    xopMOVSarr     ,
    xopMOVSrri     ,
    xopMOVSrrr     ,
    xopMOVSlli     ,
    xopUNP         ,  // Unallocated
    xopMOVSlri     ,
    xopLDRHprrp    ,
    xopMOVSari     ,
    xopLDRSBprrp   ,
    xopMOVSrri     ,
    xopLDRSHprrp   ,

    /*0x1C*/
    xopBIClli      ,
    xopBICllr      ,
    xopBIClri      ,
    xopBIClrr      ,
    xopBICari      ,
    xopBICarr      ,
    xopBICrri      ,
    xopBICrrr      ,
    xopBIClli      ,
    xopUNP         ,  // Unallocated
    xopBIClri      ,
    xopSTRHofip    ,
    xopBICari      ,
    xopUND,     // Defined in ARM5
    xopBICrri      ,
    xopUND,     // Defined in ARM5

    /*0x1D*/
    xopBICSlli     ,
    xopBICSllr     ,
    xopBICSlri     ,
    xopBICSlrr     ,
    xopBICSari     ,
    xopBICSarr     ,
    xopBICSrri     ,
    xopBICSrrr     ,
    xopBICSlli     ,
    xopUNP         ,  // Unallocated
    xopBICSlri     ,
    xopLDRHofip    ,
    xopBICSari     ,
    xopLDRSBofip   ,
    xopBICSrri     ,
    xopLDRSHofip   ,

    /*0x1E*/
    xopMVNlli      ,
    xopMVNllr      ,
    xopMVNlri      ,
    xopMVNlrr      ,
    xopMVNari      ,
    xopMVNarr      ,
    xopMVNrri      ,
    xopMVNrrr      ,
    xopMVNlli      ,
    xopUNP         ,  // Unallocated
    xopMVNlri      ,
    xopSTRHprip    ,
    xopMVNari      ,
    xopUND,     // Defined in ARM5
    xopMVNrri      ,
    xopUND,     // Defined in ARM5

    /*0x1F*/
    xopMVNSlli     ,
    xopMVNSllr     ,
    xopMVNSlri     ,
    xopMVNSlrr     ,
    xopMVNSari     ,
    xopMVNSarr     ,
    xopMVNSrri     ,
    xopMVNSrrr     ,
    xopMVNSlli     ,
    xopUNP         ,  // Unallocated
    xopMVNSlri     ,
    xopLDRHprip    ,
    xopMVNSari     ,
    xopLDRSBprip   ,
    xopMVNSrri     ,
    xopLDRSHprip   ,

    /*0x20*/
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,
    xopANDimm      ,

    /*0x21*/
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,
    xopANDSimm     ,

    /*0x22*/
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,
    xopEORimm      ,

    /*0x23*/
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,
    xopEORSimm     ,

    /*0x24*/
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,
    xopSUBimm      ,

    /*0x25*/
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,
    xopSUBSimm     ,

    /*0x26*/
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,
    xopRSBimm      ,

    /*0x27*/
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,
    xopRSBSimm     ,

    /*0x28*/
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,
    xopADDimm      ,

    /*0x29*/
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,
    xopADDSimm     ,

    /*0x2A*/
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,
    xopADCimm      ,

    /*0x2B*/
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,
    xopADCSimm     ,

    /*0x2C*/
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,
    xopSBCimm      ,

    /*0x2D*/
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,
    xopSBCSimm     ,

    /*0x2E*/
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,
    xopRSCimm      ,

    /*0x2F*/
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,
    xopRSCSimm     ,

    /*0x30*/
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,

    /*0x31*/
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,
    xopTSTSimm     ,

    /*0x32*/
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,
    xopMSRic       ,

    /*0x33*/
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,
    xopTEQSimm     ,

    /*0x34*/
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,
    xopUND         ,

    /*0x35*/
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,
    xopCMPSimm     ,

    /*0x36*/
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,
    xopMSRis       ,

    /*0x37*/
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,
    xopCMNSimm     ,

    /*0x38*/
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,
    xopORRimm      ,

    /*0x39*/
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,
    xopORRSimm     ,

    /*0x3A*/
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,
    xopMOVimm      ,

    /*0x3B*/
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,
    xopMOVSimm     ,

    /*0x3C*/
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,
    xopBICimm      ,

    /*0x3D*/
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,
    xopBICSimm     ,

    /*0x3E*/
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,
    xopMVNimm      ,

    /*0x3F*/
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,
    xopMVNSimm     ,

    /*0x40*/
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,
    xopSTRptim     ,

    /*0x41*/
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,
    xopLDRptim     ,

    /*0x42*/
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,
    xopSTRTptim    ,

    /*0x43*/
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,
    xopLDRTptim    ,

    /*0x44*/
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,
    xopSTRBptim    ,

    /*0x45*/
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,
    xopLDRBptim    ,

    /*0x46*/
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,
    xopSTRBTptim   ,

    /*0x47*/
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,
    xopLDRBTptim   ,

    /*0x48*/
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,
    xopSTRptip     ,

    /*0x49*/
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,
    xopLDRptip     ,

    /*0x4A*/
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,
    xopSTRTptip    ,

    /*0x4B*/
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,
    xopLDRTptip    ,

    /*0x4C*/
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,
    xopSTRBptip    ,

    /*0x4D*/
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,
    xopLDRBptip    ,

    /*0x4E*/
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,
    xopSTRBTptip   ,

    /*0x4F*/
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,
    xopLDRBTptip   ,

    /*0x50*/
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,
    xopSTRofim     ,

    /*0x51*/
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,
    xopLDRofim     ,

    /*0x52*/
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,
    xopSTRprim     ,

    /*0x53*/
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,
    xopLDRprim     ,

    /*0x54*/
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,
    xopSTRBofim    ,

    /*0x55*/
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,
    xopLDRBofim    ,

    /*0x56*/
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,
    xopSTRBprim    ,

    /*0x57*/
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,
    xopLDRBprim    ,

    /*0x58*/
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,
    xopSTRofip     ,

    /*0x59*/
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,
    xopLDRofip     ,

    /*0x5A*/
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,
    xopSTRprip     ,

    /*0x5B*/
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,
    xopLDRprip     ,

    /*0x5C*/
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,
    xopSTRBofip    ,

    /*0x5D*/
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,
    xopLDRBofip    ,

    /*0x5E*/
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,
    xopSTRBprip    ,

    /*0x5F*/
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,
    xopLDRBprip    ,

    /*0x60*/
    xopSTRptrmll   ,
    xopUND         ,
    xopSTRptrmlr   ,
    xopUND         ,
    xopSTRptrmar   ,
    xopUND         ,
    xopSTRptrmrr   ,
    xopUND         ,
    xopSTRptrmll   ,
    xopUND         ,
    xopSTRptrmlr   ,
    xopUND         ,
    xopSTRptrmar   ,
    xopUND         ,
    xopSTRptrmrr   ,
    xopUND         ,

    /*0x61*/
    xopLDRptrmll   ,
    xopUND         ,
    xopLDRptrmlr   ,
    xopUND         ,
    xopLDRptrmar   ,
    xopUND         ,
    xopLDRptrmrr   ,
    xopUND         ,
    xopLDRptrmll   ,
    xopUND         ,
    xopLDRptrmlr   ,
    xopUND         ,
    xopLDRptrmar   ,
    xopUND         ,
    xopLDRptrmrr   ,
    xopUND         ,

    /*0x62*/
    xopSTRTptrmll  ,
    xopUND         ,
    xopSTRTptrmlr  ,
    xopUND         ,
    xopSTRTptrmar  ,
    xopUND         ,
    xopSTRTptrmrr  ,
    xopUND         ,
    xopSTRTptrmll  ,
    xopUND         ,
    xopSTRTptrmlr  ,
    xopUND         ,
    xopSTRTptrmar  ,
    xopUND         ,
    xopSTRTptrmrr  ,
    xopUND         ,

    /*0x63*/
    xopLDRTptrmll  ,
    xopUND         ,
    xopLDRTptrmlr  ,
    xopUND         ,
    xopLDRTptrmar  ,
    xopUND         ,
    xopLDRTptrmrr  ,
    xopUND         ,
    xopLDRTptrmll  ,
    xopUND         ,
    xopLDRTptrmlr  ,
    xopUND         ,
    xopLDRTptrmar  ,
    xopUND         ,
    xopLDRTptrmrr  ,
    xopUND         ,

    /*0x64*/
    xopSTRBptrmll  ,
    xopUND         ,
    xopSTRBptrmlr  ,
    xopUND         ,
    xopSTRBptrmar  ,
    xopUND         ,
    xopSTRBptrmrr  ,
    xopUND         ,
    xopSTRBptrmll  ,
    xopUND         ,
    xopSTRBptrmlr  ,
    xopUND         ,
    xopSTRBptrmar  ,
    xopUND         ,
    xopSTRBptrmrr  ,
    xopUND         ,

    /*0x65*/
    xopLDRBptrmll  ,
    xopUND         ,
    xopLDRBptrmlr  ,
    xopUND         ,
    xopLDRBptrmar  ,
    xopUND         ,
    xopLDRBptrmrr  ,
    xopUND         ,
    xopLDRBptrmll  ,
    xopUND         ,
    xopLDRBptrmlr  ,
    xopUND         ,
    xopLDRBptrmar  ,
    xopUND         ,
    xopLDRBptrmrr  ,
    xopUND         ,

    /*0x66*/
    xopSTRBTptrmll ,
    xopUND         ,
    xopSTRBTptrmlr ,
    xopUND         ,
    xopSTRBTptrmar ,
    xopUND         ,
    xopSTRBTptrmrr ,
    xopUND         ,
    xopSTRBTptrmll ,
    xopUND         ,
    xopSTRBTptrmlr ,
    xopUND         ,
    xopSTRBTptrmar ,
    xopUND         ,
    xopSTRBTptrmrr ,
    xopUND         ,

    /*0x67*/
    xopLDRBTptrmll ,
    xopUND         ,
    xopLDRBTptrmlr ,
    xopUND         ,
    xopLDRBTptrmar ,
    xopUND         ,
    xopLDRBTptrmrr ,
    xopUND         ,
    xopLDRBTptrmll ,
    xopUND         ,
    xopLDRBTptrmlr ,
    xopUND         ,
    xopLDRBTptrmar ,
    xopUND         ,
    xopLDRBTptrmrr ,
    xopUND         ,

    /*0x68*/
    xopSTRptrpll   ,
    xopUND         ,
    xopSTRptrplr   ,
    xopUND         ,
    xopSTRptrpar   ,
    xopUND         ,
    xopSTRptrprr   ,
    xopUND         ,
    xopSTRptrpll   ,
    xopUND         ,
    xopSTRptrplr   ,
    xopUND         ,
    xopSTRptrpar   ,
    xopUND         ,
    xopSTRptrprr   ,
    xopUND         ,

    /*0x69*/
    xopLDRptrpll   ,
    xopUND         ,
    xopLDRptrplr   ,
    xopUND         ,
    xopLDRptrpar   ,
    xopUND         ,
    xopLDRptrprr   ,
    xopUND         ,
    xopLDRptrpll   ,
    xopUND         ,
    xopLDRptrplr   ,
    xopUND         ,
    xopLDRptrpar   ,
    xopUND         ,
    xopLDRptrprr   ,
    xopUND         ,

    /*0x6A*/
    xopSTRTptrpll  ,
    xopUND         ,
    xopSTRTptrplr  ,
    xopUND         ,
    xopSTRTptrpar  ,
    xopUND         ,
    xopSTRTptrprr  ,
    xopUND         ,
    xopSTRTptrpll  ,
    xopUND         ,
    xopSTRTptrplr  ,
    xopUND         ,
    xopSTRTptrpar  ,
    xopUND         ,
    xopSTRTptrprr  ,
    xopUND         ,

    /*0x6B*/
    xopLDRTptrpll  ,
    xopUND         ,
    xopLDRTptrplr  ,
    xopUND         ,
    xopLDRTptrpar  ,
    xopUND         ,
    xopLDRTptrprr  ,
    xopUND         ,
    xopLDRTptrpll  ,
    xopUND         ,
    xopLDRTptrplr  ,
    xopUND         ,
    xopLDRTptrpar  ,
    xopUND         ,
    xopLDRTptrprr  ,
    xopUND         ,

    /*0x6C*/
    xopSTRBptrpll  ,
    xopUND         ,
    xopSTRBptrplr  ,
    xopUND         ,
    xopSTRBptrpar  ,
    xopUND         ,
    xopSTRBptrprr  ,
    xopUND         ,
    xopSTRBptrpll  ,
    xopUND         ,
    xopSTRBptrplr  ,
    xopUND         ,
    xopSTRBptrpar  ,
    xopUND         ,
    xopSTRBptrprr  ,
    xopUND         ,

    /*0x6D*/
    xopLDRBptrpll  ,
    xopUND         ,
    xopLDRBptrplr  ,
    xopUND         ,
    xopLDRBptrpar  ,
    xopUND         ,
    xopLDRBptrprr  ,
    xopUND         ,
    xopLDRBptrpll  ,
    xopUND         ,
    xopLDRBptrplr  ,
    xopUND         ,
    xopLDRBptrpar  ,
    xopUND         ,
    xopLDRBptrprr  ,
    xopUND         ,

    /*0x6E*/
    xopSTRBTptrpll ,
    xopUND         ,
    xopSTRBTptrplr ,
    xopUND         ,
    xopSTRBTptrpar ,
    xopUND         ,
    xopSTRBTptrprr ,
    xopUND         ,
    xopSTRBTptrpll ,
    xopUND         ,
    xopSTRBTptrplr ,
    xopUND         ,
    xopSTRBTptrpar ,
    xopUND         ,
    xopSTRBTptrprr ,
    xopUND         ,

    /*0x6F*/
    xopLDRBTptrpll ,
    xopUND         ,
    xopLDRBTptrplr ,
    xopUND         ,
    xopLDRBTptrpar ,
    xopUND         ,
    xopLDRBTptrprr ,
    xopUND         ,
    xopLDRBTptrpll ,
    xopUND         ,
    xopLDRBTptrplr ,
    xopUND         ,
    xopLDRBTptrpar ,
    xopUND         ,
    xopLDRBTptrprr ,
    xopUND         ,

    /*0x70*/
    xopSTRofrmll   ,
    xopUND         ,
    xopSTRofrmlr   ,
    xopUND         ,
    xopSTRofrmar   ,
    xopUND         ,
    xopSTRofrmrr   ,
    xopUND         ,
    xopSTRofrmll   ,
    xopUND         ,
    xopSTRofrmlr   ,
    xopUND         ,
    xopSTRofrmar   ,
    xopUND         ,
    xopSTRofrmrr   ,
    xopUND         ,

    /*0x71*/
    xopLDRofrmll   ,
    xopUND         ,
    xopLDRofrmlr   ,
    xopUND         ,
    xopLDRofrmar   ,
    xopUND         ,
    xopLDRofrmrr   ,
    xopUND         ,
    xopLDRofrmll   ,
    xopUND         ,
    xopLDRofrmlr   ,
    xopUND         ,
    xopLDRofrmar   ,
    xopUND         ,
    xopLDRofrmrr   ,
    xopUND         ,

    /*0x72*/
    xopSTRprrmll   ,
    xopUND         ,
    xopSTRprrmlr   ,
    xopUND         ,
    xopSTRprrmar   ,
    xopUND         ,
    xopSTRprrmrr   ,
    xopUND         ,
    xopSTRprrmll   ,
    xopUND         ,
    xopSTRprrmlr   ,
    xopUND         ,
    xopSTRprrmar   ,
    xopUND         ,
    xopSTRprrmrr   ,
    xopUND         ,

    /*0x73*/
    xopLDRprrmll   ,
    xopUND         ,
    xopLDRprrmlr   ,
    xopUND         ,
    xopLDRprrmar   ,
    xopUND         ,
    xopLDRprrmrr   ,
    xopUND         ,
    xopLDRprrmll   ,
    xopUND         ,
    xopLDRprrmlr   ,
    xopUND         ,
    xopLDRprrmar   ,
    xopUND         ,
    xopLDRprrmrr   ,
    xopUND         ,

    /*0x74*/
    xopSTRBofrmll  ,
    xopUND         ,
    xopSTRBofrmlr  ,
    xopUND         ,
    xopSTRBofrmar  ,
    xopUND         ,
    xopSTRBofrmrr  ,
    xopUND         ,
    xopSTRBofrmll  ,
    xopUND         ,
    xopSTRBofrmlr  ,
    xopUND         ,
    xopSTRBofrmar  ,
    xopUND         ,
    xopSTRBofrmrr  ,
    xopUND         ,

    /*0x75*/
    xopLDRBofrmll  ,
    xopUND         ,
    xopLDRBofrmlr  ,
    xopUND         ,
    xopLDRBofrmar  ,
    xopUND         ,
    xopLDRBofrmrr  ,
    xopUND         ,
    xopLDRBofrmll  ,
    xopUND         ,
    xopLDRBofrmlr  ,
    xopUND         ,
    xopLDRBofrmar  ,
    xopUND         ,
    xopLDRBofrmrr  ,
    xopUND         ,

    /*0x76*/
    xopSTRBprrmll  ,
    xopUND         ,
    xopSTRBprrmlr  ,
    xopUND         ,
    xopSTRBprrmar  ,
    xopUND         ,
    xopSTRBprrmrr  ,
    xopUND         ,
    xopSTRBprrmll  ,
    xopUND         ,
    xopSTRBprrmlr  ,
    xopUND         ,
    xopSTRBprrmar  ,
    xopUND         ,
    xopSTRBprrmrr  ,
    xopUND         ,

    /*0x77*/
    xopLDRBprrmll  ,
    xopUND         ,
    xopLDRBprrmlr  ,
    xopUND         ,
    xopLDRBprrmar  ,
    xopUND         ,
    xopLDRBprrmrr  ,
    xopUND         ,
    xopLDRBprrmll  ,
    xopUND         ,
    xopLDRBprrmlr  ,
    xopUND         ,
    xopLDRBprrmar  ,
    xopUND         ,
    xopLDRBprrmrr  ,
    xopUND         ,

    /*0x78*/
    xopSTRofrpll   ,
    xopUND         ,
    xopSTRofrplr   ,
    xopUND         ,
    xopSTRofrpar   ,
    xopUND         ,
    xopSTRofrprr   ,
    xopUND         ,
    xopSTRofrpll   ,
    xopUND         ,
    xopSTRofrplr   ,
    xopUND         ,
    xopSTRofrpar   ,
    xopUND         ,
    xopSTRofrprr   ,
    xopUND         ,

    /*0x79*/
    xopLDRofrpll   ,
    xopUND         ,
    xopLDRofrplr   ,
    xopUND         ,
    xopLDRofrpar   ,
    xopUND         ,
    xopLDRofrprr   ,
    xopUND         ,
    xopLDRofrpll   ,
    xopUND         ,
    xopLDRofrplr   ,
    xopUND         ,
    xopLDRofrpar   ,
    xopUND         ,
    xopLDRofrprr   ,
    xopUND         ,

    /*0x7A*/
    xopSTRprrpll   ,
    xopUND         ,
    xopSTRprrplr   ,
    xopUND         ,
    xopSTRprrpar   ,
    xopUND         ,
    xopSTRprrprr   ,
    xopUND         ,
    xopSTRprrpll   ,
    xopUND         ,
    xopSTRprrplr   ,
    xopUND         ,
    xopSTRprrpar   ,
    xopUND         ,
    xopSTRprrprr   ,
    xopUND         ,

    /*0x7B*/
    xopLDRprrpll   ,
    xopUND         ,
    xopLDRprrplr   ,
    xopUND         ,
    xopLDRprrpar   ,
    xopUND         ,
    xopLDRprrprr   ,
    xopUND         ,
    xopLDRprrpll   ,
    xopUND         ,
    xopLDRprrplr   ,
    xopUND         ,
    xopLDRprrpar   ,
    xopUND         ,
    xopLDRprrprr   ,
    xopUND         ,

    /*0x7C*/
    xopSTRBofrpll  ,
    xopUND         ,
    xopSTRBofrplr  ,
    xopUND         ,
    xopSTRBofrpar  ,
    xopUND         ,
    xopSTRBofrprr  ,
    xopUND         ,
    xopSTRBofrpll  ,
    xopUND         ,
    xopSTRBofrplr  ,
    xopUND         ,
    xopSTRBofrpar  ,
    xopUND         ,
    xopSTRBofrprr  ,
    xopUND         ,

    /*0x7D*/
    xopLDRBofrpll  ,
    xopUND         ,
    xopLDRBofrplr  ,
    xopUND         ,
    xopLDRBofrpar  ,
    xopUND         ,
    xopLDRBofrprr  ,
    xopUND         ,
    xopLDRBofrpll  ,
    xopUND         ,
    xopLDRBofrplr  ,
    xopUND         ,
    xopLDRBofrpar  ,
    xopUND         ,
    xopLDRBofrprr  ,
    xopUND         ,

    /*0x7E*/
    xopSTRBprrpll  ,
    xopUND         ,
    xopSTRBprrplr  ,
    xopUND         ,
    xopSTRBprrpar  ,
    xopUND         ,
    xopSTRBprrprr  ,
    xopUND         ,
    xopSTRBprrpll  ,
    xopUND         ,
    xopSTRBprrplr  ,
    xopUND         ,
    xopSTRBprrpar  ,
    xopUND         ,
    xopSTRBprrprr  ,
    xopUND         ,

    /*0x7F*/
    xopLDRBprrpll  ,
    xopUND         ,
    xopLDRBprrplr  ,
    xopUND         ,
    xopLDRBprrpar  ,
    xopUND         ,
    xopLDRBprrprr  ,
    xopUND         ,
    xopLDRBprrpll  ,
    xopUND         ,
    xopLDRBprrplr  ,
    xopUND         ,
    xopLDRBprrpar  ,
    xopUND         ,
    xopLDRBprrprr  ,
    xopUND         ,

    /*0x80*/
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,
    xopSTMDA       ,

    /*0x81*/
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,
    xopLDMDA       ,

    /*0x82*/
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,
    xopSTMDAw      ,

    /*0x83*/
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,
    xopLDMDAw      ,

    /*0x84*/
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,
    xopSTMDAu      ,

    /*0x85*/
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,
    xopLDMDAu      ,

    /*0x86*/
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,
    xopSTMDAuw     ,

    /*0x87*/
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,
    xopLDMDAuw     ,

    /*0x88*/
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,
    xopSTMIA       ,

    /*0x89*/
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,
    xopLDMIA       ,

    /*0x8A*/
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,
    xopSTMIAw      ,

    /*0x8B*/
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,
    xopLDMIAw      ,

    /*0x8C*/
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,
    xopSTMIAu      ,

    /*0x8D*/
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,
    xopLDMIAu      ,

    /*0x8E*/
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,
    xopSTMIAuw     ,

    /*0x8F*/
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,
    xopLDMIAuw     ,

    /*0x90*/
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,
    xopSTMDB       ,

    /*0x91*/
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,
    xopLDMDB       ,

    /*0x92*/
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,
    xopSTMDBw      ,

    /*0x93*/
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,
    xopLDMDBw      ,

    /*0x94*/
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,
    xopSTMDBu      ,

    /*0x95*/
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,
    xopLDMDBu      ,

    /*0x96*/
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,
    xopSTMDBuw     ,

    /*0x97*/
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,
    xopLDMDBuw     ,

    /*0x98*/
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,
    xopSTMIB       ,

    /*0x99*/
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,
    xopLDMIB       ,

    /*0x9A*/
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,
    xopSTMIBw      ,

    /*0x9B*/
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,
    xopLDMIBw      ,

    /*0x9C*/
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,
    xopSTMIBu      ,

    /*0x9D*/
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,
    xopLDMIBu      ,

    /*0x9E*/
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,
    xopSTMIBuw     ,

    /*0x9F*/
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,
    xopLDMIBuw     ,

    /*0xA0*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA1*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA2*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA3*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA4*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA5*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA6*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA7*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA8*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xA9*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xAA*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xAB*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xAC*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xAD*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xAE*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xAF*/
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,
    xopB           ,

    /*0xB0*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB1*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB2*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB3*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB4*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB5*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB6*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB7*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB8*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xB9*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xBA*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xBB*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xBC*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xBD*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xBE*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xBF*/
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,
    xopBL          ,

    /*0xC0*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC1*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC2*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC3*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC4*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC5*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC6*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC7*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC8*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xC9*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xCA*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xCB*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xCC*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xCD*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xCE*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xCF*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD0*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD1*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD2*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD3*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD4*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD5*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD6*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD7*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD8*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xD9*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xDA*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xDB*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xDC*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xDD*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xDE*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xDF*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE0*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE1*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE2*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE3*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE4*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE5*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE6*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE7*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE8*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xE9*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xEA*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xEB*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xEC*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xED*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xEE*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xEF*/
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,
    xopUNI         ,

    /*0xF0*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF1*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF2*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF3*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF4*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF5*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF6*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF7*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF8*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xF9*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xFA*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xFB*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xFC*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xFD*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xFE*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,

    /*0xFF*/
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
    xopSWI         ,
};

#endif

/*** EOF: arm7tdmi-olut.cpp **********************************************/

