/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM7TDMI core: Opcode lookup table [arm7tdmi-olut.cpp]         *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#include "arm7tdmi.h"

//---Conditional execution lookup table------------------------------------
// Addressed on bits 31-28 of the opcode
// Each of these instructions returns 0 or 1 dependent on the flags.

const ARM7TDMI::intfptr ARM7TDMI::condLUT[16]={
    condEQ, condNE, condCS, condCC,
    condMI, condPL, condVS, condVC,
    condHI, condLS, condGE, condLT,
    condGT, condLE, condAL, condNV
};

//---The 4096-entry opcode table-------------------------------------------
// Addressed on bits [27-20][7-4] of the opcode
// No coprocessor support exists in ARM7TDMI, hence the opcodes are UNI.
// Opcodes undefined in ARMv4 but replaced in ARMv5 are marked.

const ARM7TDMI::opfptr ARM7TDMI::OLUT[4096]={
    /*0x00*/
    opANDlli      ,
    opANDllr      ,
    opANDlri      ,
    opANDlrr      ,
    opANDari      ,
    opANDarr      ,
    opANDrri      ,
    opANDrrr      ,
    opANDlli      ,
    opMUL         ,
    opANDlri      ,
    opSTRHptrm    ,
    opANDari      ,
    opUND,     // Defined in ARM5
    opANDrri      ,
    opUND,     // Defined in ARM5

    /*0x01*/
    opANDSlli     ,
    opANDSllr     ,
    opANDSlri     ,
    opANDSlrr     ,
    opANDSari     ,
    opANDSarr     ,
    opANDSrri     ,
    opANDSrrr     ,
    opANDSlli     ,
    opMULS        ,
    opANDSlri     ,
    opLDRHptrm    ,
    opANDSari     ,
    opLDRSBptrm   ,
    opANDSrri     ,
    opLDRSHptrm   ,

    /*0x02*/
    opEORlli      ,
    opEORllr      ,
    opEORlri      ,
    opEORlrr      ,
    opEORari      ,
    opEORarr      ,
    opEORrri      ,
    opEORrrr      ,
    opEORlli      ,
    opMLA         ,
    opEORlri      ,
    opSTRHptrm    ,
    opEORari      ,
    opUND,     // Defined in ARM5
    opEORrri      ,
    opUND,     // Defined in ARM5

    /*0x03*/
    opEORSlli     ,
    opEORSllr     ,
    opEORSlri     ,
    opEORSlrr     ,
    opEORSari     ,
    opEORSarr     ,
    opEORSrri     ,
    opEORSrrr     ,
    opEORSlli     ,
    opMLAS        ,
    opEORSlri     ,
    opLDRHptrm    ,
    opEORSari     ,
    opLDRSBptrm   ,
    opEORSrri     ,
    opLDRSHptrm   ,

    /*0x04*/
    opSUBlli      ,
    opSUBllr      ,
    opSUBlri      ,
    opSUBlrr      ,
    opSUBari      ,
    opSUBarr      ,
    opSUBrri      ,
    opSUBrrr      ,
    opSUBlli      ,
    opUNP         ,  // Unallocated
    opSUBlri      ,
    opSTRHptim    ,
    opSUBari      ,
    opUND,     // Defined in ARM5
    opSUBrri      ,
    opUND,     // Defined in ARM5

    /*0x05*/
    opSUBSlli     ,
    opSUBSllr     ,
    opSUBSlri     ,
    opSUBSlrr     ,
    opSUBSari     ,
    opSUBSarr     ,
    opSUBSrri     ,
    opSUBSrrr     ,
    opSUBSlli     ,
    opUNL         ,  // Unallocated [reindexed]
    opSUBSlri     ,
    opLDRHptim    ,
    opSUBSari     ,
    opLDRSBptim   ,
    opSUBSrri     ,
    opLDRSHptim   ,

    /*0x06*/
    opRSBlli      ,
    opRSBllr      ,
    opRSBlri      ,
    opRSBlrr      ,
    opRSBari      ,
    opRSBarr      ,
    opRSBrri      ,
    opRSBrrr      ,
    opRSBlli      ,
    opUNP         ,  // Unallocated
    opRSBlri      ,
    opSTRHptim    ,
    opRSBari      ,
    opUND,     // Defined in ARM5
    opRSBrri      ,
    opUND,     // Defined in ARM5

    /*0x07*/
    opRSBSlli     ,
    opRSBSllr     ,
    opRSBSlri     ,
    opRSBSlrr     ,
    opRSBSari     ,
    opRSBSarr     ,
    opRSBSrri     ,
    opRSBSrrr     ,
    opRSBSlli     ,
    opUNP         ,  // Unallocated
    opRSBSlri     ,
    opLDRHptim    ,
    opRSBSari     ,
    opLDRSBptim   ,
    opRSBSrri     ,
    opLDRSHptim   ,

    /*0x08*/
    opADDlli      ,
    opADDllr      ,
    opADDlri      ,
    opADDlrr      ,
    opADDari      ,
    opADDarr      ,
    opADDrri      ,
    opADDrrr      ,
    opADDlli      ,
    opUMULL       ,
    opADDlri      ,
    opSTRHptrp    ,
    opADDari      ,
    opUND,     // Defined in ARM5
    opADDrri      ,
    opUND,     // Defined in ARM5

    /*0x09*/
    opADDSlli     ,
    opADDSllr     ,
    opADDSlri     ,
    opADDSlrr     ,
    opADDSari     ,
    opADDSarr     ,
    opADDSrri     ,
    opADDSrrr     ,
    opADDSlli     ,
    opUMULLS      ,
    opADDSlri     ,
    opLDRHptrp    ,
    opADDSari     ,
    opLDRSBptrp   ,
    opADDSrri     ,
    opLDRSHptrp   ,

    /*0x0A*/
    opADClli      ,
    opADCllr      ,
    opADClri      ,
    opADClrr      ,
    opADCari      ,
    opADCarr      ,
    opADCrri      ,
    opADCrrr      ,
    opADClli      ,
    opUMLAL       ,
    opADClri      ,
    opSTRHptrp    ,
    opADCari      ,
    opUND,     // Defined in ARM5
    opADCrri      ,
    opUND,     // Defined in ARM5

    /*0x0B*/
    opADCSlli     ,
    opADCSllr     ,
    opADCSlri     ,
    opADCSlrr     ,
    opADCSari     ,
    opADCSarr     ,
    opADCSrri     ,
    opADCSrrr     ,
    opADCSlli     ,
    opUMLALS      ,
    opADCSlri     ,
    opLDRHptrp    ,
    opADCSari     ,
    opLDRSBptrp   ,
    opADCSrri     ,
    opLDRSHptrp   ,

    /*0x0C*/
    opSBClli      ,
    opSBCllr      ,
    opSBClri      ,
    opSBClrr      ,
    opSBCari      ,
    opSBCarr      ,
    opSBCrri      ,
    opSBCrrr      ,
    opSBClli      ,
    opSMULL       ,
    opSBClri      ,
    opSTRHptip    ,
    opSBCari      ,
    opUND,     // Defined in ARM5
    opSBCrri      ,
    opUND,     // Defined in ARM5

    /*0x0D*/
    opSBCSlli     ,
    opSBCSllr     ,
    opSBCSlri     ,
    opSBCSlrr     ,
    opSBCSari     ,
    opSBCSarr     ,
    opSBCSrri     ,
    opSBCSrrr     ,
    opSBCSlli     ,
    opSMULLS      ,
    opSBCSlri     ,
    opLDRHptip    ,
    opSBCSari     ,
    opLDRSBptip   ,
    opSBCSrri     ,
    opLDRSHptip   ,

    /*0x0E*/
    opRSClli      ,
    opRSCllr      ,
    opRSClri      ,
    opRSClrr      ,
    opRSCari      ,
    opRSCarr      ,
    opRSCrri      ,
    opRSCrrr      ,
    opRSClli      ,
    opSMLAL       ,
    opRSClri      ,
    opSTRHptip    ,
    opRSCari      ,
    opUND,     // Defined in ARM5
    opRSCrri      ,
    opUND,     // Defined in ARM5

    /*0x0F*/
    opRSCSlli     ,
    opRSCSllr     ,
    opRSCSlri     ,
    opRSCSlrr     ,
    opRSCSari     ,
    opRSCSarr     ,
    opRSCSrri     ,
    opRSCSrrr     ,
    opRSCSlli     ,
    opSMLALS      ,
    opRSCSlri     ,
    opLDRHptip    ,
    opRSCSari     ,
    opLDRSBptip   ,
    opRSCSrri     ,
    opLDRSHptip   ,

    /*0x10*/
    opMRSrc       ,
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUND         ,  // Defined in ARM5
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUND         ,  // Defined in ARM5
    opSWP         ,
    opUND         ,  // Defined in ARM5
    opSTRHofrm    ,
    opUND         ,  // Defined in ARM5
    opUND         ,  // Defined in ARM5
    opUND         ,  // Defined in ARM5
    opUND         ,  // Defined in ARM5

    /*0x11*/
    opTSTSlli     ,
    opTSTSllr     ,
    opTSTSlri     ,
    opTSTSlrr     ,
    opTSTSari     ,
    opTSTSarr     ,
    opTSTSrri     ,
    opTSTSrrr     ,
    opTSTSlli     ,
    opUNP         ,  // Unallocated
    opTSTSlri     ,
    opLDRHofrm    ,
    opTSTSari     ,
    opLDRSBofrm   ,
    opTSTSrri     ,
    opLDRSHofrm   ,

    /*0x12*/
    opMSRrc       ,
    opBX          ,
    opUNP         ,  // Unallocated
    opUND         ,  // Defined in ARM5T
    opUNP         ,  // Unallocated
    opUNP         ,  // Defined in ARM5E!
    opUNP         ,  // Unallocated
    opUND        ,   // Defined in ARM5
    opUNP         ,  // Defined in ARM5E!
    opUNP         ,  // Unallocated
    opUNP         ,  // Defined in ARM5E!
    opSTRHprrm    ,
    opUNP         ,  // Defined in ARM5E!
    opUND,     // Defined in ARM5
    opUNP         ,  // Defined in ARM5E!
    opUND,     // Defined in ARM5

    /*0x13*/
    opTEQSlli     ,
    opTEQSllr     ,
    opTEQSlri     ,
    opTEQSlrr     ,
    opTEQSari     ,
    opTEQSarr     ,
    opTEQSrri     ,
    opTEQSrrr     ,
    opTEQSlli     ,
    opUNP         ,  // Unallocated
    opTEQSlri     ,
    opLDRHprrm    ,
    opTEQSari     ,
    opLDRSBprrm   ,
    opTEQSrri     ,
    opLDRSHprrm   ,

    /*0x14*/
    opMRSrs       ,
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Defined in ARM5E!
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Defined in ARM5E!
    opSWPB        ,
    opUNP         ,  // Defined in ARM5E!
    opSTRHofim    ,
    opUNP         ,  // Defined in ARM5E!
    opUND,     // Defined in ARM5
    opUNP         ,  // Defined in ARM5E!
    opUND,     // Defined in ARM5

    /*0x15*/
    opCMPSlli     ,
    opCMPSllr     ,
    opCMPSlri     ,
    opCMPSlrr     ,
    opCMPSari     ,
    opCMPSarr     ,
    opCMPSrri     ,
    opCMPSrrr     ,
    opCMPSlli     ,
    opUNP         ,  // Unallocated
    opCMPSlri     ,
    opLDRHofim    ,
    opCMPSari     ,
    opLDRSBofim   ,
    opCMPSrri     ,
    opLDRSHofim   ,

    /*0x16*/
    opMSRrs       ,
    opUND         ,  // Defined in ARM5
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUNP         ,  // Defined in ARM5E!
    opUNP         ,  // Unallocated
    opUNP         ,  // Unallocated
    opUND,     // Defined in ARM5
    opUNP         ,  // Unallocated
    opUND,     // Defined in ARM5
    opSTRHprim    ,
    opUND,     // Defined in ARM5
    opUND,     // Defined in ARM5
    opUND,     // Defined in ARM5
    opUND,     // Defined in ARM5

    /*0x17*/
    opCMNSlli     ,
    opCMNSllr     ,
    opCMNSlri     ,
    opCMNSlrr     ,
    opCMNSari     ,
    opCMNSarr     ,
    opCMNSrri     ,
    opCMNSrrr     ,
    opCMNSlli     ,
    opUNP         ,  // Unallocated
    opCMNSlri     ,
    opLDRHprim    ,
    opCMNSari     ,
    opLDRSBprim   ,
    opCMNSrri     ,
    opLDRSHprim   ,

    /*0x18*/
    opORRlli      ,
    opORRllr      ,
    opORRlri      ,
    opORRlrr      ,
    opORRari      ,
    opORRarr      ,
    opORRrri      ,
    opORRrrr      ,
    opORRlli      ,
    opUNP         ,  // Unallocated
    opORRlri      ,
    opSTRHofrp    ,
    opORRari      ,
    opUND,     // Defined in ARM5
    opORRrri      ,
    opUND,     // Defined in ARM5

    /*0x19*/
    opORRSlli     ,
    opORRSllr     ,
    opORRSlri     ,
    opORRSlrr     ,
    opORRSari     ,
    opORRSarr     ,
    opORRSrri     ,
    opORRSrrr     ,
    opORRSlli     ,
    opUNP         ,  // Unallocated
    opORRSlri     ,
    opLDRHofrp    ,
    opORRSari     ,
    opLDRSBofrp   ,
    opORRSrri     ,
    opLDRSHofrp   ,

    /*0x1A*/
    opMOVlli      ,
    opMOVllr      ,
    opMOVlri      ,
    opMOVlrr      ,
    opMOVari      ,
    opMOVarr      ,
    opMOVrri      ,
    opMOVrrr      ,
    opMOVlli      ,
    opUNP         ,  // Unallocated
    opMOVlri      ,
    opSTRHprrp    ,
    opMOVari      ,
    opUND,     // Defined in ARM5
    opMOVrri      ,
    opUND,     // Defined in ARM5

    /*0x1B*/
    opMOVSlli     ,
    opMOVSllr     ,
    opMOVSlri     ,
    opMOVSlrr     ,
    opMOVSari     ,
    opMOVSarr     ,
    opMOVSrri     ,
    opMOVSrrr     ,
    opMOVSlli     ,
    opUNP         ,  // Unallocated
    opMOVSlri     ,
    opLDRHprrp    ,
    opMOVSari     ,
    opLDRSBprrp   ,
    opMOVSrri     ,
    opLDRSHprrp   ,

    /*0x1C*/
    opBIClli      ,
    opBICllr      ,
    opBIClri      ,
    opBIClrr      ,
    opBICari      ,
    opBICarr      ,
    opBICrri      ,
    opBICrrr      ,
    opBIClli      ,
    opUNP         ,  // Unallocated
    opBIClri      ,
    opSTRHofip    ,
    opBICari      ,
    opUND,     // Defined in ARM5
    opBICrri      ,
    opUND,     // Defined in ARM5

    /*0x1D*/
    opBICSlli     ,
    opBICSllr     ,
    opBICSlri     ,
    opBICSlrr     ,
    opBICSari     ,
    opBICSarr     ,
    opBICSrri     ,
    opBICSrrr     ,
    opBICSlli     ,
    opUNP         ,  // Unallocated
    opBICSlri     ,
    opLDRHofip    ,
    opBICSari     ,
    opLDRSBofip   ,
    opBICSrri     ,
    opLDRSHofip   ,

    /*0x1E*/
    opMVNlli      ,
    opMVNllr      ,
    opMVNlri      ,
    opMVNlrr      ,
    opMVNari      ,
    opMVNarr      ,
    opMVNrri      ,
    opMVNrrr      ,
    opMVNlli      ,
    opUNP         ,  // Unallocated
    opMVNlri      ,
    opSTRHprip    ,
    opMVNari      ,
    opUND,     // Defined in ARM5
    opMVNrri      ,
    opUND,     // Defined in ARM5

    /*0x1F*/
    opMVNSlli     ,
    opMVNSllr     ,
    opMVNSlri     ,
    opMVNSlrr     ,
    opMVNSari     ,
    opMVNSarr     ,
    opMVNSrri     ,
    opMVNSrrr     ,
    opMVNSlli     ,
    opUNP         ,  // Unallocated
    opMVNSlri     ,
    opLDRHprip    ,
    opMVNSari     ,
    opLDRSBprip   ,
    opMVNSrri     ,
    opLDRSHprip   ,

    /*0x20*/
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,
    opANDimm      ,

    /*0x21*/
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,
    opANDSimm     ,

    /*0x22*/
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,
    opEORimm      ,

    /*0x23*/
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,
    opEORSimm     ,

    /*0x24*/
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,
    opSUBimm      ,

    /*0x25*/
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,
    opSUBSimm     ,

    /*0x26*/
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,
    opRSBimm      ,

    /*0x27*/
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,
    opRSBSimm     ,

    /*0x28*/
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,
    opADDimm      ,

    /*0x29*/
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,
    opADDSimm     ,

    /*0x2A*/
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,
    opADCimm      ,

    /*0x2B*/
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,
    opADCSimm     ,

    /*0x2C*/
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,
    opSBCimm      ,

    /*0x2D*/
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,
    opSBCSimm     ,

    /*0x2E*/
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,
    opRSCimm      ,

    /*0x2F*/
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,
    opRSCSimm     ,

    /*0x30*/
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,

    /*0x31*/
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,
    opTSTSimm     ,

    /*0x32*/
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,
    opMSRic       ,

    /*0x33*/
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,
    opTEQSimm     ,

    /*0x34*/
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,
    opUND         ,

    /*0x35*/
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,
    opCMPSimm     ,

    /*0x36*/
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,
    opMSRis       ,

    /*0x37*/
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,
    opCMNSimm     ,

    /*0x38*/
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,
    opORRimm      ,

    /*0x39*/
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,
    opORRSimm     ,

    /*0x3A*/
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,
    opMOVimm      ,

    /*0x3B*/
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,
    opMOVSimm     ,

    /*0x3C*/
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,
    opBICimm      ,

    /*0x3D*/
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,
    opBICSimm     ,

    /*0x3E*/
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,
    opMVNimm      ,

    /*0x3F*/
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,
    opMVNSimm     ,

    /*0x40*/
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,
    opSTRptim     ,

    /*0x41*/
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,
    opLDRptim     ,

    /*0x42*/
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,
    opSTRTptim    ,

    /*0x43*/
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,
    opLDRTptim    ,

    /*0x44*/
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,
    opSTRBptim    ,

    /*0x45*/
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,
    opLDRBptim    ,

    /*0x46*/
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,
    opSTRBTptim   ,

    /*0x47*/
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,
    opLDRBTptim   ,

    /*0x48*/
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,
    opSTRptip     ,

    /*0x49*/
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,
    opLDRptip     ,

    /*0x4A*/
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,
    opSTRTptip    ,

    /*0x4B*/
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,
    opLDRTptip    ,

    /*0x4C*/
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,
    opSTRBptip    ,

    /*0x4D*/
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,
    opLDRBptip    ,

    /*0x4E*/
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,
    opSTRBTptip   ,

    /*0x4F*/
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,
    opLDRBTptip   ,

    /*0x50*/
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,
    opSTRofim     ,

    /*0x51*/
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,
    opLDRofim     ,

    /*0x52*/
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,
    opSTRprim     ,

    /*0x53*/
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,
    opLDRprim     ,

    /*0x54*/
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,
    opSTRBofim    ,

    /*0x55*/
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,
    opLDRBofim    ,

    /*0x56*/
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,
    opSTRBprim    ,

    /*0x57*/
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,
    opLDRBprim    ,

    /*0x58*/
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,
    opSTRofip     ,

    /*0x59*/
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,
    opLDRofip     ,

    /*0x5A*/
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,
    opSTRprip     ,

    /*0x5B*/
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,
    opLDRprip     ,

    /*0x5C*/
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,
    opSTRBofip    ,

    /*0x5D*/
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,
    opLDRBofip    ,

    /*0x5E*/
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,
    opSTRBprip    ,

    /*0x5F*/
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,
    opLDRBprip    ,

    /*0x60*/
    opSTRptrmll   ,
    opUND         ,
    opSTRptrmlr   ,
    opUND         ,
    opSTRptrmar   ,
    opUND         ,
    opSTRptrmrr   ,
    opUND         ,
    opSTRptrmll   ,
    opUND         ,
    opSTRptrmlr   ,
    opUND         ,
    opSTRptrmar   ,
    opUND         ,
    opSTRptrmrr   ,
    opUND         ,

    /*0x61*/
    opLDRptrmll   ,
    opUND         ,
    opLDRptrmlr   ,
    opUND         ,
    opLDRptrmar   ,
    opUND         ,
    opLDRptrmrr   ,
    opUND         ,
    opLDRptrmll   ,
    opUND         ,
    opLDRptrmlr   ,
    opUND         ,
    opLDRptrmar   ,
    opUND         ,
    opLDRptrmrr   ,
    opUND         ,

    /*0x62*/
    opSTRTptrmll  ,
    opUND         ,
    opSTRTptrmlr  ,
    opUND         ,
    opSTRTptrmar  ,
    opUND         ,
    opSTRTptrmrr  ,
    opUND         ,
    opSTRTptrmll  ,
    opUND         ,
    opSTRTptrmlr  ,
    opUND         ,
    opSTRTptrmar  ,
    opUND         ,
    opSTRTptrmrr  ,
    opUND         ,

    /*0x63*/
    opLDRTptrmll  ,
    opUND         ,
    opLDRTptrmlr  ,
    opUND         ,
    opLDRTptrmar  ,
    opUND         ,
    opLDRTptrmrr  ,
    opUND         ,
    opLDRTptrmll  ,
    opUND         ,
    opLDRTptrmlr  ,
    opUND         ,
    opLDRTptrmar  ,
    opUND         ,
    opLDRTptrmrr  ,
    opUND         ,

    /*0x64*/
    opSTRBptrmll  ,
    opUND         ,
    opSTRBptrmlr  ,
    opUND         ,
    opSTRBptrmar  ,
    opUND         ,
    opSTRBptrmrr  ,
    opUND         ,
    opSTRBptrmll  ,
    opUND         ,
    opSTRBptrmlr  ,
    opUND         ,
    opSTRBptrmar  ,
    opUND         ,
    opSTRBptrmrr  ,
    opUND         ,

    /*0x65*/
    opLDRBptrmll  ,
    opUND         ,
    opLDRBptrmlr  ,
    opUND         ,
    opLDRBptrmar  ,
    opUND         ,
    opLDRBptrmrr  ,
    opUND         ,
    opLDRBptrmll  ,
    opUND         ,
    opLDRBptrmlr  ,
    opUND         ,
    opLDRBptrmar  ,
    opUND         ,
    opLDRBptrmrr  ,
    opUND         ,

    /*0x66*/
    opSTRBTptrmll ,
    opUND         ,
    opSTRBTptrmlr ,
    opUND         ,
    opSTRBTptrmar ,
    opUND         ,
    opSTRBTptrmrr ,
    opUND         ,
    opSTRBTptrmll ,
    opUND         ,
    opSTRBTptrmlr ,
    opUND         ,
    opSTRBTptrmar ,
    opUND         ,
    opSTRBTptrmrr ,
    opUND         ,

    /*0x67*/
    opLDRBTptrmll ,
    opUND         ,
    opLDRBTptrmlr ,
    opUND         ,
    opLDRBTptrmar ,
    opUND         ,
    opLDRBTptrmrr ,
    opUND         ,
    opLDRBTptrmll ,
    opUND         ,
    opLDRBTptrmlr ,
    opUND         ,
    opLDRBTptrmar ,
    opUND         ,
    opLDRBTptrmrr ,
    opUND         ,

    /*0x68*/
    opSTRptrpll   ,
    opUND         ,
    opSTRptrplr   ,
    opUND         ,
    opSTRptrpar   ,
    opUND         ,
    opSTRptrprr   ,
    opUND         ,
    opSTRptrpll   ,
    opUND         ,
    opSTRptrplr   ,
    opUND         ,
    opSTRptrpar   ,
    opUND         ,
    opSTRptrprr   ,
    opUND         ,

    /*0x69*/
    opLDRptrpll   ,
    opUND         ,
    opLDRptrplr   ,
    opUND         ,
    opLDRptrpar   ,
    opUND         ,
    opLDRptrprr   ,
    opUND         ,
    opLDRptrpll   ,
    opUND         ,
    opLDRptrplr   ,
    opUND         ,
    opLDRptrpar   ,
    opUND         ,
    opLDRptrprr   ,
    opUND         ,

    /*0x6A*/
    opSTRTptrpll  ,
    opUND         ,
    opSTRTptrplr  ,
    opUND         ,
    opSTRTptrpar  ,
    opUND         ,
    opSTRTptrprr  ,
    opUND         ,
    opSTRTptrpll  ,
    opUND         ,
    opSTRTptrplr  ,
    opUND         ,
    opSTRTptrpar  ,
    opUND         ,
    opSTRTptrprr  ,
    opUND         ,

    /*0x6B*/
    opLDRTptrpll  ,
    opUND         ,
    opLDRTptrplr  ,
    opUND         ,
    opLDRTptrpar  ,
    opUND         ,
    opLDRTptrprr  ,
    opUND         ,
    opLDRTptrpll  ,
    opUND         ,
    opLDRTptrplr  ,
    opUND         ,
    opLDRTptrpar  ,
    opUND         ,
    opLDRTptrprr  ,
    opUND         ,

    /*0x6C*/
    opSTRBptrpll  ,
    opUND         ,
    opSTRBptrplr  ,
    opUND         ,
    opSTRBptrpar  ,
    opUND         ,
    opSTRBptrprr  ,
    opUND         ,
    opSTRBptrpll  ,
    opUND         ,
    opSTRBptrplr  ,
    opUND         ,
    opSTRBptrpar  ,
    opUND         ,
    opSTRBptrprr  ,
    opUND         ,

    /*0x6D*/
    opLDRBptrpll  ,
    opUND         ,
    opLDRBptrplr  ,
    opUND         ,
    opLDRBptrpar  ,
    opUND         ,
    opLDRBptrprr  ,
    opUND         ,
    opLDRBptrpll  ,
    opUND         ,
    opLDRBptrplr  ,
    opUND         ,
    opLDRBptrpar  ,
    opUND         ,
    opLDRBptrprr  ,
    opUND         ,

    /*0x6E*/
    opSTRBTptrpll ,
    opUND         ,
    opSTRBTptrplr ,
    opUND         ,
    opSTRBTptrpar ,
    opUND         ,
    opSTRBTptrprr ,
    opUND         ,
    opSTRBTptrpll ,
    opUND         ,
    opSTRBTptrplr ,
    opUND         ,
    opSTRBTptrpar ,
    opUND         ,
    opSTRBTptrprr ,
    opUND         ,

    /*0x6F*/
    opLDRBTptrpll ,
    opUND         ,
    opLDRBTptrplr ,
    opUND         ,
    opLDRBTptrpar ,
    opUND         ,
    opLDRBTptrprr ,
    opUND         ,
    opLDRBTptrpll ,
    opUND         ,
    opLDRBTptrplr ,
    opUND         ,
    opLDRBTptrpar ,
    opUND         ,
    opLDRBTptrprr ,
    opUND         ,

    /*0x70*/
    opSTRofrmll   ,
    opUND         ,
    opSTRofrmlr   ,
    opUND         ,
    opSTRofrmar   ,
    opUND         ,
    opSTRofrmrr   ,
    opUND         ,
    opSTRofrmll   ,
    opUND         ,
    opSTRofrmlr   ,
    opUND         ,
    opSTRofrmar   ,
    opUND         ,
    opSTRofrmrr   ,
    opUND         ,

    /*0x71*/
    opLDRofrmll   ,
    opUND         ,
    opLDRofrmlr   ,
    opUND         ,
    opLDRofrmar   ,
    opUND         ,
    opLDRofrmrr   ,
    opUND         ,
    opLDRofrmll   ,
    opUND         ,
    opLDRofrmlr   ,
    opUND         ,
    opLDRofrmar   ,
    opUND         ,
    opLDRofrmrr   ,
    opUND         ,

    /*0x72*/
    opSTRprrmll   ,
    opUND         ,
    opSTRprrmlr   ,
    opUND         ,
    opSTRprrmar   ,
    opUND         ,
    opSTRprrmrr   ,
    opUND         ,
    opSTRprrmll   ,
    opUND         ,
    opSTRprrmlr   ,
    opUND         ,
    opSTRprrmar   ,
    opUND         ,
    opSTRprrmrr   ,
    opUND         ,

    /*0x73*/
    opLDRprrmll   ,
    opUND         ,
    opLDRprrmlr   ,
    opUND         ,
    opLDRprrmar   ,
    opUND         ,
    opLDRprrmrr   ,
    opUND         ,
    opLDRprrmll   ,
    opUND         ,
    opLDRprrmlr   ,
    opUND         ,
    opLDRprrmar   ,
    opUND         ,
    opLDRprrmrr   ,
    opUND         ,

    /*0x74*/
    opSTRBofrmll  ,
    opUND         ,
    opSTRBofrmlr  ,
    opUND         ,
    opSTRBofrmar  ,
    opUND         ,
    opSTRBofrmrr  ,
    opUND         ,
    opSTRBofrmll  ,
    opUND         ,
    opSTRBofrmlr  ,
    opUND         ,
    opSTRBofrmar  ,
    opUND         ,
    opSTRBofrmrr  ,
    opUND         ,

    /*0x75*/
    opLDRBofrmll  ,
    opUND         ,
    opLDRBofrmlr  ,
    opUND         ,
    opLDRBofrmar  ,
    opUND         ,
    opLDRBofrmrr  ,
    opUND         ,
    opLDRBofrmll  ,
    opUND         ,
    opLDRBofrmlr  ,
    opUND         ,
    opLDRBofrmar  ,
    opUND         ,
    opLDRBofrmrr  ,
    opUND         ,

    /*0x76*/
    opSTRBprrmll  ,
    opUND         ,
    opSTRBprrmlr  ,
    opUND         ,
    opSTRBprrmar  ,
    opUND         ,
    opSTRBprrmrr  ,
    opUND         ,
    opSTRBprrmll  ,
    opUND         ,
    opSTRBprrmlr  ,
    opUND         ,
    opSTRBprrmar  ,
    opUND         ,
    opSTRBprrmrr  ,
    opUND         ,

    /*0x77*/
    opLDRBprrmll  ,
    opUND         ,
    opLDRBprrmlr  ,
    opUND         ,
    opLDRBprrmar  ,
    opUND         ,
    opLDRBprrmrr  ,
    opUND         ,
    opLDRBprrmll  ,
    opUND         ,
    opLDRBprrmlr  ,
    opUND         ,
    opLDRBprrmar  ,
    opUND         ,
    opLDRBprrmrr  ,
    opUND         ,

    /*0x78*/
    opSTRofrpll   ,
    opUND         ,
    opSTRofrplr   ,
    opUND         ,
    opSTRofrpar   ,
    opUND         ,
    opSTRofrprr   ,
    opUND         ,
    opSTRofrpll   ,
    opUND         ,
    opSTRofrplr   ,
    opUND         ,
    opSTRofrpar   ,
    opUND         ,
    opSTRofrprr   ,
    opUND         ,

    /*0x79*/
    opLDRofrpll   ,
    opUND         ,
    opLDRofrplr   ,
    opUND         ,
    opLDRofrpar   ,
    opUND         ,
    opLDRofrprr   ,
    opUND         ,
    opLDRofrpll   ,
    opUND         ,
    opLDRofrplr   ,
    opUND         ,
    opLDRofrpar   ,
    opUND         ,
    opLDRofrprr   ,
    opUND         ,

    /*0x7A*/
    opSTRprrpll   ,
    opUND         ,
    opSTRprrplr   ,
    opUND         ,
    opSTRprrpar   ,
    opUND         ,
    opSTRprrprr   ,
    opUND         ,
    opSTRprrpll   ,
    opUND         ,
    opSTRprrplr   ,
    opUND         ,
    opSTRprrpar   ,
    opUND         ,
    opSTRprrprr   ,
    opUND         ,

    /*0x7B*/
    opLDRprrpll   ,
    opUND         ,
    opLDRprrplr   ,
    opUND         ,
    opLDRprrpar   ,
    opUND         ,
    opLDRprrprr   ,
    opUND         ,
    opLDRprrpll   ,
    opUND         ,
    opLDRprrplr   ,
    opUND         ,
    opLDRprrpar   ,
    opUND         ,
    opLDRprrprr   ,
    opUND         ,

    /*0x7C*/
    opSTRBofrpll  ,
    opUND         ,
    opSTRBofrplr  ,
    opUND         ,
    opSTRBofrpar  ,
    opUND         ,
    opSTRBofrprr  ,
    opUND         ,
    opSTRBofrpll  ,
    opUND         ,
    opSTRBofrplr  ,
    opUND         ,
    opSTRBofrpar  ,
    opUND         ,
    opSTRBofrprr  ,
    opUND         ,

    /*0x7D*/
    opLDRBofrpll  ,
    opUND         ,
    opLDRBofrplr  ,
    opUND         ,
    opLDRBofrpar  ,
    opUND         ,
    opLDRBofrprr  ,
    opUND         ,
    opLDRBofrpll  ,
    opUND         ,
    opLDRBofrplr  ,
    opUND         ,
    opLDRBofrpar  ,
    opUND         ,
    opLDRBofrprr  ,
    opUND         ,

    /*0x7E*/
    opSTRBprrpll  ,
    opUND         ,
    opSTRBprrplr  ,
    opUND         ,
    opSTRBprrpar  ,
    opUND         ,
    opSTRBprrprr  ,
    opUND         ,
    opSTRBprrpll  ,
    opUND         ,
    opSTRBprrplr  ,
    opUND         ,
    opSTRBprrpar  ,
    opUND         ,
    opSTRBprrprr  ,
    opUND         ,

    /*0x7F*/
    opLDRBprrpll  ,
    opUND         ,
    opLDRBprrplr  ,
    opUND         ,
    opLDRBprrpar  ,
    opUND         ,
    opLDRBprrprr  ,
    opUND         ,
    opLDRBprrpll  ,
    opUND         ,
    opLDRBprrplr  ,
    opUND         ,
    opLDRBprrpar  ,
    opUND         ,
    opLDRBprrprr  ,
    opUND         ,

    /*0x80*/
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,
    opSTMDA       ,

    /*0x81*/
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,
    opLDMDA       ,

    /*0x82*/
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,
    opSTMDAw      ,

    /*0x83*/
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,
    opLDMDAw      ,

    /*0x84*/
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,
    opSTMDAu      ,

    /*0x85*/
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,
    opLDMDAu      ,

    /*0x86*/
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,
    opSTMDAuw     ,

    /*0x87*/
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,
    opLDMDAuw     ,

    /*0x88*/
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,
    opSTMIA       ,

    /*0x89*/
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,
    opLDMIA       ,

    /*0x8A*/
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,
    opSTMIAw      ,

    /*0x8B*/
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,
    opLDMIAw      ,

    /*0x8C*/
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,
    opSTMIAu      ,

    /*0x8D*/
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,
    opLDMIAu      ,

    /*0x8E*/
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,
    opSTMIAuw     ,

    /*0x8F*/
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,
    opLDMIAuw     ,

    /*0x90*/
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,
    opSTMDB       ,

    /*0x91*/
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,
    opLDMDB       ,

    /*0x92*/
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,
    opSTMDBw      ,

    /*0x93*/
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,
    opLDMDBw      ,

    /*0x94*/
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,
    opSTMDBu      ,

    /*0x95*/
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,
    opLDMDBu      ,

    /*0x96*/
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,
    opSTMDBuw     ,

    /*0x97*/
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,
    opLDMDBuw     ,

    /*0x98*/
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,
    opSTMIB       ,

    /*0x99*/
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,
    opLDMIB       ,

    /*0x9A*/
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,
    opSTMIBw      ,

    /*0x9B*/
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,
    opLDMIBw      ,

    /*0x9C*/
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,
    opSTMIBu      ,

    /*0x9D*/
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,
    opLDMIBu      ,

    /*0x9E*/
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,
    opSTMIBuw     ,

    /*0x9F*/
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,
    opLDMIBuw     ,

    /*0xA0*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA1*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA2*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA3*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA4*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA5*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA6*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA7*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA8*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xA9*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xAA*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xAB*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xAC*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xAD*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xAE*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xAF*/
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,
    opB           ,

    /*0xB0*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB1*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB2*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB3*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB4*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB5*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB6*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB7*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB8*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xB9*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xBA*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xBB*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xBC*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xBD*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xBE*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xBF*/
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,
    opBL          ,

    /*0xC0*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC1*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC2*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC3*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC4*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC5*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC6*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC7*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC8*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xC9*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xCA*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xCB*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xCC*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xCD*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xCE*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xCF*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD0*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD1*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD2*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD3*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD4*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD5*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD6*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD7*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD8*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xD9*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xDA*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xDB*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xDC*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xDD*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xDE*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xDF*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE0*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE1*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE2*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE3*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE4*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE5*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE6*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE7*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE8*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xE9*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xEA*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xEB*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xEC*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xED*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xEE*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xEF*/
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,
    opUNI         ,

    /*0xF0*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF1*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF2*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF3*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF4*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF5*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF6*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF7*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF8*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xF9*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xFA*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xFB*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xFC*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xFD*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xFE*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,

    /*0xFF*/
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
    opSWI         ,
};

/*** EOF: arm7tdmi-olut.cpp **********************************************/

