/**************************************************************************
* DSemu - The Next Generation                                             *
* Null GPU plugin: Class definition [nullgpu.h]                           *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#include "defs.h"
#include "plggpu.h"
#include "plggui.h"
#include "plgcpu.h"
#include "plgmmu32.h"
#include "datadefs.h"

// An implementation of the GPU plugin interface.
class dsGPU : public GPUPlugin
{
    public:
        dsGPU(std::string, REQPTR, UNREQPTR);
	~dsGPU();

        PLUGININFO *getinfo();
	void reset();
	void release();
	void status(int,int);
	
	void setDisplay(u8*);
	
    private:
        //---Plugin-generic stuff
        std::string pName, pClass;
        REQPTR pRequest;
        UNREQPTR pUnrequest;
	static PLUGININFO pInfo;
       
        static std::string pluginName;
	static GUIPlugin *GUI;

	static u8 *dispbuffer;
	static CPUPlugin *CPU;
	static MMU32Plugin *MMU, *MMUsub;

	static u8  *VRAMb[9];
	static u16 *VRAMh[9];
	static u32 *VRAMw[9];
	static const int VRAMsizes[9];

	static u8  *PALb, *OAMb;
	static u16 *PALh, *OAMh;
	static u32 *PALw, *OAMw;

        typedef struct {
	    u8 *b;
	    u16 *h;
	    u32 *w;
	} VRAMPTR;
	
	static std::map<u32,VRAMPTR> bankptrs;
	static const u32 bankaddrs[9][32];
	
	static void drawline(Plugin*);
	inline void line();

	typedef struct {
            union {
                u16 data;
	        u8 b[2];
            };
            u16 flags;
        } IOREG;
	
	static IOREG bankio[5];
	
	static u8  rdB(u32);
	static u16 rdH(u32);
	static u32 rdW(u32);
	static void wrB(u32, u8 );
	static void wrH(u32, u16);
	static void wrW(u32, u32);
	static void bankChange(u8, u8);

};

/*** EOF: dsgpu.h *******************************************************/

