/**************************************************************************
* DSemu - The Next Generation                                             *
* Error handling through exceptions [err.cpp]                             *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

// This file is quite old, so it's very well documented.

#include "log.h"
#include "err.h"

//-------------------------------------------------------------------------
// Function:   Constructor (Exception)
// Purpose:    Fill private variables with blanks.
// Parameters: None.

Exception::Exception()
{
    type = ERR_UNKNOWN;
    msg = std::string("");
}

//-------------------------------------------------------------------------
// Function:   Constructor (Exception) [overload]
// Purpose:    Fill private variables with supplied type and blank msg.
// Parameters: exctype - Type of exception

Exception::Exception(int exctype)
{
    type = exctype;
    msg = std::string("");
}

//-------------------------------------------------------------------------
// Function:   Constructor (Exception) [overload]
// Purpose:    Fill private variables with supplied values.
// Parameters: exctype - Type of exception
//             excmsg  - Message providing exception details

Exception::Exception(int exctype, std::string excmod, std::string excmsg)
{
    type = exctype;
    module = excmod;
    msg = excmsg;
}

//-------------------------------------------------------------------------
// Function:   Non-logging print-out (Report)
// Purpose:    Print error message based on contents of Exception, without
//             saving to log file.
// Parameters: None
// Returns:    None

void Exception::Report()
{
    char str[1024];
    
    sprintf(str, "Error type x%04X: %s", type, msg.c_str());

    #ifdef WIN32
        MessageBox(NULL, str, "Emulation Error", MB_OK);
    #else
        fprintf(stderr, "%s\n", str);
    #endif
}

//-------------------------------------------------------------------------
// Function:   Logging print-out (Log)
// Purpose:    Print error message based on Exception contents, to log.
// Parameters: None
// Returns:    None

void Exception::Log()
{
    char str[1024];
    
    sprintf(str, "Error type x%04X", type);
    Logger::log(module) << str << ": " << msg;
}

/*** EOF: err.cpp ********************************************************/

