/**************************************************************************
* DSemu - The Next Generation                                             *
* Null APU plugin: Implementation [nullapu.cpp]                           *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#include "defs.h"
#include "nullapu.h"
#include "log.h"

//---Static private class members------------------------------------------
// Every plugin has an INFO structure attached, with info about the plugin.

PLUGININFO nullAPU::pInfo={
    PLUGIN_TYPE_APU,
    0x00010001,
    "Null audio plugin",
    "DSemu-ng"
};

std::string nullAPU::pluginName;

//---Implementation--------------------------------------------------------

void nullAPU::togglePause()
{
}

nullAPU::nullAPU(std::string name, REQPTR req, UNREQPTR unreq)
{
    pName = std::string(name);
    pClass = pName.substr(0, pName.find(".")+1);
    pRequest = req;
    pUnrequest = unreq;

    pluginName = std::string(pName);

    Logger::log(pName) << "Initialised.";
}

nullAPU::~nullAPU()
{
    Logger::log(pName) << "Shutdown.";
}

// Reset plugin: stop the stream, clear registers
void nullAPU::reset()
{
    Logger::log(pName) << "Reset";
}

// Provide plugin status: Not much really to give
void nullAPU::status(int opt1=0, int opt2=0)
{
}

//---Plugin architecture support-------------------------------------------
// Retrieve Plugin class from outside
// Parameters: plg   - Address of a pointer to a Plugin class to 'new'
//             name  - FQPN of plugin as listed in INI file
//             req   - Pointer to PluginRequest API function
//             unreq - Pointer to PluginUnrequest API function
EXPORTFUNC void getPlugin(Plugin **plg,
                          std::string name,
                          REQPTR req, UNREQPTR unreq)
{
   *plg = new nullAPU(name, req, unreq);
}

// Provide plugin version information
PLUGININFO *nullAPU::getinfo()
{
    return &pInfo;
}

// Release plugin from outside
void nullAPU::release()
{
    // Delete the Test plugin that was 'new'd in getPlugin.
    delete this;
}

/*** EOF: nullapu.cpp ****************************************************/

