/**************************************************************************
* DSemu - The Next Generation                                             *
* Null GPU plugin: Implementation [nullgpu.cpp]                           *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#include <string>
#include "events.h"
#include "defs.h"
#include "nullgpu.h"
#include "log.h"

//---Static private class members------------------------------------------
// Every plugin has an INFO structure attached, with info about the plugin.

PLUGININFO nullGPU::pInfo={
    PLUGIN_TYPE_GPU,
    0x00010001,
    "Null graphics plugin",
    "DSemu-ng"
};

GUIPlugin *nullGPU::GUI;
std::string nullGPU::pluginName;

void nullGPU::drawline(Plugin *in) { ((nullGPU*)in)->line(); }
void nullGPU::line()
{
    GUI->eventPush(1000, EVENT_HBLANK_START, (vfptr)drawline, this);
}

// Tell the GPU where it'll be rendering to
void nullGPU::setDisplay(u8 *buf)
{
}

// Initialise plugin: Allocate memories, register with MMU
nullGPU::nullGPU(std::string name, REQPTR req, UNREQPTR unreq)
{
    pName = std::string(name);
    pClass = pName.substr(0, pName.find(".")+1);
    pRequest = req;
    pUnrequest = unreq;

    pluginName = std::string(pName);

    GUI = (GUIPlugin*)pRequest("UI");
    
    Logger::log(pName) << "Initialised.";
}

// Shut down plugin: Free up memories
nullGPU::~nullGPU()
{
    pUnrequest("UI", 0); GUI = NULL;
    
    Logger::log(pName) << "Shutdown.";
}

// Reset plugin: Clear memories, read config file
void nullGPU::reset()
{
    // First scanline call: In a thousand cycles, come back and draw a line
    GUI->eventPush(1000, EVENT_HBLANK_START, (vfptr)drawline, this);
    
    Logger::log(pName) << "Reset";
}

// Provide status: Fill palette-window framebuffer
void nullGPU::status(int opt1=0, int opt2=0)
{
}

//---Plugin architecture support-------------------------------------------
// Retrieve Plugin class from outside
// Parameters: plg   - Address of a pointer to a Plugin class to 'new'
//             name  - FQPN of plugin as listed in INI file
//             req   - Pointer to PluginRequest API function
//             unreq - Pointer to PluginUnrequest API function
EXPORTFUNC void getPlugin(Plugin **plg,
                          std::string name,
                          REQPTR req, UNREQPTR unreq)
{
   *plg = new nullGPU(name, req, unreq);
}

// Provide plugin version information
PLUGININFO *nullGPU::getinfo()
{
    return &pInfo;
}

// Release plugin from outside
void nullGPU::release()
{
    // Delete the Test plugin that was 'new'd in getPlugin.
    delete this;
}

/*** EOF: nullgpu.cpp ****************************************************/

