/**************************************************************************
* DSemu - The Next Generation                                             *
* Plugin definitions: Base interface [plgbase.h]                          *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#ifndef __PLGBASE_H_
#define __PLGBASE_H_

#include "defs.h"
#include "datadefs.h"
#include <string>

// Definition of the information a plugin needs to carry on its version.
typedef struct {
    u32 type;
    u32 version;
    const char *desc;
    const char *provider;
} PLUGININFO;

class Plugin 
{
    public:
        virtual ~Plugin() { }                     // Top-level destructor

        virtual PLUGININFO *getinfo() = 0;        // Plugin info
        virtual void reset() = 0;                 // Reset
        virtual void release() = 0;               // Outside cleanup
        virtual void status(int,int) = 0;         // Status
};

// Typedefs to help plugin implementors
typedef Plugin* (*REQPTR)(std::string);
typedef void (*UNREQPTR)(std::string, int);

typedef void (*GETPLUGIN)(Plugin**,std::string,REQPTR,UNREQPTR);

#define PLUGIN_TYPE_NULL 0

#endif//__PLGBASE_H_

/*** EOF: plgbase.h ******************************************************/

