/**************************************************************************
* DSemu - The Next Generation                                             *
* Plugin definitions: 32-bit MMU interface [plgmmu32.h]                   *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#ifndef __PLGMMU32_H_
#define __PLGMMU32_H_

#include "plgmmu.h"
#include "datadefs.h"

// Extending the simple MMU plugin interface.
class MMU32Plugin : public MMUPlugin
{
    public:
	// Reads: 8-bit, 16-bit, 32-bit
        virtual u8  rdB(u32) = 0;
        virtual u16 rdH(u32) = 0;
        virtual u32 rdW(u32) = 0;

	// Writes: 8-bit, 16-bit, 32-bit
        virtual void wrB(u32, u8 ) = 0;
        virtual void wrH(u32, u16) = 0;
        virtual void wrW(u32, u32) = 0;
	
	// Typedefs for use by plugin developers
	typedef u8  (*rdBptr)(u32);
	typedef u16 (*rdHptr)(u32);
	typedef u32 (*rdWptr)(u32);
	
	typedef void (*wrBptr)(u32,u8 );
	typedef void (*wrHptr)(u32,u16);
	typedef void (*wrWptr)(u32,u32);
	
	// Definition of a page
	typedef struct {
	    rdBptr rdB; rdHptr rdH; rdWptr rdW;
	    wrBptr wrB; wrHptr wrH; wrWptr wrW;
	    u32 set, pad;
	} PAGE;

	// Register pages with the MMU
	virtual void rangeReg(u8, rdBptr, rdHptr, rdWptr,
			          wrBptr, wrHptr, wrWptr) = 0;
		
	virtual void mmioReg(u8, rdBptr, rdHptr, rdWptr,
			         wrBptr, wrHptr, wrWptr) = 0;
};

#endif//__PLGMMU32_H_

/*** EOF: plgmmu32.h *****************************************************/

