/**************************************************************************
* DSemu - The Next Generation                                             *
* Plugin handling: Definitions [plugin.h]                                 *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#ifndef __PLUGIN_H_
#define __PLUGIN_H_

#include <string>
#include "log.h"
#include "err.h"
#include "plgbase.h"
#include "w32compile.h"

#ifdef COMPILING_ON_WINDERZ
    #include <windows.h>
#else
    #include <stdio.h>
    #include <dlfcn.h>
#endif

// Information about a plugin, from the loader's perspective.
typedef struct {
    std::string name;      // Name of the plugin, minus any file extensions
    Plugin *plg;           // Pointer to the Plugin class in the library
    int refs;              // Number of active references

    #ifdef WIN32
        HMODULE file;      // Pointer to the library itself (for Windows)
    #else
        void *file;        // (or for Unix)
    #endif
} PLUGINREF;

// Functions
Plugin *PluginLoad(std::string);
void PluginUnload(std::string);

Plugin *PluginRequest(std::string);
void PluginUnrequest(std::string, int);

// Possible errors
#define ERR_PLG_LOAD        0x0021  // Failed while loading new plugin
#define ERR_PLG_UNLD        0x0022  // Failed to unload
#define ERR_PLG_UNRQ        0x0023  // Failed attempt to unrequest

#endif//__PLUGIN_H_

/*** EOF: plugin.h *******************************************************/

