/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM7TDMI core: Thumb opcode table [thumb7-olut.cpp]            *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#ifdef ARM7TDMI_X86

#include "arm7tdmi.h"

// The opcode table, indexed on opcode bits 15-8.
// Instructions not defined in ARMv4 but in v5 are marked.
const ARM7TDMI::opfptr ARM7TDMI::XTLUT[256]={
    //0x00
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSLimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,
    xtopLSRimm    ,

    //0x10
    xtopASRimm    ,
    xtopASRimm    ,
    xtopASRimm    ,
    xtopASRimm    ,
    xtopASRimm    ,
    xtopASRimm    ,
    xtopASRimm    ,
    xtopASRimm    ,
    xtopADDreg    ,
    xtopADDreg    ,
    xtopSUBreg    ,
    xtopSUBreg    ,
    xtopADDimm3   ,
    xtopADDimm3   ,
    xtopSUBimm3   ,
    xtopSUBimm3   ,

    //0x20
    xtopMOVimm8r0 ,
    xtopMOVimm8r1 ,
    xtopMOVimm8r2 ,
    xtopMOVimm8r3 ,
    xtopMOVimm8r4 ,
    xtopMOVimm8r5 ,
    xtopMOVimm8r6 ,
    xtopMOVimm8r7 ,
    xtopCMPimm8r0 ,
    xtopCMPimm8r1 ,
    xtopCMPimm8r2 ,
    xtopCMPimm8r3 ,
    xtopCMPimm8r4 ,
    xtopCMPimm8r5 ,
    xtopCMPimm8r6 ,
    xtopCMPimm8r7 ,

    //0x30
    xtopADDimm8r0 ,
    xtopADDimm8r1 ,
    xtopADDimm8r2 ,
    xtopADDimm8r3 ,
    xtopADDimm8r4 ,
    xtopADDimm8r5 ,
    xtopADDimm8r6 ,
    xtopADDimm8r7 ,
    xtopSUBimm8r0 ,
    xtopSUBimm8r1 ,
    xtopSUBimm8r2 ,
    xtopSUBimm8r3 ,
    xtopSUBimm8r4 ,
    xtopSUBimm8r5 ,
    xtopSUBimm8r6 ,
    xtopSUBimm8r7 ,

    //0x40
    xtopDPg1      ,
    xtopDPg2      ,
    xtopDPg3      ,
    xtopDPg4      ,
    xtopADDH      ,
    xtopCMPH      ,
    xtopMOVH      ,
    xtopBXreg     ,
    xtopLDRPCr0   ,
    xtopLDRPCr1   ,
    xtopLDRPCr2   ,
    xtopLDRPCr3   ,
    xtopLDRPCr4   ,
    xtopLDRPCr5   ,
    xtopLDRPCr6   ,
    xtopLDRPCr7   ,

    //0x50
    xtopSTRreg    ,
    xtopSTRreg    ,
    xtopSTRHreg   ,
    xtopSTRHreg   ,
    xtopSTRBreg   ,
    xtopSTRBreg   ,
    xtopLDRSBreg  ,
    xtopLDRSBreg  ,
    xtopLDRreg    ,
    xtopLDRreg    ,
    xtopLDRHreg   ,
    xtopLDRHreg   ,
    xtopLDRBreg   ,
    xtopLDRBreg   ,
    xtopLDRSHreg  ,
    xtopLDRSHreg  ,

    //0x60
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopSTRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,
    xtopLDRimm5   ,

    //0x70
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopSTRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,
    xtopLDRBimm5  ,

    //0x80
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopSTRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,
    xtopLDRHimm5  ,

    //0x90
    xtopSTRSPr0   ,
    xtopSTRSPr1   ,
    xtopSTRSPr2   ,
    xtopSTRSPr3   ,
    xtopSTRSPr4   ,
    xtopSTRSPr5   ,
    xtopSTRSPr6   ,
    xtopSTRSPr7   ,
    xtopLDRSPr0   ,
    xtopLDRSPr1   ,
    xtopLDRSPr2   ,
    xtopLDRSPr3   ,
    xtopLDRSPr4   ,
    xtopLDRSPr5   ,
    xtopLDRSPr6   ,
    xtopLDRSPr7   ,

    //0xA0
    xtopADDPCr0   ,
    xtopADDPCr1   ,
    xtopADDPCr2   ,
    xtopADDPCr3   ,
    xtopADDPCr4   ,
    xtopADDPCr5   ,
    xtopADDPCr6   ,
    xtopADDPCr7   ,
    xtopADDSPr0   ,
    xtopADDSPr1   ,
    xtopADDSPr2   ,
    xtopADDSPr3   ,
    xtopADDSPr4   ,
    xtopADDSPr5   ,
    xtopADDSPr6   ,
    xtopADDSPr7   ,

    //0xB0
    xtopADDSPimm7 ,
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopPUSH      ,
    xtopPUSHlr    ,
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated
    xtopPOP       ,
    xtopPOPpc     ,
    xtopUND       ,  // Misc map: unallocated
    xtopUND       ,  // Misc map: unallocated

    //0xC0
    xtopSTMIAr0   ,
    xtopSTMIAr1   ,
    xtopSTMIAr2   ,
    xtopSTMIAr3   ,
    xtopSTMIAr4   ,
    xtopSTMIAr5   ,
    xtopSTMIAr6   ,
    xtopSTMIAr7   ,
    xtopLDMIAr0   ,
    xtopLDMIAr1   ,
    xtopLDMIAr2   ,
    xtopLDMIAr3   ,
    xtopLDMIAr4   ,
    xtopLDMIAr5   ,
    xtopLDMIAr6   ,
    xtopLDMIAr7   ,

    //0xD0
    xtopBEQ       ,
    xtopBNE       ,
    xtopBCS       ,
    xtopBCC       ,
    xtopBMI       ,
    xtopBPL       ,
    xtopBVS       ,
    xtopBVC       ,
    xtopBHI       ,
    xtopBLS       ,
    xtopBGE       ,
    xtopBLT       ,
    xtopBGT       ,
    xtopBLE       ,
    xtopUND       ,  // Covered by unconditional
    xtopSWI       ,

    //0xE0
    xtopB         ,
    xtopB         ,
    xtopB         ,
    xtopB         ,
    xtopB         ,
    xtopB         ,
    xtopB         ,
    xtopB         ,
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!
    xtopUND       ,  // Defined in ARM5!

    //0xF0
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLsetup   ,
    xtopBLoff     ,
    xtopBLoff     ,
    xtopBLoff     ,
    xtopBLoff     ,
    xtopBLoff     ,
    xtopBLoff     ,
    xtopBLoff     ,
    xtopBLoff     ,
};

#endif

/*** EOF: thumb7-olut.cpp ************************************************/

