/**************************************************************************
* DSemu - The Next Generation                                             *
* Portable ARM cores: Common disassembler [armdasm.cpp]                   *
* Copyright Imran Nazar, 2005; released under the BSD public licence.     *
**************************************************************************/

#include <string>
#include "armmasks.h"
#include "armdasm.h"
#include "datadefs.h"

char ARMDasm::dout[128]="";
u32 ARMDasm::tmp1=0;
u32 ARMDasm::tmp2=0;
u32 ARMDasm::tmp3=0;
u32 ARMDasm::tmp4=0;
u32 (*ARMDasm::rdW)(u32);

// The constructor is passed a function that can read words from memory;
// this is used by LDR to retrieve PC-relative values.
ARMDasm::ARMDasm(u32 (*addr)(u32))
{
    rdW=addr;
}

// ARM-mode lookup
std::string ARMDasm::disasm(u32 op, u32 r15)
{
    static char str[100];
    u16 idx=DASMOP_EOP;
    ops[idx].addr(op,r15);
    dout[32]=0;
    sprintf(str, ops[idx].str, condstr[DASMOP_COND], dout);
    return std::string(str);
}

// Thumb-mode lookup
std::string ARMDasm::tdisasm(u16 op, u32 r15)
{
    static char str[100];
    u8 idx=(op&0xFF00)>>8;
    tops[idx].addr((u32)op, r15);
    dout[32]=0;
    sprintf(str, tops[idx].str, dout);
    return std::string(str);
}

void ARMDasm::opun(u32 op, u32 r15)
{
    sprintf(dout," ");
}

//---ARM mode disassemblies------------------------------------------------

// Branching
void ARMDasm::opb(u32 op, u32 r15)
{
    signed int b = (op&0x00800000)?(0xFF000000|(op&0x00FFFFFF))
                                  :(op&0x00FFFFFF);
    sprintf(dout,"$%08X",r15+8+(b*4));
}

void ARMDasm::opbreg(u32 op, u32 r15)
{
    sprintf(dout,"r%d",DASMOP_RM);
}

// Data Processing
void ARMDasm::opreg(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opimm(u32 op, u32 r15)
{
    tmp3 = op&255;
    tmp4 = (op&0x00000F00)>>7;
    tmp1 = (tmp3>>tmp4)|((tmp3&((1<<tmp4)-1))<<(32-tmp4));
    sprintf(dout,"r%d, r%d, #$%08X", DASMOP_RD, DASMOP_RN, tmp1);
}

void ARMDasm::opcimm(u32 op, u32 r15)
{
    tmp3 = op&255;
    tmp4 = (op&0x00000F00)>>7;
    tmp1 = (tmp3>>tmp4)|((tmp3&((1<<tmp4)-1))<<(32-tmp4));
    sprintf(dout,"r%d, #$%08X", DASMOP_RN, tmp1);
}

void ARMDasm::oplli(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
       sprintf(dout,"r%d, r%d, r%d, lsl #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
       sprintf(dout,"r%d, r%d, r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opllr(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d, lsl r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

void ARMDasm::oplri(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, r%d, r%d, lsr #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::oplrr(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d, lsr r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

void ARMDasm::opari(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, r%d, r%d, asr #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::oparr(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d, asr r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

void ARMDasm::oprri(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
        sprintf(dout,"r%d, r%d, r%d, ror #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, r%d, r%d, rrx", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::oprrr(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, r%d, r%d, ror r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

// Compare-specific disassemblies (Rd not required)
void ARMDasm::opclli(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
       sprintf(dout,"r%d, r%d, lsl #%d", DASMOP_RN, DASMOP_RM, tmp4):
       sprintf(dout,"r%d, r%d", DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opcllr(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, lsl r%d", DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

void ARMDasm::opclri(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, r%d, lsr #%d", DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opclrr(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, lsr r%d", DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

void ARMDasm::opcari(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, r%d, asr #%d", DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opcarr(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, asr r%d", DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

void ARMDasm::opcrri(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
        sprintf(dout,"r%d, r%d, ror #%d", DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, r%d, rrx", DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opcrrr(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, r%d, ror r%d", DASMOP_RN, DASMOP_RM, DASMOP_RS);
}

// Load/Store
void ARMDasm::opofim(u32 op, u32 r15) 
{
    ((DASMOP_RN==15) && (op&0x00100000))?
        sprintf(dout,"r%d, =$%08X", DASMOP_RD, rdW(r15+8-(op&0x00000FFF))):
        sprintf(dout,"r%d, [r%d, #-$%03X]", DASMOP_RD, DASMOP_RN, op&0x00000FFF);
}

void ARMDasm::opofip(u32 op, u32 r15)
{
    ((DASMOP_RN==15) && (op&0x00100000))?
        sprintf(dout,"r%d, =$%08X", DASMOP_RD, rdW(r15+8+(op&0x00000FFF))):
        sprintf(dout,"r%d, [r%d, #+$%03X]", DASMOP_RD, DASMOP_RN, op&0x00000FFF);
}

void ARMDasm::opofrm(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, [r%d, -r%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opofrmll(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, -r%d, lsl #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opofrmlr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, [r%d, -r%d, lsr #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opofrmar(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, -r%d, asr #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opofrmrr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
        sprintf(dout,"r%d, [r%d, -r%d, ror #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, [r%d, -r%d, rrx]", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opofrp(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, [r%d, +r%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}    

void ARMDasm::opofrpll(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, [r%d, +r%d, lsl #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opofrplr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, +r%d, lsr #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opofrpar(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, +r%d, asr #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opofrprr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
        sprintf(dout,"r%d, [r%d, +r%d, ror #%d]", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, [r%d, +r%d, rrx]", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opprim(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, [r%d, #-$%03X]!", DASMOP_RD, DASMOP_RN, op&0x00000FFF);
}

void ARMDasm::opprip(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, [r%d, #+$%03X]!", DASMOP_RD, DASMOP_RN, op&0x00000FFF);
}

void ARMDasm::opprrm(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, -r%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opprrmll(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, -r%d, lsl #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opprrmlr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, -r%d, lsr #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opprrmar(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, [r%d, -r%d, asr #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opprrmrr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
        sprintf(dout,"r%d, [r%d, -r%d, ror #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, [r%d, -r%d, rrx]!", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opprrp(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, +r%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opprrpll(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, +r%d, lsl #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opprrplr(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d, +r%d, lsr #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opprrpar(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, [r%d, +r%d, asr #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opprrprr(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    (tmp4)?
        sprintf(dout,"r%d, [r%d, +r%d, ror #%d]!", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, [r%d, +r%d, rrx]!", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opptim(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d], #-$%03X", DASMOP_RD, DASMOP_RN, op&0x00000FFF);
}

void ARMDasm::opptip(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, [r%d], #+$%03X", DASMOP_RD, DASMOP_RN, op&0x00000FFF);
}

void ARMDasm::opptrm(u32 op, u32 r15) 
{
    sprintf(dout,"r%d, [r%d], -r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opptrmll(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d], -r%d, lsl #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opptrmlr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, [r%d], -r%d, lsr #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opptrmar(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d], -r%d, asr #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opptrmrr(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    (tmp4)?
        sprintf(dout,"r%d, [r%d], -r%d, ror #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, [r%d], -r%d, rrx", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opptrp(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d], +r%d", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

void ARMDasm::opptrpll(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7;
    sprintf(dout,"r%d, [r%d], +r%d, lsl #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opptrplr(u32 op, u32 r15) 
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d], +r%d, lsr #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opptrpar(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    sprintf(dout,"r%d, [r%d], +r%d, asr #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4);
}

void ARMDasm::opptrprr(u32 op, u32 r15)
{
    tmp4=(op&0x00000F80)>>7; 
    (tmp4)?
        sprintf(dout,"r%d, [r%d], +r%d, ror #%d", DASMOP_RD, DASMOP_RN, DASMOP_RM, tmp4):
        sprintf(dout,"r%d, [r%d], +r%d, rrx", DASMOP_RD, DASMOP_RN, DASMOP_RM);
}

// Load/store miscellaneous (halfword-style)
void ARMDasm::oplmofim(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, #-$%1X%1X]", DASMOP_RD, DASMOP_RN, DASMOP_RS, DASMOP_RM);
}

void ARMDasm::oplmofip(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, #+$%1X%1X]", DASMOP_RD, DASMOP_RN, DASMOP_RS, DASMOP_RM);
}

void ARMDasm::oplmprim(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, #-$%1X%1X]!", DASMOP_RD, DASMOP_RN, DASMOP_RS, DASMOP_RM);
}

void ARMDasm::oplmprip(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, #+$%1X%1X]!", DASMOP_RD, DASMOP_RN, DASMOP_RS, DASMOP_RM);
}

void ARMDasm::oplmptim(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d], #-$%1X%1X", DASMOP_RD, DASMOP_RN, DASMOP_RS, DASMOP_RM);
}

void ARMDasm::oplmptip(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d], #+$%1X%1X", DASMOP_RD, DASMOP_RN, DASMOP_RS, DASMOP_RM);
}

// Load/store multiple
// ACK: Costis (LDM disassembly algorithm, original)
void ARMDasm::oplm(u32 op, u32 r15)
{
    int i, b_start, b_end, inWord=op;
    char str[520],regstr[512];
    sprintf(str,"r%d",DASMOP_RN);
    if(op&0x00200000) sprintf(str,"%s!",str);
    sprintf(regstr,",{");
    b_start = b_end = -1;
    for (i = 0; i <= 16; i++)
    {
        if ((inWord & 1) && (b_start < 0))  { if(i<16) b_start = i; }
	else if (!(inWord & 1))
	{
	    b_end = i - 1;
	    if (b_start >= 0)
	    {
 	        if(strlen(regstr)==2)
 	        {
		    if(b_start!=b_end) sprintf(regstr, "%sr%d-r%d", regstr, b_start, b_end);
		    else sprintf(regstr, "%sr%d", regstr, b_start);
		} else {
		    if(b_start!=b_end) sprintf(regstr, "%s,r%d-r%d", regstr, b_start, b_end);
		    else sprintf(regstr, "%s,r%d", regstr, b_start);
		}
	    }
  	    b_start = -1;
	}
	inWord >>= 1;
    }
    sprintf(str,"%s%s}",str,regstr);
    if(op&0x00400000) sprintf(str,"%s^",str);
    sprintf(dout,"%s",str);
}

// MSR/MRS
void ARMDasm::opmrsrs(u32 op, u32 r15)
{
    sprintf(dout,"r%d, cpsr",DASMOP_RD);
}

void ARMDasm::opmrsrc(u32 op, u32 r15)
{
    sprintf(dout,"r%d, spsr",DASMOP_RD);
}

void ARMDasm::opmsric(u32 op, u32 r15)
{
    tmp3 = op&255; 
    tmp4 = (op&0x00000F00)>>7; 
    tmp1 = (tmp3>>tmp4)|((tmp3&((1<<tmp4)-1))<<(32-tmp4)); 
    sprintf(dout,"cpsr_%c%c%c%c, #%08X",
        ((op&0x00010000)?'c':'_'), ((op&0x00020000)?'x':'_'),
        ((op&0x00040000)?'s':'_'), ((op&0x00080000)?'f':'_'), tmp1);
	
}

void ARMDasm::opmsris(u32 op, u32 r15)
{
    tmp3 = op&255; 
    tmp4 = (op&0x00000F00)>>7;
    tmp1 = (tmp3>>tmp4)|((tmp3&((1<<tmp4)-1))<<(32-tmp4));
    sprintf(dout,"spsr_%c%c%c%c, #%08X",
        ((op&0x00010000)?'c':'_'), ((op&0x00020000)?'x':'_'),
        ((op&0x00040000)?'s':'_'), ((op&0x00080000)?'f':'_'), tmp1);
}

void ARMDasm::opmsrrc(u32 op, u32 r15)
{
    sprintf(dout,"cpsr_%c%c%c%c, r%d",
        ((op&0x00010000)?'c':'_'), ((op&0x00020000)?'x':'_'),
        ((op&0x00040000)?'s':'_'), ((op&0x00080000)?'f':'_'), DASMOP_RM);
}

void ARMDasm::opmsrrs(u32 op, u32 r15)
{
    sprintf(dout,"spsr_%c%c%c%c, r%d",
        ((op&0x00010000)?'c':'_'), ((op&0x00020000)?'x':'_'),
        ((op&0x00040000)?'s':'_'), ((op&0x00080000)?'f':'_'), DASMOP_RM);
}

// Swap
void ARMDasm::opswp(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, [r%d]",DASMOP_RD,DASMOP_RM,DASMOP_RN);
}

void ARMDasm::opswi(u32 op, u32 r15)
{
    sprintf(dout,"$%06X",op&0x00FFFFFF);
}

// Multiply
void ARMDasm::opmul(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d",DASMOP_RN,DASMOP_RM,DASMOP_RS);
}

void ARMDasm::opmla(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d, r%d",DASMOP_RN,DASMOP_RM,DASMOP_RS,DASMOP_RD);
}

void ARMDasm::opmull(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d, r%d",DASMOP_RD,DASMOP_RN,DASMOP_RM,DASMOP_RS);
}

// Coprocessors
void ARMDasm::opmcr(u32 op, u32 r15)
{
    sprintf(dout,"p%d, %1X, r%d, c%d, c%d, %1X",DASMOP_RS,DASMOP_RO>>1,DASMOP_RD,DASMOP_RN,DASMOP_RM,DASMOP_RP>>1);
}

void ARMDasm::opcpd(u32 op, u32 r15)
{
    sprintf(dout,"p%d, %1X, c%d, c%d, c%d, %1X",DASMOP_RS,DASMOP_RO>>1,DASMOP_RD,DASMOP_RN,DASMOP_RM,DASMOP_RP>>1);
}

// Enhanced DSP QADD
void ARMDasm::opqadd(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, r%d", DASMOP_RD, DASMOP_RM, DASMOP_RN);
}

// CLZ
void ARMDasm::opclz(u32 op, u32 r15)
{
    sprintf(dout, "r%d, r%d", DASMOP_RD, DASMOP_RM);
}

//---Thumb disasms---------------------------------------------------------

void ARMDasm::topimm5(u32 op, u32 r15)
{
    sprintf(dout,"r%d, [r%d, #$%02X",TDASMOP_RD,TDASMOP_RN,TDASMOP_IMM5);
}

void ARMDasm::topimm5shft(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, #$%02X",TDASMOP_RD,TDASMOP_RN,TDASMOP_IMM5);
}

void ARMDasm::topimm7(u32 op, u32 r15)
{
    if(op&0x0080) sprintf(dout,"SUB sp, sp, #$%02X",TDASMOP_IMM7);
    else          sprintf(dout,"ADD sp, sp, #$%02X",TDASMOP_IMM7);
}

void ARMDasm::topimm8(u32 op, u32 r15)
{
    sprintf(dout,"r%d, #$%02X",TDASMOP_RS, TDASMOP_IMM8);
}

void ARMDasm::topimm3(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d, #$%01X",TDASMOP_RD,TDASMOP_RN,TDASMOP_RM);
}

void ARMDasm::topb(u32 op, u32 r15)
{
    signed int b = (op&0x0400)?(0xFFFFFC00|(op&0x03FF))
                                          :(op&0x03FF);
    sprintf(dout,"$%08X",r15+4+(b*2));
}

void ARMDasm::topbx(u32 op, u32 r15)
{
    if(op&0x0080)
        sprintf(dout,"BLX r%d",TDASMOP_RNH);
    else
        sprintf(dout,"BX r%d",TDASMOP_RNH);
}

void ARMDasm::topbl1(u32 op, u32 r15)
{
    tmp4=(TDASMOP_IMM11)<<12; tmp4=(tmp4&0x400000)?(0xFF800000|tmp4):tmp4; tmp4+=r15; tmp4+=4;
    sprintf(dout,";Long branch");
}

void ARMDasm::topbl2(u32 op, u32 r15)
{
    tmp4 += TDASMOP_IMM11*2;
    sprintf(dout,"$%08X", tmp4);
}

void ARMDasm::topbc(u32 op, u32 r15)
{
    signed int b = (op&0x0080)?(0xFFFFFF00|(op&0x00FF))
                                          :(op&0x00FF);
    sprintf(dout,"$%08X",r15+4+(b*2));
}

void ARMDasm::toph(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d",TDASMOP_RDH,TDASMOP_RNH);
}

void ARMDasm::topldm(u32 op, u32 r15)
{
    int i, b_start, b_end, inWord=op;
    char str[512];
    sprintf(str,"{");
    b_start = b_end = -1;
    for (i = 0; i <= 8; i++)
    {
        if ((inWord & 1) && (b_start < 0))  { if(i<8) b_start = i; }
	else if (!(inWord & 1))
	{
	    b_end = i - 1;
	    if (b_start >= 0)
	    {
 	        if(strlen(str)==1)
 	        {
		    if (b_start != b_end) sprintf (str, "%sr%d-r%d", str, b_start, b_end);
		    else sprintf (str, "%sr%d", str, b_start);
		} else {
		    if (b_start != b_end) sprintf (str, "%s,r%d-r%d", str, b_start, b_end);
		    else sprintf (str, "%s,r%d", str, b_start);
		}
	    }
  	    b_start = -1;
	}
	inWord >>= 1;
    }
    sprintf(dout,"%s}",str);
}

void ARMDasm::topdp1(u32 op, u32 r15)
{
    switch((op&0x00C0)>>6)
    {
        case 0: sprintf(dout,"AND r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 1: sprintf(dout,"EOR r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 2: sprintf(dout,"LSL r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 3: sprintf(dout,"LSR r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
    }
}

void ARMDasm::topdp2(u32 op, u32 r15)
{
    switch((op&0x00C0)>>6)
    {
        case 0: sprintf(dout,"ASR r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 1: sprintf(dout,"ADC r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 2: sprintf(dout,"SBC r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 3: sprintf(dout,"ROR r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
    }
}

void ARMDasm::topdp3(u32 op, u32 r15)
{
    switch((op&0x00C0)>>6)
    {
        case 0: sprintf(dout,"TST r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 1: sprintf(dout,"NEG r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 2: sprintf(dout,"CMP r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 3: sprintf(dout,"CMN r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
    }
}

void ARMDasm::topdp4(u32 op, u32 r15)
{
    switch((op&0x00C0)>>6)
    {
        case 0: sprintf(dout,"ORR r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 1: sprintf(dout,"MUL r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 2: sprintf(dout,"BIC r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
        case 3: sprintf(dout,"MVN r%d, r%d",TDASMOP_RD,TDASMOP_RN); break;
    }
}

void ARMDasm::topreg(u32 op, u32 r15)
{
    sprintf(dout,"r%d, r%d",TDASMOP_RD,TDASMOP_RN);
}

void ARMDasm::topund(u32 op, u32 r15)
{
    sprintf(dout,"<unknown>");
}

void ARMDasm::toppc(u32 op, u32 r15)
{
    sprintf(dout,"=$%08X",rdW(r15+(TDASMOP_IMM8<<2)+4));
}

void ARMDasm::topsp(u32 op, u32 r15)
{
    sprintf(dout,"[sp, #$%02X*4]",TDASMOP_IMM8);
}

void ARMDasm::topbkpt(u32 op, u32 r15)
{
    sprintf(dout,"<unimpl>");
}

void ARMDasm::topswi(u32 op, u32 r15)
{
    sprintf(dout,"$%02X",op&0x00FF);
}

//---Tables----------------------------------------------------------------

const char *ARMDasm::condstr[]={
    "eq","ne","cs","cc",
    "mi","pl","vs","vc",
    "hi","ls","ge","lt",
    "gt","le","","nv",
};

const ARMDasm::DASMOP ARMDasm::ops[]={
    //0x00
    {"and%s %s", ARMDasm::oplli      },
    {"and%s %s", ARMDasm::opllr      },
    {"and%s %s", ARMDasm::oplri      },
    {"and%s %s", ARMDasm::oplrr      },
    {"and%s %s", ARMDasm::opari      },
    {"and%s %s", ARMDasm::oparr      },
    {"and%s %s", ARMDasm::oprri      },
    {"and%s %s", ARMDasm::oprrr      },
    {"and%s %s", ARMDasm::oplli      },
    {"mul%s %s", ARMDasm::opmul      },
    {"and%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::opptrm    },
    {"and%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::opptrm    },  // ++ARM5E
    {"and%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::opptrm    },  // ++ARM5E

    //0x01
    {"and%ss %s", ARMDasm::oplli     },
    {"and%ss %s", ARMDasm::opllr     },
    {"and%ss %s", ARMDasm::oplri     },
    {"and%ss %s", ARMDasm::oplrr     },
    {"and%ss %s", ARMDasm::opari     },
    {"and%ss %s", ARMDasm::oparr     },
    {"and%ss %s", ARMDasm::oprri     },
    {"and%ss %s", ARMDasm::oprrr     },
    {"and%ss %s", ARMDasm::oplli     },
    {"mul%ss %s", ARMDasm::opmul     },
    {"and%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::opptrm    },
    {"and%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::opptrm   },
    {"and%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::opptrm   },

    //0x02
    {"eor%s %s", ARMDasm::oplli      },
    {"eor%s %s", ARMDasm::opllr      },
    {"eor%s %s", ARMDasm::oplri      },
    {"eor%s %s", ARMDasm::oplrr      },
    {"eor%s %s", ARMDasm::opari      },
    {"eor%s %s", ARMDasm::oparr      },
    {"eor%s %s", ARMDasm::oprri      },
    {"eor%s %s", ARMDasm::oprrr      },
    {"eor%s %s", ARMDasm::oplli      },
    {"mla%s %s", ARMDasm::opmla      },
    {"eor%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::opptrm    },
    {"eor%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::opptrm    },  // ++ARM5E
    {"eor%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::opptrm    },  // ++ARM5E

    //0x03
    {"eor%ss %s", ARMDasm::oplli     },
    {"eor%ss %s", ARMDasm::opllr     },
    {"eor%ss %s", ARMDasm::oplri     },
    {"eor%ss %s", ARMDasm::oplrr     },
    {"eor%ss %s", ARMDasm::opari     },
    {"eor%ss %s", ARMDasm::oparr     },
    {"eor%ss %s", ARMDasm::oprri     },
    {"eor%ss %s", ARMDasm::oprrr     },
    {"eor%ss %s", ARMDasm::oplli     },
    {"mla%ss %s", ARMDasm::opmla     },
    {"eor%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::opptrm    },
    {"eor%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::opptrm   },
    {"eor%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::opptrm   },

    //0x04
    {"sub%s %s", ARMDasm::oplli      },
    {"sub%s %s", ARMDasm::opllr      },
    {"sub%s %s", ARMDasm::oplri      },
    {"sub%s %s", ARMDasm::oplrr      },
    {"sub%s %s", ARMDasm::opari      },
    {"sub%s %s", ARMDasm::oparr      },
    {"sub%s %s", ARMDasm::oprri      },
    {"sub%s %s", ARMDasm::oprrr      },
    {"sub%s %s", ARMDasm::oplli      },
    {"unp%s %s", ARMDasm::opun       },
    {"sub%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::oplmptim  },
    {"sub%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::oplmptim  },  // ++ARM5E
    {"sub%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::oplmptim  },  // ++ARM5E

    //0x05
    {"sub%ss %s", ARMDasm::oplli     },
    {"sub%ss %s", ARMDasm::opllr     },
    {"sub%ss %s", ARMDasm::oplri     },
    {"sub%ss %s", ARMDasm::oplrr     },
    {"sub%ss %s", ARMDasm::opari     },
    {"sub%ss %s", ARMDasm::oparr     },
    {"sub%ss %s", ARMDasm::oprri     },
    {"sub%ss %s", ARMDasm::oprrr     },
    {"sub%ss %s", ARMDasm::oplli     },
    {"unp%s %s", ARMDasm::opun       },
    {"sub%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::oplmptim  },
    {"sub%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::oplmptim },
    {"sub%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::oplmptim },

    //0x06
    {"rsb%s %s", ARMDasm::oplli      },
    {"rsb%s %s", ARMDasm::opllr      },
    {"rsb%s %s", ARMDasm::oplri      },
    {"rsb%s %s", ARMDasm::oplrr      },
    {"rsb%s %s", ARMDasm::opari      },
    {"rsb%s %s", ARMDasm::oparr      },
    {"rsb%s %s", ARMDasm::oprri      },
    {"rsb%s %s", ARMDasm::oprrr      },
    {"rsb%s %s", ARMDasm::oplli      },
    {"unp%s %s", ARMDasm::opun       },
    {"rsb%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::oplmptim  },
    {"rsb%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::oplmptim  },  // ++ARM5E
    {"rsb%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::oplmptim  },  // ++ARM5E

    //0x07
    {"rsb%ss %s", ARMDasm::oplli     },
    {"rsb%ss %s", ARMDasm::opllr     },
    {"rsb%ss %s", ARMDasm::oplri     },
    {"rsb%ss %s", ARMDasm::oplrr     },
    {"rsb%ss %s", ARMDasm::opari     },
    {"rsb%ss %s", ARMDasm::oparr     },
    {"rsb%ss %s", ARMDasm::oprri     },
    {"rsb%ss %s", ARMDasm::oprrr     },
    {"rsb%ss %s", ARMDasm::oplli     },
    {"unp%s %s", ARMDasm::opun       },
    {"rsb%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::oplmptim  },
    {"rsb%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::oplmptim },
    {"rsb%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::oplmptim },

    //0x08
    {"add%s %s", ARMDasm::oplli      },
    {"add%s %s", ARMDasm::opllr      },
    {"add%s %s", ARMDasm::oplri      },
    {"add%s %s", ARMDasm::oplrr      },
    {"add%s %s", ARMDasm::opari      },
    {"add%s %s", ARMDasm::oparr      },
    {"add%s %s", ARMDasm::oprri      },
    {"add%s %s", ARMDasm::oprrr      },
    {"add%s %s", ARMDasm::oplli      },
    {"umull%s %s", ARMDasm::opmull   },
    {"add%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::opptrp    },
    {"add%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::opptrp    },  // ++ARM5E
    {"add%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::opptrp    },  // ++ARM5E

    //0x09
    {"add%ss %s", ARMDasm::oplli     },
    {"add%ss %s", ARMDasm::opllr     },
    {"add%ss %s", ARMDasm::oplri     },
    {"add%ss %s", ARMDasm::oplrr     },
    {"add%ss %s", ARMDasm::opari     },
    {"add%ss %s", ARMDasm::oparr     },
    {"add%ss %s", ARMDasm::oprri     },
    {"add%ss %s", ARMDasm::oprrr     },
    {"add%ss %s", ARMDasm::oplli     },
    {"umull%ss %s", ARMDasm::opmull  },
    {"add%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::opptrp    },
    {"add%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::opptrp   },
    {"add%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::opptrp   },

    //0x0A
    {"adc%s %s", ARMDasm::oplli      },
    {"adc%s %s", ARMDasm::opllr      },
    {"adc%s %s", ARMDasm::oplri      },
    {"adc%s %s", ARMDasm::oplrr      },
    {"adc%s %s", ARMDasm::opari      },
    {"adc%s %s", ARMDasm::oparr      },
    {"adc%s %s", ARMDasm::oprri      },
    {"adc%s %s", ARMDasm::oprrr      },
    {"adc%s %s", ARMDasm::oplli      },
    {"umlal%s %s", ARMDasm::opmull   },
    {"adc%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::opptrp    },
    {"adc%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::opptrp    },  // ++ARM5E
    {"adc%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::opptrp    },  // ++ARM5E

    //0x0B
    {"adc%ss %s", ARMDasm::oplli     },
    {"adc%ss %s", ARMDasm::opllr     },
    {"adc%ss %s", ARMDasm::oplri     },
    {"adc%ss %s", ARMDasm::oplrr     },
    {"adc%ss %s", ARMDasm::opari     },
    {"adc%ss %s", ARMDasm::oparr     },
    {"adc%ss %s", ARMDasm::oprri     },
    {"adc%ss %s", ARMDasm::oprrr     },
    {"adc%ss %s", ARMDasm::oplli     },
    {"umlal%ss %s", ARMDasm::opmull  },
    {"adc%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::opptrm    },
    {"adc%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::opptrm   },
    {"adc%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::opptrm   },

    //0x0C
    {"sbc%s %s", ARMDasm::oplli      },
    {"sbc%s %s", ARMDasm::opllr      },
    {"sbc%s %s", ARMDasm::oplri      },
    {"sbc%s %s", ARMDasm::oplrr      },
    {"sbc%s %s", ARMDasm::opari      },
    {"sbc%s %s", ARMDasm::oparr      },
    {"sbc%s %s", ARMDasm::oprri      },
    {"sbc%s %s", ARMDasm::oprrr      },
    {"sbc%s %s", ARMDasm::oplli      },
    {"smull%s %s", ARMDasm::opmull   },
    {"sbc%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::oplmptip  },
    {"sbc%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::oplmptim  },  // ++ARM5E
    {"sbc%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::oplmptim  },  // ++ARM5E

    //0x0D
    {"sbc%ss %s", ARMDasm::oplli     },
    {"sbc%ss %s", ARMDasm::opllr     },
    {"sbc%ss %s", ARMDasm::oplri     },
    {"sbc%ss %s", ARMDasm::oplrr     },
    {"sbc%ss %s", ARMDasm::opari     },
    {"sbc%ss %s", ARMDasm::oparr     },
    {"sbc%ss %s", ARMDasm::oprri     },
    {"sbc%ss %s", ARMDasm::oprrr     },
    {"sbc%ss %s", ARMDasm::oplli     },
    {"smull%ss %s", ARMDasm::opmull  },
    {"sbc%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::oplmptip  },
    {"sbc%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::oplmptip },
    {"sbc%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::oplmptip },

    //0x0E
    {"rsc%s %s", ARMDasm::oplli      },
    {"rsc%s %s", ARMDasm::opllr      },
    {"rsc%s %s", ARMDasm::oplri      },
    {"rsc%s %s", ARMDasm::oplrr      },
    {"rsc%s %s", ARMDasm::opari      },
    {"rsc%s %s", ARMDasm::oparr      },
    {"rsc%s %s", ARMDasm::oprri      },
    {"rsc%s %s", ARMDasm::oprrr      },
    {"rsc%s %s", ARMDasm::oplli      },
    {"smlal%s %s", ARMDasm::opmull   },
    {"rsc%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::oplmptip  },
    {"rsc%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::oplmptip  },
    {"rsc%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::oplmptip  },

    //0x0F
    {"rsc%ss %s", ARMDasm::oplli     },
    {"rsc%ss %s", ARMDasm::opllr     },
    {"rsc%ss %s", ARMDasm::oplri     },
    {"rsc%ss %s", ARMDasm::oplrr     },
    {"rsc%ss %s", ARMDasm::opari     },
    {"rsc%ss %s", ARMDasm::oparr     },
    {"rsc%ss %s", ARMDasm::oprri     },
    {"rsc%ss %s", ARMDasm::oprrr     },
    {"rsc%ss %s", ARMDasm::oplli     },
    {"smlal%ss %s", ARMDasm::opmull  },
    {"rsc%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::oplmptip  },
    {"rsc%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::oplmptip },
    {"rsc%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::oplmptip },

    //0x10
    {"mrs%s %s", ARMDasm::opmrsrc    },
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"qadd%s %s", ARMDasm::opqadd    },  // ++ARM5E
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"smlabb%s %s", ARMDasm::opmla   },  // ++ARM5E
    {"swp%s %s", ARMDasm::opswp      },
    {"smlatb%s %s", ARMDasm::opmla   },  // ++ARM5E
    {"strh%s %s", ARMDasm::opofrm    },
    {"smlabt%s %s", ARMDasm::opmla   },  // ++ARM5E
    {"ldrd%s %s", ARMDasm::opofrm    },  // ++ARM5E
    {"smlatt%s %s", ARMDasm::opmla   },  // ++ARM5E
    {"ldrd%s %s", ARMDasm::opofrm    },  // ++ARM5E

    //0x11
    {"tst%ss %s", ARMDasm::opclli    },
    {"tst%ss %s", ARMDasm::opcllr    },
    {"tst%ss %s", ARMDasm::opclri    },
    {"tst%ss %s", ARMDasm::opclrr    },
    {"tst%ss %s", ARMDasm::opcari    },
    {"tst%ss %s", ARMDasm::opcarr    },
    {"tst%ss %s", ARMDasm::opcrri    },
    {"tst%ss %s", ARMDasm::opcrrr    },
    {"tst%ss %s", ARMDasm::opclli    },
    {"unp%s %s", ARMDasm::opun       },
    {"tst%ss %s", ARMDasm::opclri    },
    {"ldrh%s %s", ARMDasm::opofrm    },
    {"tst%ss %s", ARMDasm::opcari    },
    {"ldrsb%s %s", ARMDasm::opofrm   },
    {"tst%ss %s", ARMDasm::opcrri    },
    {"ldrsh%s %s", ARMDasm::opofrm   },

    //0x12
    {"msr%s %s", ARMDasm::opmsrrc    },
    {"bx%s %s", ARMDasm::opbreg      },
    {"unp%s %s", ARMDasm::opun       },
    {"blx%s %s", ARMDasm::opbreg     },
    {"unp%s %s", ARMDasm::opun       },
    {"qsub%s %s", ARMDasm::opqadd    },  // ++ARM5E
    {"unp%s %s", ARMDasm::opun       },
    {"bkpt%s %s", ARMDasm::opun      },  // ++ARM5
    {"smlawb%s %s", ARMDasm::opmla   },  // ++ARM5E
    {"unp%s %s", ARMDasm::opun       },
    {"smlawt%s %s", ARMDasm::opmla   },  // ++ARM5E
    {"strh%s %s", ARMDasm::opprrm    },
    {"smulwb%s %s", ARMDasm::opmul   },  // ++ARM5E
    {"ldrd%s %s", ARMDasm::opprrm    },  // ++ARM5E
    {"smulwt%s %s", ARMDasm::opmul   },  // ++ARM5E
    {"strd%s %s", ARMDasm::opprrm    },  // ++ARM5E

    //0x13
    {"teq%ss %s", ARMDasm::opclli    },
    {"teq%ss %s", ARMDasm::opcllr    },
    {"teq%ss %s", ARMDasm::opclri    },
    {"teq%ss %s", ARMDasm::opclrr    },
    {"teq%ss %s", ARMDasm::opcari    },
    {"teq%ss %s", ARMDasm::opcarr    },
    {"teq%ss %s", ARMDasm::opcrri    },
    {"teq%ss %s", ARMDasm::opcrrr    },
    {"teq%ss %s", ARMDasm::opclli    },
    {"unp%s %s", ARMDasm::opun       },
    {"teq%ss %s", ARMDasm::opclri    },
    {"ldrh%s %s", ARMDasm::opprrm    },
    {"teq%ss %s", ARMDasm::opcari    },
    {"ldrsb%s %s", ARMDasm::opprrm   },
    {"teq%ss %s", ARMDasm::opcrri    },
    {"ldrsh%s %s", ARMDasm::opprrm   },

    //0x14
    {"mrs%s %s", ARMDasm::opmrsrs    },
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"qdadd%s %s", ARMDasm::opqadd   },  // ++ARM5E
    {"unp%s %s", ARMDasm::opun       },
    {"unp%s %s", ARMDasm::opun       },
    {"smlalbb%s %s", ARMDasm::opmla  },  // ++ARM5E
    {"swpb%s %s", ARMDasm::opswp     },
    {"smlaltb%s %s", ARMDasm::opmla  },  // ++ARM5E
    {"strh%s %s", ARMDasm::oplmofim  },
    {"smlalbt%s %s", ARMDasm::opmla  },  // ++ARM5E
    {"ldrd%s %s", ARMDasm::oplmofim  },  // ++ARM5E
    {"smlaltt%s %s", ARMDasm::opmla  },  // ++ARM5E
    {"strd%s %s", ARMDasm::oplmofim  },  // ++ARM5E

    //0x15
    {"cmp%ss %s", ARMDasm::opclli    },
    {"cmp%ss %s", ARMDasm::opcllr    },
    {"cmp%ss %s", ARMDasm::opclri    },
    {"cmp%ss %s", ARMDasm::opclrr    },
    {"cmp%ss %s", ARMDasm::opcari    },
    {"cmp%ss %s", ARMDasm::opcarr    },
    {"cmp%ss %s", ARMDasm::opcrri    },
    {"cmp%ss %s", ARMDasm::opcrrr    },
    {"cmp%ss %s", ARMDasm::opclli    },
    {"unp%s %s", ARMDasm::opun       },
    {"cmp%ss %s", ARMDasm::opclri    },
    {"ldrh%s %s", ARMDasm::oplmofim  },
    {"cmp%ss %s", ARMDasm::opcari    },
    {"ldrsb%s %s", ARMDasm::oplmofim },
    {"cmp%ss %s", ARMDasm::opcrri    },
    {"ldrsh%s %s", ARMDasm::oplmofim },

    //0x16
    {"msr%s %s", ARMDasm::opmsrrs    },
    {"clz%s %s", ARMDasm::opclz      },  // ++ARM5
    {"unp%s %s", ARMDasm::opun       }, 
    {"unp%s %s", ARMDasm::opun       }, 
    {"unp%s %s", ARMDasm::opun       }, 
    {"qdsub%s %s", ARMDasm::opqadd   },  // ++ARM5E
    {"unp%s %s", ARMDasm::opun       }, 
    {"unp%s %s", ARMDasm::opun       }, 
    {"smulbb%s %s", ARMDasm::opmul   },  // ++ARM5E
    {"unp%s %s", ARMDasm::opun       }, 
    {"smultb%s %s", ARMDasm::opmul   },  // ++ARM5E
    {"strh%s %s", ARMDasm::oplmprim  },
    {"smulbt%s %s", ARMDasm::opmul   },  // ++ARM5E
    {"ldrd%s %s", ARMDasm::oplmprim  },  // ++ARM5E
    {"smultt%s %s", ARMDasm::opmul   },  // ++ARM5E
    {"strd%s %s", ARMDasm::oplmprim  },  // ++ARM5E

    //0x17
    {"cmn%ss %s", ARMDasm::opclli    },
    {"cmn%ss %s", ARMDasm::opcllr    },
    {"cmn%ss %s", ARMDasm::opclri    },
    {"cmn%ss %s", ARMDasm::opclrr    },
    {"cmn%ss %s", ARMDasm::opcari    },
    {"cmn%ss %s", ARMDasm::opcarr    },
    {"cmn%ss %s", ARMDasm::opcrri    },
    {"cmn%ss %s", ARMDasm::opcrrr    },
    {"cmn%ss %s", ARMDasm::opclli    },
    {"unp%s %s", ARMDasm::opun       },
    {"cmn%ss %s", ARMDasm::opclri    },
    {"ldrh%s %s", ARMDasm::oplmprim  },
    {"cmn%ss %s", ARMDasm::opcari    },
    {"ldrsb%s %s", ARMDasm::oplmprim },
    {"cmn%ss %s", ARMDasm::opcrri    },
    {"ldrsh%s %s", ARMDasm::oplmprim },

    //0x18
    {"orr%s %s", ARMDasm::oplli      },
    {"orr%s %s", ARMDasm::opllr      },
    {"orr%s %s", ARMDasm::oplri      },
    {"orr%s %s", ARMDasm::oplrr      },
    {"orr%s %s", ARMDasm::opari      },
    {"orr%s %s", ARMDasm::oparr      },
    {"orr%s %s", ARMDasm::oprri      },
    {"orr%s %s", ARMDasm::oprrr      },
    {"orr%s %s", ARMDasm::oplli      },
    {"unp%s %s", ARMDasm::opun       },
    {"orr%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::opofrp    },
    {"orr%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::opofrp    },  // ++ARM5E
    {"orr%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::opofrp    },  // ++ARM5E

    //0x19
    {"orr%ss %s", ARMDasm::oplli     },
    {"orr%ss %s", ARMDasm::opllr     },
    {"orr%ss %s", ARMDasm::oplri     },
    {"orr%ss %s", ARMDasm::oplrr     },
    {"orr%ss %s", ARMDasm::opari     },
    {"orr%ss %s", ARMDasm::oparr     },
    {"orr%ss %s", ARMDasm::oprri     },
    {"orr%ss %s", ARMDasm::oprrr     },
    {"orr%ss %s", ARMDasm::oplli     },
    {"unp%s %s", ARMDasm::opun       },
    {"orr%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::opofrp    },
    {"orr%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::opofrp   },
    {"orr%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::opofrp   },

    //0x1A
    {"mov%s %s", ARMDasm::oplli      },
    {"mov%s %s", ARMDasm::opllr      },
    {"mov%s %s", ARMDasm::oplri      },
    {"mov%s %s", ARMDasm::oplrr      },
    {"mov%s %s", ARMDasm::opari      },
    {"mov%s %s", ARMDasm::oparr      },
    {"mov%s %s", ARMDasm::oprri      },
    {"mov%s %s", ARMDasm::oprrr      },
    {"mov%s %s", ARMDasm::oplli      },
    {"unp%s %s", ARMDasm::opun       },
    {"mov%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::opprrp    },
    {"mov%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::opprrp    },  // ++ARM5E
    {"mov%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::opprrp    },  // ++ARM5E

    //0x1B
    {"mov%ss %s", ARMDasm::oplli     },
    {"mov%ss %s", ARMDasm::opllr     },
    {"mov%ss %s", ARMDasm::oplri     },
    {"mov%ss %s", ARMDasm::oplrr     },
    {"mov%ss %s", ARMDasm::opari     },
    {"mov%ss %s", ARMDasm::oparr     },
    {"mov%ss %s", ARMDasm::oprri     },
    {"mov%ss %s", ARMDasm::oprrr     },
    {"mov%ss %s", ARMDasm::oplli     },
    {"unp%s %s", ARMDasm::opun       },
    {"mov%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::opprrp    },
    {"mov%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::opprrp   },
    {"mov%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::opprrp   },

    //0x1C
    {"bic%s %s", ARMDasm::oplli      },
    {"bic%s %s", ARMDasm::opllr      },
    {"bic%s %s", ARMDasm::oplri      },
    {"bic%s %s", ARMDasm::oplrr      },
    {"bic%s %s", ARMDasm::opari      },
    {"bic%s %s", ARMDasm::oparr      },
    {"bic%s %s", ARMDasm::oprri      },
    {"bic%s %s", ARMDasm::oprrr      },
    {"bic%s %s", ARMDasm::oplli      },
    {"unp%s %s", ARMDasm::opun       },
    {"bic%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::oplmofip  },
    {"bic%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::oplmofip  },  // ++ARM5E
    {"bic%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::oplmofip  },  // ++ARM5E

    //0x1D
    {"bic%ss %s", ARMDasm::oplli     },
    {"bic%ss %s", ARMDasm::opllr     },
    {"bic%ss %s", ARMDasm::oplri     },
    {"bic%ss %s", ARMDasm::oplrr     },
    {"bic%ss %s", ARMDasm::opari     },
    {"bic%ss %s", ARMDasm::oparr     },
    {"bic%ss %s", ARMDasm::oprri     },
    {"bic%ss %s", ARMDasm::oprrr     },
    {"bic%ss %s", ARMDasm::oplli     },
    {"unp%s %s", ARMDasm::opun       },
    {"bic%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::oplmofip  },
    {"bic%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::oplmofip },
    {"bic%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::oplmofip },

    //0x1E
    {"mvn%s %s", ARMDasm::oplli      },
    {"mvn%s %s", ARMDasm::opllr      },
    {"mvn%s %s", ARMDasm::oplri      },
    {"mvn%s %s", ARMDasm::oplrr      },
    {"mvn%s %s", ARMDasm::opari      },
    {"mvn%s %s", ARMDasm::oparr      },
    {"mvn%s %s", ARMDasm::oprri      },
    {"mvn%s %s", ARMDasm::oprrr      },
    {"mvn%s %s", ARMDasm::oplli      },
    {"unp%s %s", ARMDasm::opun       },
    {"mvn%s %s", ARMDasm::oplri      },
    {"strh%s %s", ARMDasm::oplmprip  },
    {"mvn%s %s", ARMDasm::opari      },
    {"ldrd%s %s", ARMDasm::oplmprip  },  // ++ARM5E
    {"mvn%s %s", ARMDasm::oprri      },
    {"strd%s %s", ARMDasm::oplmprip  },  // ++ARM5E

    //0x1F
    {"mvn%ss %s", ARMDasm::oplli     },
    {"mvn%ss %s", ARMDasm::opllr     },
    {"mvn%ss %s", ARMDasm::oplri     },
    {"mvn%ss %s", ARMDasm::oplrr     },
    {"mvn%ss %s", ARMDasm::opari     },
    {"mvn%ss %s", ARMDasm::oparr     },
    {"mvn%ss %s", ARMDasm::oprri     },
    {"mvn%ss %s", ARMDasm::oprrr     },
    {"mvn%ss %s", ARMDasm::oplli     },
    {"unp%s %s", ARMDasm::opun       },
    {"mvn%ss %s", ARMDasm::oplri     },
    {"ldrh%s %s", ARMDasm::oplmprip  },
    {"mvn%ss %s", ARMDasm::opari     },
    {"ldrsb%s %s", ARMDasm::oplmprip },
    {"mvn%ss %s", ARMDasm::oprri     },
    {"ldrsh%s %s", ARMDasm::oplmprip },

    //0x20
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },
    {"and%s %s", ARMDasm::opimm      },

    //0x21
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },
    {"and%ss %s", ARMDasm::opimm     },

    //0x22
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },
    {"eor%s %s", ARMDasm::opimm      },

    //0x23
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },
    {"eor%ss %s", ARMDasm::opimm     },

    //0x24
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },
    {"sub%s %s", ARMDasm::opimm      },

    //0x25
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },
    {"sub%ss %s", ARMDasm::opimm     },

    //0x26
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },
    {"rsb%s %s", ARMDasm::opimm      },

    //0x27
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },
    {"rsb%ss %s", ARMDasm::opimm     },

    //0x28
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },
    {"add%s %s", ARMDasm::opimm      },

    //0x29
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },
    {"add%ss %s", ARMDasm::opimm     },

    //0x2A
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },
    {"adc%s %s", ARMDasm::opimm      },

    //0x2B
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },
    {"adc%ss %s", ARMDasm::opimm     },

    //0x2C
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },
    {"sbc%s %s", ARMDasm::opimm      },

    //0x2D
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },
    {"sbc%ss %s", ARMDasm::opimm     },

    //0x2E
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },
    {"rsc%s %s", ARMDasm::opimm      },

    //0x2F
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },
    {"rsc%ss %s", ARMDasm::opimm     },

    //0x30
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },

    //0x31
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },
    {"tst%ss %s", ARMDasm::opcimm     },

    //0x32
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },
    {"msr%s %s", ARMDasm::opmsric       },

    //0x33
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },
    {"teq%ss %s", ARMDasm::opcimm     },

    //0x34
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },
    {"und%s %s", ARMDasm::opun       },

    //0x35
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },
    {"cmp%ss %s", ARMDasm::opcimm     },

    //0x36
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },
    {"msr%s %s", ARMDasm::opmsris       },

    //0x37
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },
    {"cmn%ss %s", ARMDasm::opcimm     },

    //0x38
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },
    {"orr%s %s", ARMDasm::opimm      },

    //0x39
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },
    {"orr%ss %s", ARMDasm::opimm     },

    //0x3A
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },
    {"mov%s %s", ARMDasm::opimm      },

    //0x3B
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },
    {"mov%ss %s", ARMDasm::opimm     },

    //0x3C
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },
    {"bic%s %s", ARMDasm::opimm      },

    //0x3D
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },
    {"bic%ss %s", ARMDasm::opimm     },

    //0x3E
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },
    {"mvn%s %s", ARMDasm::opimm      },

    //0x3F
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },
    {"mvn%ss %s", ARMDasm::opimm     },

    //0x40
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },
    {"str%s %s", ARMDasm::opptim     },

    //0x41
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },
    {"ldr%s %s", ARMDasm::opptim     },

    //0x42
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },
    {"strt%s %s", ARMDasm::opptim    },

    //0x43
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },
    {"ldrt%s %s", ARMDasm::opptim    },

    //0x44
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },
    {"strb%s %s", ARMDasm::opptim    },

    //0x45
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },
    {"ldrb%s %s", ARMDasm::opptim    },

    //0x46
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },
    {"strbt%s %s", ARMDasm::opptim   },

    //0x47
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },
    {"ldrbt%s %s", ARMDasm::opptim   },

    //0x48
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },
    {"str%s %s", ARMDasm::opptip     },

    //0x49
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },
    {"ldr%s %s", ARMDasm::opptip     },

    //0x4A
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },
    {"strt%s %s", ARMDasm::opptip    },

    //0x4B
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },
    {"ldrt%s %s", ARMDasm::opptip    },

    //0x4C
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },
    {"strb%s %s", ARMDasm::opptip    },

    //0x4D
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },
    {"ldrb%s %s", ARMDasm::opptip    },

    //0x4E
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },
    {"strbt%s %s", ARMDasm::opptip   },

    //0x4F
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },
    {"ldrbt%s %s", ARMDasm::opptip   },

    //0x50
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },
    {"str%s %s", ARMDasm::opofim     },

    //0x51
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },
    {"ldr%s %s", ARMDasm::opofim     },

    //0x52
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },
    {"str%s %s", ARMDasm::opprim     },

    //0x53
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },
    {"ldr%s %s", ARMDasm::opprim     },

    //0x54
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },
    {"strb%s %s", ARMDasm::opofim    },

    //0x55
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },
    {"ldrb%s %s", ARMDasm::opofim    },

    //0x56
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },
    {"strb%s %s", ARMDasm::opprim    },

    //0x57
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },
    {"ldrb%s %s", ARMDasm::opprim    },

    //0x58
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },
    {"str%s %s", ARMDasm::opofip     },

    //0x59
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },
    {"ldr%s %s", ARMDasm::opofip     },

    //0x5A
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },
    {"str%s %s", ARMDasm::opprip     },

    //0x5B
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },
    {"ldr%s %s", ARMDasm::opprip     },

    //0x5C
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },
    {"strb%s %s", ARMDasm::opofip    },

    //0x5D
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },
    {"ldrb%s %s", ARMDasm::opofip    },

    //0x5E
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },
    {"strb%s %s", ARMDasm::opprip    },

    //0x5F
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },
    {"ldrb%s %s", ARMDasm::opprip    },

    //0x60
    {"str%s %s", ARMDasm::opptrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmrr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrmrr   },
    {"und%s %s", ARMDasm::opun       },

    //0x61
    {"ldr%s %s", ARMDasm::opptrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmrr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrmrr   },
    {"und%s %s", ARMDasm::opun       },

    //0x62
    {"strt%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x63
    {"ldrt%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x64
    {"strb%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x65
    {"ldrb%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x66
    {"strbt%s %s", ARMDasm::opptrmll },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmlr },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmar },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmrr },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmll },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmlr },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmar },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrmrr },
    {"und%s %s", ARMDasm::opun       },

    //0x67
    {"ldrbt%s %s", ARMDasm::opptrmll },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmlr },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmar },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmrr },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmll },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmlr },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmar },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrmrr },
    {"und%s %s", ARMDasm::opun       },

    //0x68
    {"str%s %s", ARMDasm::opptrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrprr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opptrprr   },
    {"und%s %s", ARMDasm::opun       },

    //0x69
    {"ldr%s %s", ARMDasm::opptrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrprr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opptrprr   },
    {"und%s %s", ARMDasm::opun       },

    //0x6A
    {"strt%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strt%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x6B
    {"ldrt%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrt%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x6C
    {"strb%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x6D
    {"ldrb%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opptrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x6E
    {"strbt%s %s", ARMDasm::opptrpll },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrplr },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrpar },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrprr },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrpll },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrplr },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrpar },
    {"und%s %s", ARMDasm::opun       },
    {"strbt%s %s", ARMDasm::opptrprr },
    {"und%s %s", ARMDasm::opun       },

    //0x6F
    {"ldrbt%s %s", ARMDasm::opptrpll },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrplr },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrpar },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrprr },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrpll },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrplr },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrpar },
    {"und%s %s", ARMDasm::opun       },
    {"ldrbt%s %s", ARMDasm::opptrprr },
    {"und%s %s", ARMDasm::opun       },

    //0x70
    {"str%s %s", ARMDasm::opofrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmrr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrmrr   },
    {"und%s %s", ARMDasm::opun       },

    //0x71
    {"ldr%s %s", ARMDasm::opofrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmrr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrmrr   },
    {"und%s %s", ARMDasm::opun       },

    //0x72
    {"str%s %s", ARMDasm::opprrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmrr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrmrr   },
    {"und%s %s", ARMDasm::opun       },

    //0x73
    {"ldr%s %s", ARMDasm::opprrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmrr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmlr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrmrr   },
    {"und%s %s", ARMDasm::opun       },

    //0x74
    {"strb%s %s", ARMDasm::opofrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x75
    {"ldrb%s %s", ARMDasm::opofrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x76
    {"strb%s %s", ARMDasm::opprrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x77
    {"ldrb%s %s", ARMDasm::opprrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmrr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmlr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrmrr  },
    {"und%s %s", ARMDasm::opun       },

    //0x78
    {"str%s %s", ARMDasm::opofrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrprr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opofrprr   },
    {"und%s %s", ARMDasm::opun       },

    //0x79
    {"ldr%s %s", ARMDasm::opofrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrprr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opofrprr   },
    {"und%s %s", ARMDasm::opun       },

    //0x7A
    {"str%s %s", ARMDasm::opprrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrprr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"str%s %s", ARMDasm::opprrprr   },
    {"und%s %s", ARMDasm::opun       },

    //0x7B
    {"ldr%s %s", ARMDasm::opprrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrprr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrpll   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrplr   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrpar   },
    {"und%s %s", ARMDasm::opun       },
    {"ldr%s %s", ARMDasm::opprrprr   },
    {"und%s %s", ARMDasm::opun       },

    //0x7C
    {"strb%s %s", ARMDasm::opofrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opofrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x7D
    {"ldrb%s %s", ARMDasm::opofrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opofrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x7E
    {"strb%s %s", ARMDasm::opprrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"strb%s %s", ARMDasm::opprrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x7F
    {"ldrb%s %s", ARMDasm::opprrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrprr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrpll  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrplr  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrpar  },
    {"und%s %s", ARMDasm::opun       },
    {"ldrb%s %s", ARMDasm::opprrprr  },
    {"und%s %s", ARMDasm::opun       },

    //0x80
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },
    {"stmda%s %s", ARMDasm::oplm       },

    //0x81
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },
    {"ldmda%s %s", ARMDasm::oplm       },

    //0x82
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },

    //0x83
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },

    //0x84
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },
    {"stmda%s %s", ARMDasm::oplm      },

    //0x85
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },
    {"ldmda%s %s", ARMDasm::oplm      },

    //0x86
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },
    {"stmda%s %s", ARMDasm::oplm     },

    //0x87
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },
    {"ldmda%s %s", ARMDasm::oplm     },

    //0x88
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },
    {"stmia%s %s", ARMDasm::oplm       },

    //0x89
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },
    {"ldmia%s %s", ARMDasm::oplm       },

    //0x8A
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },

    //0x8B
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },

    //0x8C
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },
    {"stmia%s %s", ARMDasm::oplm      },

    //0x8D
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },
    {"ldmia%s %s", ARMDasm::oplm      },

    //0x8E
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },
    {"stmia%s %s", ARMDasm::oplm     },

    //0x8F
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },
    {"ldmia%s %s", ARMDasm::oplm     },

    //0x90
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },
    {"stmdb%s %s", ARMDasm::oplm       },

    //0x91
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },
    {"ldmdb%s %s", ARMDasm::oplm       },

    //0x92
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },

    //0x93
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },

    //0x94
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },
    {"stmdb%s %s", ARMDasm::oplm      },

    //0x95
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },
    {"ldmdb%s %s", ARMDasm::oplm      },

    //0x96
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },
    {"stmdb%s %s", ARMDasm::oplm     },

    //0x97
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },
    {"ldmdb%s %s", ARMDasm::oplm     },

    //0x98
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },
    {"stmib%s %s", ARMDasm::oplm       },

    //0x99
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },
    {"ldmib%s %s", ARMDasm::oplm       },

    //0x9A
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },

    //0x9B
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },

    //0x9C
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },
    {"stmib%s %s", ARMDasm::oplm      },

    //0x9D
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },
    {"ldmib%s %s", ARMDasm::oplm      },

    //0x9E
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },
    {"stmib%s %s", ARMDasm::oplm     },

    //0x9F
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },
    {"ldmib%s %s", ARMDasm::oplm     },

    //0xA0
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA1
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA2
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA3
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA4
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA5
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA6
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA7
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA8
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xA9
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xAA
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xAB
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xAC
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xAD
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xAE
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xAF
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },
    {"b%s %s", ARMDasm::opb           },

    //0xB0
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB1
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB2
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB3
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB4
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB5
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB6
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB7
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB8
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xB9
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xBA
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xBB
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xBC
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xBD
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xBE
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xBF
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },
    {"bl%s %s", ARMDasm::opb          },

    //0xC0 // Coprocessors undefined here!
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC1
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC2
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC3
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC4
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC5
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC6
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC7
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC8
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xC9
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xCA
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xCB
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xCC
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xCD
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xCE
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xCF
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD0
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD1
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD2
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD3
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD4
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD5
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD6
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD7
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD8
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xD9
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xDA
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xDB
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xDC
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xDD
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xDE
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xDF
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },
    {"uni%s %s", ARMDasm::opun  },

    //0xE0
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xE1
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xE2
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xE3
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xE4
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xE5
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xE6
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xE7
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xE8
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xE9
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xEA
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xEB
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xEC
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xED
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xEE
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mcr%s %s", ARMDasm::opmcr         },

    //0xEF
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },
    {"cdp%s %s", ARMDasm::opcpd         },
    {"mrc%s %s", ARMDasm::opmcr         },

    //0xF0
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF1
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF2
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF3
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF4
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF5
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF6
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF7
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF8
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xF9
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xFA
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xFB
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xFC
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xFD
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xFE
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },

    //0xFF
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         },
    {"swi%s %s", ARMDasm::opswi         }

};

const ARMDasm::DASMOP ARMDasm::tops[256]={
    //0x00
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSL %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},
    {"LSR %s",   ARMDasm::topimm5shft},

    //0x10
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ASR %s",   ARMDasm::topimm5shft},
    {"ADD %s",   ARMDasm::topreg     },
    {"ADD %s",   ARMDasm::topreg     },
    {"SUB %s",   ARMDasm::topreg     },
    {"SUB %s",   ARMDasm::topreg     },
    {"ADD %s",   ARMDasm::topimm3    },
    {"ADD %s",   ARMDasm::topimm3    },
    {"SUB %s",   ARMDasm::topimm3    },
    {"SUB %s",   ARMDasm::topimm3    },

    //0x20
    {"MOV r0, %s",ARMDasm::topimm8    },
    {"MOV r1, %s",ARMDasm::topimm8    },
    {"MOV r2, %s",ARMDasm::topimm8    },
    {"MOV r3, %s",ARMDasm::topimm8    },
    {"MOV r4, %s",ARMDasm::topimm8    },
    {"MOV r5, %s",ARMDasm::topimm8    },
    {"MOV r6, %s",ARMDasm::topimm8    },
    {"MOV r7, %s",ARMDasm::topimm8    },
    {"CMP r0, %s",ARMDasm::topimm8    },
    {"CMP r1, %s",ARMDasm::topimm8    },
    {"CMP r2, %s",ARMDasm::topimm8    },
    {"CMP r3, %s",ARMDasm::topimm8    },
    {"CMP r4, %s",ARMDasm::topimm8    },
    {"CMP r5, %s",ARMDasm::topimm8    },
    {"CMP r6, %s",ARMDasm::topimm8    },
    {"CMP r7, %s",ARMDasm::topimm8    },

    //0x30
    {"ADD r0, %s",ARMDasm::topimm8    },
    {"ADD r1, %s",ARMDasm::topimm8    },
    {"ADD r2, %s",ARMDasm::topimm8    },
    {"ADD r3, %s",ARMDasm::topimm8    },
    {"ADD r4, %s",ARMDasm::topimm8    },
    {"ADD r5, %s",ARMDasm::topimm8    },
    {"ADD r6, %s",ARMDasm::topimm8    },
    {"ADD r7, %s",ARMDasm::topimm8    },
    {"SUB r0, %s",ARMDasm::topimm8    },
    {"SUB r1, %s",ARMDasm::topimm8    },
    {"SUB r2, %s",ARMDasm::topimm8    },
    {"SUB r3, %s",ARMDasm::topimm8    },
    {"SUB r4, %s",ARMDasm::topimm8    },
    {"SUB r5, %s",ARMDasm::topimm8    },
    {"SUB r6, %s",ARMDasm::topimm8    },
    {"SUB r7, %s",ARMDasm::topimm8    },

    //0x40
    {"%s",       ARMDasm::topdp1     },
    {"%s",       ARMDasm::topdp2     },
    {"%s",       ARMDasm::topdp3     },
    {"%s",       ARMDasm::topdp4     },
    {"ADD %s",   ARMDasm::toph       },
    {"CMP %s",   ARMDasm::toph       },
    {"MOV %s",   ARMDasm::toph       },
    {"%s",       ARMDasm::topbx      },
    {"LDR r0, %s",ARMDasm::toppc      },
    {"LDR r1, %s",ARMDasm::toppc      },
    {"LDR r2, %s",ARMDasm::toppc      },
    {"LDR r3, %s",ARMDasm::toppc      },
    {"LDR r4, %s",ARMDasm::toppc      },
    {"LDR r5, %s",ARMDasm::toppc      },
    {"LDR r6, %s",ARMDasm::toppc      },
    {"LDR r7, %s",ARMDasm::toppc      },

    //0x50
    {"STR %s",   ARMDasm::topreg     },
    {"STR %s",   ARMDasm::topreg     },
    {"STRH %s",  ARMDasm::topreg     },
    {"STRH %s",  ARMDasm::topreg     },
    {"STRB %s",  ARMDasm::topreg     },
    {"STRB %s",  ARMDasm::topreg     },
    {"LDRSB %s", ARMDasm::topreg     },
    {"LDRSB %s", ARMDasm::topreg     },
    {"LDR %s",   ARMDasm::topreg     },
    {"LDR %s",   ARMDasm::topreg     },
    {"LDRH %s",  ARMDasm::topreg     },
    {"LDRH %s",  ARMDasm::topreg     },
    {"LDRB %s",  ARMDasm::topreg     },
    {"LDRB %s",  ARMDasm::topreg     },
    {"LDRSH %s", ARMDasm::topreg     },
    {"LDRSH %s", ARMDasm::topreg     },

    //0x60
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"STR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },
    {"LDR %s*4]",ARMDasm::topimm5    },

    //0x70
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"STRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },
    {"LDRB %s]", ARMDasm::topimm5    },

    //0x80
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"STRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },
    {"LDRH %s*2]",ARMDasm::topimm5   },

    //0x90
    {"STR r0, %s",ARMDasm::topsp      },
    {"STR r1, %s",ARMDasm::topsp      },
    {"STR r2, %s",ARMDasm::topsp      },
    {"STR r3, %s",ARMDasm::topsp      },
    {"STR r4, %s",ARMDasm::topsp      },
    {"STR r5, %s",ARMDasm::topsp      },
    {"STR r6, %s",ARMDasm::topsp      },
    {"STR r7, %s",ARMDasm::topsp      },
    {"LDR r0, %s",ARMDasm::topsp      },
    {"LDR r1, %s",ARMDasm::topsp      },
    {"LDR r2, %s",ARMDasm::topsp      },
    {"LDR r3, %s",ARMDasm::topsp      },
    {"LDR r4, %s",ARMDasm::topsp      },
    {"LDR r5, %s",ARMDasm::topsp      },
    {"LDR r6, %s",ARMDasm::topsp      },
    {"LDR r7, %s",ARMDasm::topsp      },

    //0xA0
    {"ADD r0, %s",ARMDasm::toppc      },
    {"ADD r1, %s",ARMDasm::toppc      },
    {"ADD r2, %s",ARMDasm::toppc      },
    {"ADD r3, %s",ARMDasm::toppc      },
    {"ADD r4, %s",ARMDasm::toppc      },
    {"ADD r5, %s",ARMDasm::toppc      },
    {"ADD r6, %s",ARMDasm::toppc      },
    {"ADD r7, %s",ARMDasm::toppc      },
    {"ADD r0, %s",ARMDasm::topsp      },
    {"ADD r1, %s",ARMDasm::topsp      },
    {"ADD r2, %s",ARMDasm::topsp      },
    {"ADD r3, %s",ARMDasm::topsp      },
    {"ADD r4, %s",ARMDasm::topsp      },
    {"ADD r5, %s",ARMDasm::topsp      },
    {"ADD r6, %s",ARMDasm::topsp      },
    {"ADD r7, %s",ARMDasm::topsp      },

    //0xB0
    {"%s",       ARMDasm::topimm7    },
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"PUSH %s",  ARMDasm::topldm     },
    {"PUSH %s, LR",ARMDasm::topldm    },
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated
    {"POP %s",   ARMDasm::topldm     },
    {"POP %s, PC",ARMDasm::topldm     },
    {"BKPT %s",  ARMDasm::topbkpt    },  // ++ARM5
    {"UND %s",   ARMDasm::topund     },  // Misc map: unallocated

    //0xC0
    {"STMIA r0!, %s", ARMDasm::topldm },
    {"STMIA r1!, %s", ARMDasm::topldm },
    {"STMIA r2!, %s", ARMDasm::topldm },
    {"STMIA r3!, %s", ARMDasm::topldm },
    {"STMIA r4!, %s", ARMDasm::topldm },
    {"STMIA r5!, %s", ARMDasm::topldm },
    {"STMIA r6!, %s", ARMDasm::topldm },
    {"STMIA r7!, %s", ARMDasm::topldm },
    {"LDMIA r0!, %s", ARMDasm::topldm },
    {"LDMIA r1!, %s", ARMDasm::topldm },
    {"LDMIA r2!, %s", ARMDasm::topldm },
    {"LDMIA r3!, %s", ARMDasm::topldm },
    {"LDMIA r4!, %s", ARMDasm::topldm },
    {"LDMIA r5!, %s", ARMDasm::topldm },
    {"LDMIA r6!, %s", ARMDasm::topldm },
    {"LDMIA r7!, %s", ARMDasm::topldm },

    //0xD0
    {"BEQ %s",   ARMDasm::topbc      },
    {"BNE %s",   ARMDasm::topbc      },
    {"BCS %s",   ARMDasm::topbc      },
    {"BCC %s",   ARMDasm::topbc      },
    {"BMI %s",   ARMDasm::topbc      },
    {"BPL %s",   ARMDasm::topbc      },
    {"BVS %s",   ARMDasm::topbc      },
    {"BVC %s",   ARMDasm::topbc      },
    {"BHI %s",   ARMDasm::topbc      },
    {"BLS %s",   ARMDasm::topbc      },
    {"BGE %s",   ARMDasm::topbc      },
    {"BLT %s",   ARMDasm::topbc      },
    {"BGT %s",   ARMDasm::topbc      },
    {"BLE %s",   ARMDasm::topbc      },
    {"UND %s",   ARMDasm::topund     },  // Covered by unconditional
    {"SWI %s",   ARMDasm::topswi     },

    //0xE0
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"B %s",     ARMDasm::topb       },
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5
    {"BLX %s",   ARMDasm::topbl2     },  // ++ARM5

    //0xF0
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"%s",   ARMDasm::topbl1     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
    {"BL %s",   ARMDasm::topbl2     },
};

/*** EOF: armdasm.cpp ****************************************************/

